PROGRAM shellklara
!	is designed to calculate lightcurves, spectra and images of 
!	interacting binaries immersed in a moving circumstellar 
!	environment which is optically thin. It solves simple radiative 
!	transfer along the line of sight in moving media. The scattered 
!	light from the two stars can be taken into account.
!	Output intensities are then integrated through 
!	the 2D projection surface of a 3D object. 
!	The assumptions include LTE and optional known state 
!	quantities and velocity fields in 3D. 
!	Optional (non)transparent objects such as a spot, disc, stream, 
!       jet, shell or stars as well as an empty space may be defined 
!	(embedded) in 3D and their composite synthetic spectrum 
!       calculated. The stars may have the Roche or spherical geometry, 
!       optional velocity or rotation, are subject
!       to the gravity darkening, limb darkening, and irradiation effect
!	including the heating, reflection and day-night heat 
!       redistribution. They may be ascribed a precalculated spectrum.
!	Synthetic light curves or trailing spectrograms can be produced
!	by changing your view points on the 3D object.
!	Opacities: lines, HI bound-free, HI free-free, 
!		Thomson scattering, Rayleigh scattering. 
!	Emissivities: thermal, Thomson and Rayleigh scattering from 
!		the stars.
!
!       INPUT:
!		shellspec.in -(9) main input (geometry, objects...)
!		line.dat - (8) atomic data for the lines
!               shellspec.mod - (10) input 3D model of the shell
!			(optional if imodel=2)
!		abundances  - (7) abundances 
!			(optional if ichemc=1 or ielnd=1)
!		phases - (15) orbital phases to calculate
!			(optional if nphase=0)
!		starspec1 - (12) spectrum of the primary star
!			(optional if lunt1>0)
!		starspec2 - (13) spectrum of the secondary
!			(optional if lunt2>0)
!		starspec3 - (14) spectrum of the third body 
!		(optional if lunt3>0)
!		albedo1 - (12) monochromatic albedo of the primary star
!			(optional if albst<0)
!		albedo2 - (13) monochromatic albedo of the secondary
!			(optional if albcp<0)
!
!       OUTPUT:
!		shellspec.out - (2) more detailed output
!		fort.xx - (21,21+iang) 2D images at some frequency
!		shellspectrum - (4) spectrum of the shell
!		lightcurve - (11) light curve or trailed spectrogram
!
!	COMMENT: CGS units are used if not specified otherwise
!		 Cartesian 3D coordinates are used
!		 implicit double precision is used and if possible
!		 use e.g. 1.d40 and not 1.e40
!		 ?? - BE CAUTIOUS
!
!	Non original routines: 
!	 	pfdwor  -from UCLSYN	
!		voigt, state0, gaunt, gfree   -from SYNSPEC
!		hunt,locate    -from Numerical recipes
!
!	Author/contact: Jan Budaj, Mercedes Richards
!			http://www.ta3.sk/~budaj
!			budaj@ta3.sk
!
!	Reference:
!		Budaj J., Richards M.T., 2004, 
!		Contrib. Astron. Obs. Skalnate Pleso, 34, 167
!
!	Things to be improved - added:
!	-take the rotation of star1 into account in the scattered 
!	light (interpolate in precalculated rotationally broadened  
!	intensity for several inclinations),
!	-introduce sets with continuum opacity at the edges
!	-allow finner grid along the line of sight, step op.dep.<0.05
!       -irradiation (heating) of the companion in case icomp=1
!	(reflection is already included)
!	-Mie scattering on dust
!	-gravity reddening of the primary star
!	-add into output into "lightcurve" column5 =lambda
!	-add H- opacity
!
!               ndim - main space xyz dimension parameters
!               mion - max. number of ions dimension (fixed=9)
!		matom - highest possible atom number 
!		mfreq - max. number of frequnces
!		mline - max. number of sp. lines considered
!		mphase - max. number of grid rotations
!		mstarx - max. number of frequences of precalculated
!			star1, star2, star3 spectra
!		mspecx - max. number of precalculated spectra/files
!			for star1 for mspecx different temperatures
!
USE param
USE unitchange
USE constants
USE sun
USE pfdwor
USE initsh

 IMPLICIT NONE

 INTEGER::iline,iang,&
	i,iprint,ii,iunt,ivac1,ivacn
 INTEGER::nalb2,nalb1,nspec1,nspec2,&
	nline,nivac,nfreq,ll,kk,k,jstar2,jstar1,jj,j,nu,nstar3,nbod
 INTEGER::n,jlo,ipsc,ndat,nnew,nalb,ialb  
 INTEGER,DIMENSION(mline)::nion,iat,iion
 INTEGER,DIMENSION(mspecx)::nstar1,nstar2
 INTEGER,DIMENSION(matom)::necod
  INTEGER,PARAMETER::miion=9
 REAL(KIND=8),DIMENSION(ndim1) :: ar
 REAL(KIND=8),DIMENSION(ndim1) :: far
 REAL(KIND=8),DIMENSION(ndim2) :: at,fat
 REAL(KIND=8),DIMENSION(ndim3) ::az,azn,faz
 REAL(KIND=8),DIMENSION(ndim1,ndim2,ndim3) :: atemp,apress,adens,ane,avr,avt,avz,&
		 avtrb,ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb
 REAL(KIND=8),DIMENSION(ndim1,ndim2,mfreq) :: ainte    
 REAL(KIND=8),DIMENSION(ndim3) :: vr,vt,vz,temp,dens,ekonc,vtrb,ophbf,ophbf1,ophbf2,&
    ophff,ophff1,ophff2,ophrs,ophrs1,ophrs2
 REAL(KIND=8),DIMENSION(ndim3) ::akonc
 REAL(KIND=8),DIMENSION(mfreq) ::alam
 REAL(KIND=8),DIMENSION(mfreq,mphase)::flux
 REAL(KIND=8),DIMENSION(ndim1,ndim2)::area
 REAL(KIND=8),DIMENSION(6,mion)::sta
 REAL(KIND=8),DIMENSION(3,matom)::d
 REAL(KIND=8),DIMENSION(8,matom)::xi
 REAL(KIND=8),DIMENSION(mphase)::alpha
 REAL(KIND=8),DIMENSION(ndim3,mion)::rr,stavs
 REAL(KIND=8),DIMENSION(ndim3)::hi,hneg,hei
 REAL(KIND=8),DIMENSION(ndim3,2)::hipf
 REAL(KIND=8),DIMENSION(mstarx,mspecx) :: xstar1,star1
 REAL(KIND=8),DIMENSION(mspecx)::tspec1,tspec2
 REAL(KIND=8),DIMENSION(mstarx)::wstar1,fstar1,wstar2,fstar2,alb1x,alb1y,alb2x,alb2y
 REAL(KIND=8),DIMENSION(mstarx,mspecx)::xstar2,star2,xstar3,star3
 REAL(KIND=8),DIMENSION(mline)::wlab,elo,eup,glo,bij,&
     gr0,gs0,gw0,gr,gs,gw
 REAL(KIND=8),DIMENSION(ndim3,mline)::aa,dop,popul
 REAL(KIND=8),DIMENSION(mion-1,mline)::zip
 REAL(KIND=8),DIMENSION(mion-1)::zipp
 REAL(KIND=8),DIMENSION(mline)::abund,hmc
 REAL(KIND=8)::aint0,aintb,abhyd,abhel
 REAL(KIND=8)::alphas,cdelta,cont,darkl,fluxl,dvel,der,&
			 freq,fluxs,fluxn,&
			 refw,ratio,&
			 plocha,phase,&
			vdif,vzcpr,&
			vystr,vycpr,vxstr,&
			vxcpr,vzsrt,vzstr,&
			ycpr,xcpr,zcpr,&
			bolt
 REAL(KIND=8)::ari,ati,&
	dl0,ep,am,chi,a,&
	eplow,&
	te,ek,zp,s1,s2,&
	xx,x,&
	theta,&
	xpp,ypp,zpp,y,z,ctheta,stheta,calpha,saplha ,&
	rosu,rotem,areas,teff,rfront,rback,rpole,rmean,xcomp,ff,&
	q,potx,poty,potz,pot,&
	entot,endf,endfp,&
	trmt,&
	xnew,ynew,&
	albx,alby
 REAL(KIND=8)::wm
 REAL(KIND=8),DIMENSION(matom)::ad,axi
 CHARACTER(LEN=4),DIMENSION(matom):: dyp,adyp

!-----------------------------------------------------------------------
        WRITE(6,'(A)')
        WRITE(6,'(A)')'                       Shellspec'
        WRITE(6,'(A)')
        WRITE(6,'(A)')'       INPUT: '
        WRITE(6,'(A)')' shellspec.in - main input (geometry,objects...)'
        WRITE(6,'(A)')' line.dat - atomic data for the lines'
        WRITE(6,'(A)')' shellspec.mod - 3D model of the shell '
        WRITE(6,'(A)')'                          (optional IF imodel=2)'
        WRITE(6,'(A)')' abundances  - abundances (optional IF ichemc=1)'
        WRITE(6,'(A)')' phases - orbital phases (optional IF nphase=0)'  
        WRITE(6,'(A)')' starspec1 - star spectrum (optional IF lunt1>0)'
        WRITE(6,'(A)')' starspec2 - star spectrum (optional IF lunt2>0)'
        WRITE(6,'(A)')' starspec3 - star spectrum (optional IF lunt3>0)'
        WRITE(6,'(A)')' albedo1 - albedo (optional IF albst<0)'
        WRITE(6,'(A)')' albedo2 - albedo (optional IF albcs<0)'        
        WRITE(6,'(A)')'       OUTPUT: '
        WRITE(6,'(A)')' shellspec.out - more detail output'
        WRITE(6,'(A)')' fort.xx - 2D images at some frequency'
        WRITE(6,'(A)')' shellspectrum - spectrum of the shell'
        WRITE(6,'(A)')' lightcurve - LC or trailed spectrogram'
	WRITE(6,*)
	OPEN(2,FILE='shellspec.out',STATUS='unknown')
	OPEN(4,FILE='shellspectrum',STATUS='unknown')
	OPEN(11,FILE='lightcurve',STATUS='unknown')
	OPEN(35,FILE='TEPLOTA.DAT',STATUS='unknown')
	OPEN(36,FILE='HUSTOTA.DAT',STATUS='unknown')
	
	call shell_init !call subroutine in initsh.f90 module - definition of parameters

!	constants and units from NIST(2002)

!	only ipart=1 is suppoted at the moment
	ipart=1
	nfreq=nint((alamn-alam1)/alams)+1
	nbodf1=2*nbodf1+1
	nbodf2=2*nbodf2+1
	nbodf3=2*nbodf3+1
	nbod1=2*nbod1+1
	nbod2=2*nbod2+1
	nbod3=2*nbod3+1
	IF(nbodf1>ndim1.or.nbodf2>ndim2.or.nbodf3>ndim3.or.&
     	   nbod1>ndim1.or.nbod2>ndim2.or.nbod3>ndim3.or.&
     	   nfreq>mfreq)THEN
	  WRITE(*,*)' error: space/frequency dimension exceeded, STOP'
	  STOP 
	ENDIF
	DO i=1,nfreq
	  alam(i)=alam1+dble(i-1)*alams
	ENDDO
	IF(nphase==0)THEN
	  OPEN(15,FILE='phases',STATUS='old')
	  i=1
	 DO
	  READ(15,*,err=17,end=17)alpha(i)
	  alpha(i)=(alpha(i)-0.25d0)*2.d0*pi
	  i=i+1
	 ENDDO
17	  nphase=i-1
	  CLOSE(15)
	ELSE IF(nphase==1)THEN
	  alpha(1)=phase1/180.d0*pi
	ELSE IF(nphase>1)THEN
	  alphas=(phasen-phase1)/dble(nphase-1)
	  DO  i=1,nphase
	    alpha(i)=(phase1+dble(i-1)*alphas)/180.d0*pi
          ENDDO
	ENDIF	
	WRITE(*,*)' nphase=',nphase
	emstar=emstar*emsol
	rstar=rstar*rsol
	rcp=rcp*rsol
        xcp=xcp*rsol
        ycp=ycp*rsol
        zcp=zcp*rsol
	rsp=rsp*rsol
        xsp=xsp*rsol
        ysp=ysp*rsol
        zsp=zsp*rsol
	rinsh=rinsh*rsol
	routsh=routsh*rsol
	rcsh=rcsh*rsol
	rindc=rindc*rsol
	routdc=routdc*rsol
	rinjt=rinjt*rsol
	routjt=routjt*rsol
	x1sm=x1sm*rsol
	y1sm=y1sm*rsol
	z1sm=z1sm*rsol
	x2sm=x2sm*rsol
	y2sm=y2sm*rsol
	z2sm=z2sm*rsol
	rsm=rsm*rsol
	rmodfx=rmodfx*rsol
	rmodfy=rmodfy*rsol
	rmodfz=rmodfz*rsol
	rmodx=rmodx*rsol
	rmody=rmody*rsol
	rmodz=rmodz*rsol
!
	vrotst=1.d5*vrotst
	vxst=1.d5*vxst
        vyst=1.d5*vyst
        vzst=1.d5*vzst
	vxcp=1.d5*vxcp
        vycp=1.d5*vycp
        vzcp=1.d5*vzcp
        vrotcp=1.d5*vrotcp
	vxsp=1.d5*vxsp
        vysp=1.d5*vysp
        vzsp=1.d5*vzsp
        vrotsp=1.d5*vrotsp
	vtrbsp=1.d5*vtrbsp
	vxdc=1.d5*vxdc
        vydc=1.d5*vydc
        vzdc=1.d5*vzdc
	vtrbdc=1.d5*vtrbdc
	vjt=1.d5*vjt
	vtrbjt=1.d5*vtrbjt
	vsh=1.d5*vsh
	vtrbsh=1.d5*vtrbsh
	v0=1.d5*v0
	v1sm=1.d5*v1sm
	v2sm=1.d5*v2sm
	vtrbsm=1.d5*vtrbsm
	dinc=dinc/180.d0*pi
	IF(idisc==1)THEN
	  adisc=adisc/180.d0*pi
	ELSE
 	  adisc=adisc*rsol
	ENDIF
	ajet=ajet/180.d0*pi
	dd=dd*pc
!	porb=porb*8.64d4
	CLOSE(9)
        IF(istar>1.or.icomp>1)THEN
          IF(xcp<=0.d0.or.qq<=0.d0.or.emstar<=0.d0)THEN
            WRITE(*,*)' error: inconsistent star/companion input data'
            STOP 
          ENDIF
        ENDIF
	IF(densdc>dcut1.and.edendc/=0.d0)THEN
          WRITE(*,*)' error: wrong densdc or edendc, STOP'
          STOP
	ENDIF
	IF(ishell>1.and.denssh>dcut1)THEN
          WRITE(*,*)' error: wrong ishell or denssh, STOP'
          STOP
	ENDIF
	IF((rstar<=0.d0.or.tstar<=0.d0)&
     	.and.(istar>0.or.icomp==2))THEN
          WRITE(*,*)' error: rstar-tstar, STOP'
          STOP
	ENDIF
	IF((rcp<=0.d0.or.tempcp<=0.d0)&
     	.and.(istar>0.or.icomp>0))THEN
          WRITE(*,*)' error: rcp-tempcp, STOP'
          STOP
	ENDIF	
	IF(rindc<=rstar.and.itdc==2.and.idisc>0)THEN
          WRITE(*,*)' error: rstar, rindc, itdc, STOP (T-singularity)'
          STOP
	ENDIF
!	reads monochromatic albedos	
	nalb1=0
	nalb2=0
	IF(albst<0.d0)THEN
	  albst=-albst
	  CALL albedo(alb1x,alb1y,nalb1,1)
	  WRITE(*,*)' albedo1',nalb1
	ENDIF
	IF(albcp<0.d0)THEN
	  albcp=-albcp
	  CALL albedo(alb2x,alb2y,nalb2,2)
	  WRITE(*,*)' albedo2',nalb2	  
	ENDIF	
	IF(albcp<0.d0.or.albcp>1.d0.or.&
     	htcp<0.d0.or.htcp>1.d0.or.&
      htcpa<0.d0.or.htcpa>1.d0.or.&
      albst<0.d0.or.albst>1.d0.or.&
     	htst<0.d0.or.htst>1.d0.or.&
      htsta<0.d0.or.htsta>1.d0)THEN
          WRITE(*,*)' error: alb**, ht**, STOP (albedo-heat transport)'
          STOP
	ENDIF
!-----------------------------------------------------------------------
	IF (imodel==1) THEN
          CALL smod1(nbodf1,nbodf2,nbodf3,rmodfx,rmodfy,rmodfz,&
     gainfx,gainfy,gainfz,&
     rstar,tstar,emstar,xstar,ystar,zstar,vrotst,vxst,vyst,vzst,&
     istar,dgst,ffst,irrst,albst,htst,htsta,&
     icomp,dgcp,ffcp,qq,vrxcp,vrycp,vrzcp,vrotcp,rcp,&
     xcp,ycp,zcp,vxcp,vycp,vzcp,tempcp,irrcp,albcp,htcp,htcpa,&
     ispot,vrxsp,vrysp,vrzsp,vrotsp,rsp,&
     xsp,ysp,zsp,vxsp,vysp,vzsp,tempsp,denssp,anesp,vtrbsp,&
     xdisc,ydisc,zdisc,adisc,rindc,routdc,idisc,edendc,itdc,&
     vxdc,vydc,vzdc,tempdc,densdc,anedc,vtrbdc,&
     x1sm,y1sm,z1sm,x2sm,y2sm,z2sm,v1sm,v2sm,rsm,ism,&
     tempsm,denssm,anesm,vtrbsm,&
     xjet,yjet,zjet,ajet,rinjt,routjt,vjt,&
     tempjt,densjt,anejt,vtrbjt,ijet,&
     rinsh,routsh,vsh,&
     tempsh,denssh,anesh,vtrbsh,ishell,evelsh,rcsh,&
     v0,temp0,dens0,ane0,&
     far,fat,faz,ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb)
	ENDIF
	IF (imodel==2) THEN
	  OPEN(10,FILE='shellspec.mod',STATUS='old')
          CALL smod2(nbodf1,nbodf2,nbodf3,&
     far,fat,faz,ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb)
	  CLOSE(10)
        ENDIF
!		definition of the line of sight grid
	CALL deflsg(nbod1,nbod2,nbod3,&
     rmodx,rmody,rmodz,gainx,gainy,gainz,ar,at,az)
!		reading the atomic line data
	CALL lindat(iat,iion,wlab,elo,eup,glo,gr0,gs0,gw0,bij,&
     nline)
!		reading the input spectra of nontransparent objects
	CALL untsp(xstar1,star1,xstar2,star2,xstar3,star3,&
     wstar1,fstar1,tstar,jstar1,wstar2,fstar2,tempcp,jstar2,&
     xunt1,yunt1,xunt2,yunt2,xunt3,yunt3,&
     nstar1,nstar2,nstar3,nspec1,nspec2,tspec1,tspec2,&
     lunt1,lunt2,lunt3,dlst,dlst2,dlcp,dlcp2)
!		reads the atomic data for all elements and ions
!		dyp(*)-name of the element
!		d(1,*)-atom weight
!		d(2,*)-solar abundance relative to hydrogen
!		d(3,*)-number of ions considered
!		xi(*,*)-first 8 ionization potentials
	CALL state0(dyp,d,xi)
!		extract data for our particular elements and 
!		change abundances: 

	CALL eldat(ichemc,iat,iion,dyp,d,xi,abhyd,abhel,&
     wm,nion,abund,hmc,zip,nline,ielnd,necod)
!		calculation of the electron number density
	IF (ielnd==1) CALL elnd(d,xi,necod,wm,abhyd,ftemp,fdens,fne,&				
     nbodf1,nbodf2,nbodf3,ane0)
	WRITE(2,130)nbod1,nbod2,nbod3,nfreq
130	FORMAT(' nbod1=',i4,' nbod2=',i4,' nbod3=',i4,' nfreq=',i4)
	WRITE(2,'(3(a,i3))')' output for:  ionu=',ionu,&
     ' iore=',iore,' iot=',iot	
!		area(i,j)-area of a projected surface element
	CALL surf(nbod1,nbod2,ar,at,area)
!-----------------------------------------------------------------------
!		grand cycle through: angles, x,y, frequencies, and z
!	aint0-incident intesity from behind
	DO  iang=1,nphase
        
	  WRITE(6,'(2(a,f7.2))')'i=',dinc*180.d0/pi,&
	     '  alpha=',alpha(iang)*180.d0/pi
          WRITE(2,'(2(a,f7.2))')'i=',dinc*180.d0/pi,&
	     '  alpha=',alpha(iang)*180.d0/pi
!	  rotation of the object:
	  IF(irotat==0)THEN

	    CALL rotat1(dinc,alpha(iang),temp0,ane0,&
     			 vxst,vyst,vzst,vxstr,vystr,vzstr,&
		         xcp,ycp,zcp,xcpr,ycpr,zcpr,&
		         vxcp,vycp,vzcp,vxcpr,vycpr,vzcpr,&
		         nbodf1,nbodf2,nbodf3,nbod1,nbod2,nbod3,&
		         ar,at,az,far,fat,faz,&
		         atemp,apress,adens,ane,avr,avt,avz,avtrb,&
		         ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb)
	   ELSE

	    CALL rotat2(dinc,alpha(iang),temp0,ane0,&
     			vxst,vyst,vzst,vxstr,vystr,vzstr,&
    			xcp,ycp,zcp,xcpr,ycpr,zcpr,&
     			vxcp,vycp,vzcp,vxcpr,vycpr,vzcpr,&
     			nbodf1,nbodf2,nbodf3,nbod1,nbod2,nbod3,&
     			ar,at,az,far,fat,faz,&
     			atemp,apress,adens,ane,avr,avt,avz,avtrb,&
  			ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb)
	  ENDIF

	  DO  ii=1,nbod1
	  	DO  jj=1,nbod2
		    IF(ii==iore.and.jj==iot)THEN
		      iprint=1
		    ELSE
		      iprint=0
		    ENDIF	
!		check for an empty space
		    ivac1=0
		    aint0=0.d0	
			    DO  kk=1,nbod3
				      IF(adens(ii,jj,kk)>denvac)THEN
					ivac1=kk
					EXIT
				      ENDIF
			   ENDDO
     		      ivacn=0
	 		   DO kk=nbod3,1,-1
	 			      IF(adens(ii,jj,kk)>denvac)THEN
					ivacn=kk
			              EXIT
				      ENDIF	
   			  ENDDO 
	 		   IF(ivac1==0)THEN

	   		  	 DO nu=1,nfreq
	    			    ainte(ii,jj,nu)=aint0
	                         ENDDO	

	   	 		  IF(ii==iore.and.jj==iot)THEN
	    	   		  WRITE(2,*)' ii=',ii,'   jj=',jj,&
    				 '   this ray goes through an empty space'
	        		  ENDIF	

	   			 CYCLE		
             
             		   ELSE
	     
		 		DO kk=ivac1,ivacn
 		        	temp(kk-ivac1+1)=atemp(ii,jj,kk)
			        ekonc(kk-ivac1+1)=ane(ii,jj,kk)
			        akonc(kk-ivac1+1)=adens(ii,jj,kk)/wm/hjed
			        dens(kk-ivac1+1)=adens(ii,jj,kk)
!	        press(kk-ivac1+1)=apress(ii,jj,kk)
			        azn(kk-ivac1+1)=az(kk)
			        vr(kk-ivac1+1)=avr(ii,jj,kk)
			        vt(kk-ivac1+1)=avt(ii,jj,kk)
			        vz(kk-ivac1+1)=avz(ii,jj,kk)
			        vtrb(kk-ivac1+1)=avtrb(ii,jj,kk)
  		           	ENDDO
		          	 nivac=ivacn-ivac1+1
	          	    ENDIF
  				 CALL bcond1(iunt,darkl,cdelta,vdif,nivac,dens,&
   				  vr,vt,vz,&
   		 		 ar(ii),at(jj),azn,dinc,alpha(iang),&
     		  		istar,rstar,vxstr,vystr,vzstr,dlst,dlst2,irrst,&
   		  		icomp,rcp,xcpr,ycpr,zcpr,vxcpr,vycpr,vzcpr,&
   		  		dlcp,dlcp2,irrcp,xcp,qq,iprint)
				  CALL hihei(ipart,nivac,temp,ekonc,akonc,hipf,hi,hneg,hei,&
		     		abhyd,abhel)
!	    	ophbf -HI bound-free opacity
!	    	ophff -HI free-free opacity
!	    	ophrs -HI Rayleigh scattering
 				 CALL ophyd(nivac,temp,ekonc,akonc,hi,abhyd,&
				   ophbf1,ophff1,ophrs1,hipf,alam1)
  				 CALL ophyd(nivac,temp,ekonc,akonc,hi,abhyd,&
    				  ophbf2,ophff2,ophrs2,hipf,alamn)
			   IF (iprint==1)THEN        
          			    WRITE(2,*)' iunt=',iunt,' ivac1=',ivac1,' nivac=',nivac
	  			    CALL tlac1(nivac,azn,vr,vt,vz,&
	     				temp,ekonc,akonc,hi,hneg,hei,hipf,&
	     				ophbf1,ophff1,ophrs1)
   		            ENDIF
  			    DO  iline=1,nline
	     	          	  IF(ipart==1)THEN
	      				  CALL pfdwir(nivac,miion,nion(iline),iat(iline),&
						     temp,ekonc,stavs)
           		  	  ELSE
              				  CALL pfirw(nivac,nion(iline),sta,temp,stavs)
            		  	  ENDIF
	      		  	  DO  ll=1,mion-1
	        				 zipp(ll)=zip(ll,iline)
	     		  	  ENDDO
	     		  	  CALL zastup(temp,ekonc,zipp,stavs,rr,nivac,nion(iline))
	    		  	  DO kk=1,nivac
	     				  popul(kk,iline)=bolt(temp(kk),stavs(kk,iion(iline)),&
	      				  glo(iline),elo(iline),1)&
	    				  *rr(kk,iion(iline))*abund(iline)*abhyd*akonc(kk)
	   		  	  ENDDO
!		calculates damping constants
!	    	aa-combined damping constant
!	    	dop-Doppler halfwidth	
           		  	  DO kk=1,nivac
	      				  CALL utlm(wlab(iline),eup(iline),iat(iline),hmc(iline),&
	     					     iion(iline),zipp(iion(iline)),hi(kk),hei(kk),&
						     ekonc(kk),temp(kk),vtrb(kk),gr(iline),gr0(iline),&
						     gs(iline),gs0(iline),gw(iline),gw0(iline),&
						     aa(kk,iline),dop(kk,iline))
	     		  	  ENDDO
!		prints various quantities	    
	 		  	  IF (iprint==1)THEN        
	    					    WRITE(2,*)' iline=',iline	
	        				    CALL tlac2(nivac,&
	 					   nion(iline),iion(iline),azn,zipp,eup(iline),&
						   temp,stavs,rr,gr(iline),gs(iline),gw(iline))
		          	  ENDIF
			    ENDDO
!
			    DO nu=1,nfreq
	   				   IF(nu==ionu.and.ii==iore.and.jj==iot)THEN
	    				    iprint=1
	 		  	           ELSE
	 	 		            iprint=0
	   			   	   ENDIF		
!		interpolates continuum opacity to alam(nu)
	   			   	   CALL medop(nivac,alam1,alamn,alam(nu),&
				     		ophbf,ophbf1,ophbf2,ophff,ophff1,ophff2,&
				     		ophrs,ophrs1,ophrs2)
!		wlab-laboratory lambda of the line center in [A]
!		alam(nu)-solving lambda in [A]
!		freq-frequency of alam(nu)
						freq=clight/alam(nu)*1.d8
!
!		calculation of aintb- boundary condition for intensity
!		behind the last untransparent object along the ray
	  			    	    CALL bcond2(iunt,dens,temp,vz,vdif,freq,&
				    		darkl,cdelta,lunt1,nstar1,xstar1,&
						star1,nspec1,tspec1,alb1x,alb1y,nalb1,&
				    		tstar,dlst,dlst2,irrst,albst,wstar1,fstar1,jstar1,&
				    		lunt2,nstar2,xstar2,star2,nspec2,&
						tspec2,alb2x,alb2y,nalb2,&
				     		tempcp,dlcp,dlcp2,irrcp,albcp,wstar2,fstar2,jstar2,&
				     		lunt3,nstar3,xstar3,star3,aint0,aintb)
	 			    	   IF(iprint==1)THEN
           			    		 WRITE(2,*)' iunt=',iunt,' aintb=',aintb
           			   	   ENDIF
!
!	        this is the 1D radiative trasfer routine
	 			     	   CALL rte(nivac,alam(nu),aintb,ainte(ii,jj,nu),&
					     iunt,wlab,bij,aa,dop,popul,nline,&
					     istar,tstar,rstar,wstar1,fstar1,jstar1,lunt1,&
			    		     vxstr,vystr,vzstr,&
					     icomp,tempcp,rcp,wstar2,fstar2,jstar2,lunt2,&
					     vxcpr,vycpr,vzcpr,xcpr,ycpr,zcpr,&
					     ophbf,ophff,ophrs,ithom,irayl,&
					     temp,dens,ekonc,akonc,azn,vr,vt,vz,&
					     ar(ii),at(jj),iprint,eps)
			    ENDDO	
		      ENDDO 
	  ENDDO
!
          plocha=0.d0
	  DO i=1,nbod1
            WRITE(20+iang,*)
	    DO j=1,nbod2
              WRITE(20+iang,'(2e15.6,e15.6e3)')ar(i),at(j),ainte(i,j,ionu)
	      DO k=nbod3,1,-1
	        IF (adens(i,j,k)>dcut1)THEN
		  plocha=area(i,j)+plocha
		 EXIT 
		ENDIF
	      ENDDO
	    ENDDO
	   ENDDO

	  IF(rstar>0.d0)THEN
	    ratio=plocha/pi/rstar/rstar
            WRITE(2,*)' star disk',pi*rstar**2
            WRITE(2,*)' sum_ij untranarea_ij/star disk',ratio            
	  ENDIF

	  DO nu=1,nfreq
          	  flux(nu,iang)=0.d0
		    DO i=1,nbod1
 			    DO j=1,nbod2
           			   flux(nu,iang)=ainte(i,j,nu)*area(i,j)/dd/dd+flux(nu,iang)
 	           	    ENDDO
                    ENDDO
          ENDDO
	
	  der=(flux(nfreq,iang)-flux(1,iang))/(alam(nfreq)-alam(1))

	  DO nu=1,nfreq
!	    flux in [erg/cm^2/s/Hz], fluxl in [erg/cm^2/s/cm], 
!	    fluxn -normalized flux 
!	    fluxs -shifted fluxn	
	    cont=flux(1,iang)+der*(alam(nu)-alam(1))
	    fluxl=flux(nu,iang)*clight/alam(nu)/alam(nu)*1.d16	
	    fluxn=flux(nu,iang)/cont
	    fluxs=fluxn-dble(iang-1)*offset
	    IF(nline>=1)THEN
	      refw=wlab(1)	
	    ELSE
              refw=alam1 
	    ENDIF
	    dvel=(alam(nu)-refw)/refw*clight*1.d-5
	    WRITE(4,'(f11.3,f10.1,5e14.6)')alam(nu),dvel,flux(nu,iang),&
		    fluxl,fluxn,fluxs
!	    WRITE(100+iang,'(f10.3,f10.1,5e14.7)')alam(nu),dvel
!  &     ,flux(nu,iang),fluxl,fluxn,fluxs
	    WRITE(11,'(f7.3,f10.1,f10.4,f11.3,e14.6)')alpha(iang)/&
		     2.d0/pi,dvel,-2.5d0*log10(fluxl),alam(nu),flux(nu,iang)
	ENDDO
	  WRITE(11,*)
	  WRITE(4,*)
	  WRITE(2,*)
!	  end of rotation cycle
	ENDDO
	CLOSE(2)
!	CLOSE(3)
	CLOSE(4)
	CLOSE(11)
!ENDIF

END PROGRAM


!      instead of using param.inc 
!      definition of global paramters
!

 SUBROUTINE rte(nbod,alam,aintb,ainte,&
     iunt,wlab,bij,aa,dop,popul,nline,&
     istar,tstar,rstar,xstar1,star1,nstar1,lunt1,&
     vxstr,vystr,vzstr,&
     icomp,tempcp,rcp,xstar2,star2,nstar2,lunt2,&
     vxcpr,vycpr,vzcpr,xcpr,ycpr,zcpr,&
     ophbf,ophff,ophrs,ithom,irayl,&
     temp,dens,ekonc,akonc,az,vr,vt,vz,&
     ari,ati,iprint,eps)
!	to solve the radiative tranfer along the line of sight
!	at the wavelength -alam
!	input:  nbod -number of depth points
!		alam -wavelength [A]
!		aintb -boundary condition for intensity 
!		iunt -last untransparent point
!		wlab -laboratory lambda of the line center [A]
!		bij - Einstein coefficient B_lu per unit solid angle
!		istar,icomp -on/off switch of the scattered light
!		tstar,rstar -temperature,radius of the spherical 
!			central star
!		xstar1,star1,nstar1,lunt1 -spectrum of the central star
!		vxstr,vystr,vzstr -velocity of the central star
!		tempcp,rcp -temperature,radius of the spherical 
!			secondary star
!		xstar2,star2,nstar2,lunt2 -spectrum of the second star
!		vxcpr,vycpr,vzcpr -velocity of the second star
!		xcpr,ycpr,zcpr -position of the second star
!		aa,dop,ophbf,ophff,ophrs,ithom,irayl -
!		temp,dens,ekonc,akonc,az,vr,vt,vz,popul -
!		ari,ati - x,y coordinates defining the line 
!			of sight along z
!		denvac - limiting density of vacuum
!		iprint - signal to print
!		eps - In LTE eps=1. ( S=eps*B+(1-eps)*J ) 
!	output: 
!		ainte-emerging intensity [erg/cm^2/s/Hz/rad]
!
!    temp, ane -are assumed to have reasonable values all along
!    the beam. An empty space can be identified as dens<denvac.
!	Integration starts behind the last untransparent object.
	USE sun
 	USE param
	USE unitchange
	USE constants

 IMPLICIT NONE

 INTEGER:: id1,ll,nbod,nline,iunt,ithom,irayl,&
	   iprint,jj1,jj2,icheck,istar, icomp,nstar1,&
	   lunt1,nstar2,lunt2
 REAL(KIND=8),DIMENSION(ndim,mline)::aa,dop,popul
 REAL(KIND=8),DIMENSION(mline)::flab,wlab,bij,bij2
 REAL(KIND=8),DIMENSION(ndim)::ophbf,ophff,ophrs,&
                      temp,dens,ekonc,&
                      vr,vt,vz,az,&
                      bnu,ejnu1,ejnu2,&
                      sf2,opac2,emis2
 REAL(KIND=8),DIMENSION(ndim)::akonc
 REAL(KIND=8),DIMENSION(mstarx)::xstar1,star1,xstar2,star2
 REAL(KIND=8)::freq,alam,aintb,ainte,&
     tstar,rstar,&
     vxstr,vystr,vzstr,&
     tempcp,rcp,&
     vxcpr,vycpr,vzcpr,xcpr,ycpr,zcpr,&     
     ari,ati,eps,&
     emth1,emth2,emth,emra,emra1,emra2,emlinb,&
     emlinj,emhiff,emhibf,dz12,dippf,dfreqd,alph,&
     opth,oplin,opdep,opac12,opac1,sf12,sf1,&
     flabs,er2z,er2,vprof
  REAL(KIND=8):: voigt,planck

!		flab -frequency [1/s] of wlab
!		flabs-frequency of the line center in comoving frame
!		alam-solving lambda in [A]
!		freq-frequency of alam
	freq=clight/alam*1.d8
 DO ll=1,nline
  flab(ll)=clight/wlab(ll)*1.d8
  bij2(ll)=bij(ll)*plank*flab(ll)/sqrt(pi)
 ENDDO
!	ainte-is set to boundary condition for intensity
	ainte=aintb
	IF (nbod-iunt<2) RETURN 
!		popul-population of the lower level
!		opdep- optical depth as measured from behind the shell
!		      or from an untransparent object
!		opa!opacity
!		emis-emisivity
!		sf-source function
        IF (iprint==1)THEN
	  WRITE(2,*)' lambda=',alam
	  WRITE(2,'(a,a,a)')' id  waves      ' ,&     
     'ophbf     ophff     opth      ophrs     oplin     opdep',&
     '     ainte      voigt'
	ENDIF
	opdep=0.d0
	icheck=0
!		initialize the first value of jj for hunt
	jj1=1
	jj2=1
!		BEGINNING OF THE DEPTH CYCLE ---------------------------
	DO id1=iunt+1,nbod
		  IF(dens(id1)<denvac)THEN
	    		icheck=0
            	  IF (iprint==1)THEN
	      		WRITE(2,'(i3,a)')id1,' empty space-skipped'
	    	  ENDIF
	  	  CYCLE 
	  	  ENDIF
	  	oplin=0.d0
	 	DO ll=1,nline
	    		flabs=vz(id1)/clight*flab(ll)+flab(ll)
	    		dfreqd=(freq-flabs)/dop(id1,ll)
!	  			\INT(VOIGHT F.)dFREQ 
!				HAS NORMALIZATION= SQRT(PI)*DOP.W.[1/S]
	    		vprof=voigt(dfreqd,aa(id1,ll))
!				line opacity
	    		oplin=(1.d0-exp(-4.7992375d-11*flab(ll)/temp(id1)))&
     	    		*bij2(ll)*popul(id1,ll)*vprof/dop(id1,ll)+oplin
	  	ENDDO
!		Thomson scattering opacity
          	IF(ithom==1)THEN
	    		opth=ekonc(id1)*6.65d-25
	  	ELSE
	    		opth=0.d0
	  	ENDIF
!		Rayleigh scattering on HI opacity
          	IF(irayl==0)ophrs(id1)=0.d0
!		total opacity
	  		opac2(id1)=oplin+opth+ophbf(id1)+ophff(id1)+ophrs(id1)
	  		bnu(id1)=planck(freq,temp(id1),1)
!		HI bound-free and free-fee emissivity
	  		emhibf=ophbf(id1)*bnu(id1)
	  		emhiff=ophff(id1)*bnu(id1)
!		ejnu - mean intensity
!		emth,emra - Thompson and Rayleigh scattering emissivity
          		IF(ithom==1.or.irayl==1)THEN
	    			IF(istar>0)THEN	
     	      			CALL jnu(rstar,tstar,ari,ati,az(id1),&
                   				vr(id1),vt(id1),vz(id1),freq,&
                   				xstar1,star1,nstar1,lunt1,&
                   				vxstr,vystr,vzstr,jj1,&
						ejnu1(id1),iprint,0.d0,0.d0,0.d0)
	      			er2z=az(id1)*az(id1)
	      			er2=ari*ari+ati*ati+er2z
	      				IF(er2>(rstar*rstar*16.d0))THEN
!		dipol phase function scattering
	        				dippf=0.75d0*(1.d0+er2z/er2)
	        				emth1=dippf*ejnu1(id1)*opth
	        				emra1=dippf*ejnu1(id1)*ophrs(id1)
	      				ELSE
!		isotropic scattering	
	        				emth1=ejnu1(id1)*opth
	        				emra1=ejnu1(id1)*ophrs(id1)
	      				ENDIF
     	    			ELSE
     	      				ejnu1(id1)=0.d0
     	      				emth1=0.d0
	      				emra1=0.d0
     	    			ENDIF
     	    			IF(icomp>0)THEN  	
       	      				CALL jnu(rcp,tempcp,ari,ati,az(id1),&
                   					vr(id1),vt(id1),vz(id1),freq,&
				                        xstar2,star2,nstar2,lunt2,&
                   					vxcpr,vycpr,vzcpr,jj2,ejnu2(id1),iprint,xcpr,ycpr,zcpr)
     	      						er2z=(az(id1)-zcpr)**2
	      						er2=(ari-xcpr)**2+(ati-ycpr)**2+er2z
	      				IF(er2>(rcp*rcp*16.d0))THEN
!		dipol phase function scattering
	        				dippf=0.75d0*(1.d0+er2z/er2)
	        				emth2=dippf*ejnu2(id1)*opth
	        				emra2=dippf*ejnu2(id1)*ophrs(id1)
	      				ELSE
!		isotropic scattering	
	        				emth2=ejnu2(id1)*opth
	        				emra2=ejnu2(id1)*ophrs(id1)
	      				ENDIF
     	    			ELSE
     	      				ejnu2(id1)=0.d0
     	      				emth2=0.d0
	      				emra2=0.d0
     	    			ENDIF     
	  		ELSE
	    			ejnu1(id1)=0.d0
	    			ejnu2(id1)=0.d0
     	   			emth1=0.d0
	    			emra1=0.d0
	    			emth2=0.d0
	    			emra2=0.d0
	  		ENDIF
	  		emth=emth1+emth2
	  		emra=emra1+emra2
!		line emissivity
	  		emlinb=eps*oplin*bnu(id1)
	  		emlinj=(1.d0-eps)*oplin*(ejnu1(id1)+ejnu2(id1))
!		total emissivity
	  		emis2(id1)=emlinb+emlinj+emth+emhibf+emhiff+emra
!		total source function
	  		sf2(id1)=emis2(id1)/opac2(id1)
!		integrates ainte and opdep
	  		IF (icheck>0)THEN
	      			opac12=(opac1+opac2(id1))*0.5d0
	      			sf12=(sf1+sf2(id1))*0.5d0
	      			dz12=(az(id1)-az(id1-1))
	      			alph=1.d0/opac12/dz12
	      			ainte=(ainte*(alph-0.5d0)+sf12)/(alph+0.5d0)
	      			opdep=opac12*dz12+opdep	    
              			IF (iprint==1)THEN
	        			WRITE(2,'(i3,f10.3,11e10.3)')id1,clight/flabs*1.d8,&
     					ophbf(id1),ophff(id1),opth,ophrs(id1),oplin,opdep,&
     					ainte,vprof
	      			ENDIF
	  		ENDIF
	  		opac1=opac2(id1)
	  		sf1=sf2(id1)
	  		icheck=icheck+1
	ENDDO
!            END OF THE DEPTH CYCLE----------------------------------
        IF (iprint==1)THEN
	  WRITE(2,'(a,a)')' id    opac2      emis2      bnu       ejnu1',&
'       ejnu2     sf2        az         vz'
	  DO id1=iunt+1,nbod
	  	IF(dens(id1)>denvac)THEN
	      		WRITE(2,'(i3,8e11.3)')id1,opac2(id1),emis2(id1),bnu(id1),&
                          ejnu1(id1),ejnu2(id1),sf2(id1),az(id1),vz(id1)
	    	ENDIF
          ENDDO
	ENDIF


 END SUBROUTINE rte


!-----------------------------------------------------------------------
 SUBROUTINE pfirw(nbod,nion,sta,temp,stavs)
!	partition functions after Irwin
!	input: sta -Irwin's tabulated coeficients for each ion
!		of an atom
!	output: stavs(i,j)-partition function of j-th ion at i-th depth
 USE param
 IMPLICIT NONE 

 INTEGER::nbod,nion,i,j
 REAL(KIND=8),DIMENSION(6,mion)::sta
 REAL(KIND=8),DIMENSION(ndim,mion)::stavs        
 REAL(KIND=8),DIMENSION(ndim)::temp
 REAL(KIND=8)::sala,arg
 
       DO  i=1,nbod
	  DO  j=1,nion
	    IF (abs(sta(1,j)-0.d0)<1.d-30) THEN 
	    stavs(i,j)=1.d0
	    ELSE
	    sala=log(temp(i))
	    arg=sta(1,j)+sta(2,j)*sala+sta(3,j)*sala*sala+&
            sta(4,j)*sala**3+sta(5,j)*sala**4+sta(6,j)*sala**5
	    stavs(i,j)=exp(arg)
	    ENDIF
	  END DO
        END DO

!     WRITE (2,'(A)')' PARTITION FUNCTIONS'
!     WRITE (2,'(A,20I10)')' TEMP',(I,I=1,NION)
!	 DO 140 I=1,NBOD
!	  WRITE(2,130)TEMP(I),(STAVS(I,J),J=1,NION)
!130       FORMAT(F7.0,20E10.3) 
!140     CONTINUE

 END SUBROUTINE pfirw

 SUBROUTINE potlow(temp,ane,eplow)
 USE constants

 IMPLICIT NONE

 REAL(KIND=8)::ane,temp,pom,eplow,debye
!    temp - Temperature[K]
!    ane - electron number density [cm^{-3}]
!    lowering of the ionization potential for neutrals, z=1 in [eV]
!    for ions(z):      eplow(z)=z*eplow(1)
 pom=bol*temp/8.d0/pi/ane
!    debye lenght in [CGS]
 debye=sqrt(pom)/e
 eplow=1.44d-7/debye

 END SUBROUTINE potlow

 SUBROUTINE pfdwir(nbod,miion,nion,iat,temp,ekonc,stavs)
!
!    partition functions from UCLSYN:
! Smith K.C., Dworetsky M.M., 1988, In: Adelman S.J., Lanz T., eds,
! Elemental Abundance Analyses.  Institut d'Astronomie de l'Univ.  de
! Lausanne, Switzerland, p. 32
!	Budaj J., Dworetsky M.M., Smalley B, 2002,
!	Comm. Univ. London Obs. No. 82
!    URL=http://www.ulo.ucl.ac.uk/ulo_comms/82/index.html
!
!	output: stavs(i,j)-partition function of j-th ion at i-th depth

 USE param
 USE pfdwor

 IMPLICIT NONE

 REAL(KIND=8),DIMENSION(ndim,miion)::stavs
 REAL(KIND=8),DIMENSION(ndim)::ekonc,temp
 REAL(KIND=8),DIMENSION(5)::u
 REAL(KIND=8)::eplow,theta
 INTEGER::i,j,nbod,nion,iat
 INTEGER::miion
 DO i=1,nbod
 CALL potlow(temp(i),ekonc(i),eplow)
    theta=5039.77d0/temp(i)
!PARTFN IS IN 'pfdwor.f90'
 CALL partfn(iat,theta,u,eplow)
    DO  j=1,nion
        IF(j<=5)THEN
	  stavs(i,j)=u(j)
	ELSE
	  stavs(i,j)=1.d0
	ENDIF
    END DO
 END DO

!     WRITE (2,'(A)')' PARTITION FUNCTIONS'
!     WRITE (2,'(A,20I10)')' TEMP',(I,I=1,NION)
!     DO 140 I=1,NBOD
!       WRITE(2,130)TEMP(I),(STAVS(I,J),J=1,NION)
!130       FORMAT(F7.0,20E10.3)
!140     CONTINUE
 END SUBROUTINE pfdwir

 SUBROUTINE zastup(temp,ekonc,zip,stavs,rr,nbod,nion)
!	input:  temp,ekonc,nbod,nion
!		zip-ionization potentials
!		stavs-partition functions
!	output:
!    RR(I,J)- POPULATION OF THE J-TH ION/TOTAL ELEMENT POPULATION
!		 AT I-TH DEPTH
!	R1(J)= N(J)/N(1)
!	RION(J)= N(J)/N(J-1)
!	SAH	-CALLING FUNCTION


USE param
 IMPLICIT NONE
 INTEGER::i,j,nbod,nion
 REAL(KIND=8),DIMENSION(mion-1)::zip
 REAL(KIND=8),DIMENSION(ndim,mion)::stavs,rr
 REAL(KIND=8),DIMENSION(mion)::temp,ekonc,rion,r1
 REAL(KIND=8)::sum1
 REAL(KIND=8):: sah

        DO  i=1,nbod

       	  sum1=1.d0
	  rion(1)=1.d0
	  r1(1)=1.d0

	     DO j=2,nion

	     rion(j)=sah(temp(i),ekonc(i),zip(j-1),stavs(i,j-1),&
             stavs(i,j))
             r1(j)=r1(j-1)*rion(j)
	     sum1=sum1+r1(j)

 	     END DO
	     
             DO j=1,nion

	     rr(i,j)=r1(j)/sum1

	     END DO
       END DO
!      WRITE(2,*)'   N(J)/N  ION POPULATIONS'
!      WRITE(2,'(A,40I11)')' DM',(I,I=1,NION)
!	DO 50 I=1,NBOD
!	  WRITE(2,130)DM(I),(RR(I,J),J=1,NION)
!130	  FORMAT(40(D10.3,1X))
!50	CONTINUE
 END SUBROUTINE zastup

 FUNCTION sah(te,ek,zp,s1,s2)

 IMPLICIT NONE

 REAL(KIND=8)::te,ek,zp,s1,s2,sah

!          SAHA EQUATION (RATIO OF THE NUMBER DENSITIES OF
!		TWO ADJACENT IONS)
!          ZP -in [EV], TE -in [K], EK -in [cm^-3]

 sah=4.8293745d15*s2/s1*sqrt(te)**3 &
     *exp(-1.16045059d4/te*zp)/ek

 END FUNCTION sah

 FUNCTION sahan(te,zp,s1,s2)

 IMPLICIT NONE

 REAL(KIND=8)::te,zp,s1,s2,sahan

!          PART OF SAHA EQUATION WITHOUT ELECTRON NUMBER DENSITY
!          ZP -in [EV], TE -in [K]

 sahan=4.8293745d15*s2/s1*sqrt(te)**3*exp(-1.16045059d4/te*zp)

 END FUNCTION sahan

 
SUBROUTINE state0 (Adyp,Ad,Axi)
!		taken from the Synspec code:
! 	Hubeny I., Lanz T., Jeffery C.S., 1994, 
!	in: Newsletter on Analysis of Astronomical spectra No.20, 
!	ed. C.S. Jeffery (CCP7; St. Andrews: St. Andrews Univ.), 30
!
!  Initialization of the basic parameters for the Saha equation
!
USE param
 IMPLICIT NONE

!	parameter(matom=99)
 CHARACTER(LEN=4),DIMENSION(matom):: dyp,adyp
 REAL(KIND=8),DIMENSION(3,matom)::d,ad
 REAL(KIND=8),DIMENSION(8,matom):: xi,axi
 INTEGER::i,j

!
      DATA DYP/' H  ',' He ',' Li ',' Be ',' B  ',' C  ',&
              ' N  ',' O  ',' F  ',' Ne ',' Na ',' Mg ',&
              ' Al ',' Si ',' P  ',' S  ',' Cl ',' Ar ',&
              ' K  ',' Ca ',' Sc ',' Ti ',' V  ',' Cr ',&
              ' Mn ',' Fe ',' Co ',' Ni ',' Cu ',' Zn ',&
              ' Ga ',' Ge ',' As ',' Se ',' Br ',' Kr ',&
              ' Rb ',' Sr ',' Y  ',' Zr ',' Nb ',' Mo ',&
              ' Tc ',' Ru ',' Rh ',' Pd ',' Ag ',' Cd ', &
              ' In ',' Sn ',' Sb ',' Te ',' I  ',' Xe ',&
              ' Cs ',' Ba ',' La ',' Ce ',' Pr ',' Nd ', &
              ' Pm ',' Sm ',' Eu ',' Gd ',' Tb ',' Dy ', &
              ' Ho ',' Er ',' Tm ',' Yb ',' Lu ',' Hf ', &
              ' Ta ',' W  ',' Re ',' Os ',' Ir ',' Pt ',& 
              ' Au ',' Hg ',' Tl ',' Pb ',' Bi ',' Po ',& 
              ' At ',' Rn ',' Fr ',' Ra ',' Ac ',' Th ',& 
              ' Pa ',' U  ',' Np ',' Pu ',' Am ',' Cm ',& 
              ' Bk ',' Cf ',' Es '/
!
! Standard atomic constants for first 99 species 
!   Abundances for the first 30 from Grevesse & Sauval,
!      (1998, Space Sci. Rev. 85, 161)
!
!         Element Atomic  Solar    Std.
!                 weight abundance highest 
! 
!                                  ionization stage 
      DATA D/ 1.008D0, 1.00D0, 2.0D0,&
             4.003D0, 1.00D-1, 3.0D0,&
             6.941D0, 1.26D-11, 4.0D0,&
             9.012D0, 2.51D-11, 5.0D0,&
            10.810D0, 5.0D-10, 5.0D0,&
            12.011D0, 3.31D-4, 5.0D0,&
            14.007D0, 8.32D-5, 5.0D0,&
            16.000D0, 6.76D-4, 5.0D0,&
            18.918D0, 3.16D-8, 5.0D0,&
            20.179D0, 1.23D-4, 5.0D0,&
            22.990D0, 2.14D-6, 5.0D0,&
            24.305D0, 3.80D-5, 5.0D0,&
            26.982D0, 2.95D-6, 5.0D0,&
            28.086D0, 3.55D-5, 5.0D0,&
            30.974D0, 2.82D-7, 5.0D0,&
            32.060D0, 2.14D-5, 5.0D0,&
            35.453D0, 3.16D-7, 5.0D0,&
            39.948D0, 2.52D-6, 5.0D0,&
            39.098D0, 1.32D-7, 5.0D0,&
            40.080D0, 2.29D-6, 5.0D0,&
            44.956D0, 1.48D-9, 5.0D0,&
            47.900D0, 1.05D-7, 5.0D0,&
            50.941D0, 1.00D-8, 5.0D0,&
            51.996D0, 4.68D-7, 5.0D0,&
            54.938D0, 2.45D-7, 5.0D0,&
            55.847D0, 3.16D-5, 5.0D0,&
            58.933D0, 8.32D-8, 5.0D0,&
            58.700D0, 1.78D-6, 5.0D0,&
            63.546D0, 1.62D-8, 5.0D0,&
            65.380D0, 3.98D-8, 5.0D0,&
            69.72D0,   1.34896324D-09  ,  3.0D0,  &
            72.60D0 ,   4.26579633D-09  ,  3.0D0, & 
            74.92D0 ,   2.34422821D-10  ,  3.0D0,  &
            78.96D0 ,   2.23872066D-09  ,  3.0D0,  &
            79.91D0 ,   4.26579633D-10  ,  3.0D0,  &
            83.80D0 ,   1.69824373D-09  ,  3.0D0,  &
            85.48D0 ,   2.51188699D-10  ,  3.0D0,  &
            87.63D0 ,   8.51138173D-10  ,  3.0D0,  &
            88.91D0 ,   1.65958702D-10  ,  3.0D0,  &
            91.22D0 ,   4.07380181D-10  ,  3.0D0,  &
            92.91D0 ,   2.51188630D-11  ,  3.0D0,  & 
            95.95D0 ,   9.12010923D-11  ,  3.0D0,  & 
            99.00D0 ,   1.00000000D-24  ,  3.0D0,  & 
            101.1D0 ,   6.60693531D-11  ,  3.0D0,  & 
            102.9D0 ,   1.23026887D-11  ,  3.0D0,  & 
            106.4D0 ,   5.01187291D-11  ,  3.0D0,  & 
            107.9D0 ,   1.73780087D-11  ,  3.0D0,  & 
            112.4D0 ,   5.75439927D-11  ,  3.0D0,  & 
            114.8D0 ,   6.60693440D-12  ,  3.0D0,  & 
            118.7D0 ,   1.38038460D-10  ,  3.0D0,  & 
            121.8D0 ,   1.09647810D-11  ,  3.0D0,  & 
            127.6D0 ,   1.73780087D-10  ,  3.0D0,  & 
            126.9D0 ,   3.23593651D-11  ,  3.0D0,  & 
            131.3D0 ,   1.69824373D-10  ,  3.0D0,  & 
            132.9D0 ,   1.31825676D-11  ,  3.0D0,  & 
            137.4D0,   1.62181025D-10  ,  3.0D0,  & 
            138.9D0 ,   1.58489337D-11  ,  3.0D0,  & 
            140.1D0 ,   4.07380293D-11  ,  3.0D0,  & 
            140.9D0 ,   6.02559549D-12  ,  3.0D0,  & 
            144.3D0 ,   2.95120943D-11  ,  3.0D0,  & 
            147.0D0 ,   1.00000000D-24  ,  3.0D0,  & 
            150.4D0 ,   9.33254366D-12  ,  3.0D0,  & 
            152.0D0 ,   3.46736869D-12  ,  3.0D0,  & 
            157.3D0 ,   1.17489770D-11  ,  3.0D0,  & 
            158.9D0 ,   2.13796216D-12  ,  3.0D0,  & 
            162.5D0 ,   1.41253747D-11  ,  3.0D0,  & 
            164.9D0 ,   3.16227767D-12  ,  3.0D0,  & 
            167.3D0 ,   8.91250917D-12  ,  3.0D0,  & 
            168.9D0 ,   1.34896287D-12  ,  3.0D0,  & 
            173.0D0 ,   8.91250917D-12  ,  3.0D0,  & 
            175.0D0 ,   1.31825674D-12  ,  3.0D0,  & 
            178.5D0 ,   5.37031822D-12  ,  3.0D0,  & 
            181.0D0 ,   1.34896287D-12  ,  3.0D0,  & 
            183.9D0 ,   4.78630102D-12  ,  3.0D0,  & 
            186.3D0 ,   1.86208719D-12  ,  3.0D0,  & 
            190.2D0 ,   2.39883290D-11  ,  3.0D0,  & 
            192.2D0 ,   2.34422885D-11  ,  3.0D0,  & 
            195.1D0 ,   4.78630036D-11  ,  3.0D0,  & 
            197.0D0 ,   6.76082952D-12  ,  3.0D0,  & 
            200.6D0 ,   1.23026887D-11  ,  3.0D0,  & 
            204.4D0 ,   6.60693440D-12  ,  3.0D0,  & 
            207.2D0 ,   1.12201834D-10  ,  3.0D0,  & 
            209.0D0 ,   5.12861361D-12  ,  3.0D0,  & 
            210.0D0 ,   1.00000000D-24  ,  3.0D0,  & 
            211.0D0 ,   1.00000000D-24  ,  3.0D0,  & 
            222.0D0 ,   1.00000000D-24  ,  3.0D0,  & 
            223.0D0 ,   1.00000000D-24  ,  3.0D0,  & 
            226.1D0 ,   1.00000000D-24  ,  3.0D0,  & 
            227.1D0 ,   1.00000000D-24  ,  3.0D0,  & 
            232.0D0 ,   1.20226443D-12  ,  3.0D0,  & 
            231.0D0 ,   1.00000000D-24  ,  3.0D0,  &
            238.0D0 ,   3.23593651D-13  ,  3.0D0,  &
            237.0D0 ,   1.00000000D-24  ,  0.0D0,  &
            244.0D0 ,   1.00000000D-24  ,  0.0D0,  &
            243.0D0 ,   1.00000000D-24  ,  0.0D0,  &
            247.0D0 ,   1.00000000D-24  ,  0.0D0,  &
            247.0D0 ,   1.00000000D-24  ,  0.0D0,  &
            251.0D0 ,   1.00000000D-24  ,  0.0D0,  &
            254.0D0 ,   1.00000000D-24  ,  0.0D0/
!
!
!  Ionization potentials for first 99 species:
      DATA XI/&
!
!  Element Ionization potentials (eV) 
!           I     II      III     IV       V     VI     VII    VIII
!
            13.595D0,  0.0D0   , 0.0D0 ,  0.0D0   ,  0.0D0  ,  0.0D0  ,  0.0D0  ,  0.0D0 ,&
            24.580D0, 54.400D0,  0.0D0   ,  0.0D0  ,  0.0D0  ,  0.0D0  ,  0.0D0  ,  0.0D0  ,&
             5.392D0, 75.619D0,122.451D0,  0.0D0  ,  0.0D0  ,  0.0D0 ,  0.0D0,  0.0D0 ,&
             9.322D0, 18.206D0,153.850D0,217.713D0,  0.0D0  ,  0.0D0  ,  0.0D0 ,  0.0D0 ,&
             8.296D0, 25.149D0, 37.920D0,259.298D0,340.22D0,  0.0D0 ,  0.0D0  ,  0.0D0  ,&
            11.264D0, 24.376D0, 47.864D0, 64.476D0,391.99D0,489.98D0,  0.0D0 ,  0.0D0  ,&
            14.530D0, 29.593D0, 47.426D0, 77.450D0, 97.86D0,551.93D0,667.03D0,  0.0D0  ,&
            13.614D0, 35.108D0, 54.886D0, 77.394D0,113.87D0,138.08D0,739.11D0,871.39D0,&
            17.418D0, 34.980D0, 62.646D0, 87.140D0,114.21D0,157.12D0,185.14D0,953.6D0 ,&
            21.559D0, 41.070D0, 63.500D0, 97.020D0,126.30D0,157.91D0,207.21D0,239.0D0 ,&
             5.138D0, 47.290D0, 71.650D0, 98.880D0,138.37D0,172.09D0,208.44D0,264.16D0,&
             7.664D0, 15.030D0, 80.120D0,102.290D0,141.23D0,186.49D0,224.9D0 ,265.96D0,& 
             5.984D0, 18.823D0, 28.440D0,119.960D0,153.77D0,190.42D0,241.38D0,284.53D0,& 
             8.151D0, 16.350D0, 33.460D0, 45.140D0,166.73D0,205.11D0,246.41D0,303.07D0, &
            10.484D0, 19.720D0, 30.156D0, 51.354D0, 65.01D0,220.41D0,263.31D0,309.26D0,&
            10.357D0, 23.400D0, 35.000D0, 47.290D0, 72.50D0, 88.03D0,280.99D0,328.8D0 ,&
            12.970D0, 23.800D0, 39.900D0, 53.500D0, 67.80D0, 96.7D0 ,114.27D0,348.D03 ,&
            15.755D0, 27.620D0, 40.900D0, 59.790D0, 75.00D0, 91.3D0 ,124.0D0 ,143.46D0,&
             4.339D0, 31.810D0, 46.000D0, 60.900D0, 82.6D0 , 99.7D0 ,118.0D0 ,155.0D0 ,&
             6.111D0, 11.870D0, 51.210D0, 67.700D0, 84.39D0,109.0D0 ,128.0D0 ,147.0D0 ,&
             6.560D0, 12.890D0, 24.750D0, 73.900D0, 92.0D0 ,111.1D0 ,138.0D0 ,158.7D0 ,&
             6.830D0, 13.630D0, 28.140D0, 43.240D0, 99.8D0 ,120.0D0 ,140.8D0 ,168.5D0 ,&
             6.740D0, 14.200D0, 29.700D0, 48.000D0, 65.2D0 ,128.9D0 ,151.0D0 ,173.7D0 ,&
             6.763D0, 16.490D0, 30.950D0, 49.600D0, 73.0D0 , 90.6D0 ,161.1D0,184.7D0 ,&
             7.432D0, 15.640D0, 33.690D0, 53.000D0, 76.0D0 , 97.0D0 ,119.24D0,196.46D0,&
             7.870D0, 16.183D0, 30.652D0, 54.800D0, 75.0D0 , 99.1D0 ,125.0D0 ,151.06D0,&
             7.860D0, 17.060D0, 33.490D0, 51.300D0, 79.5D0 ,102.0D0 ,129.0D0 ,157.0D0 ,&
             7.635D0, 18.168D0, 35.170D0, 54.900D0, 75.5D0 ,108.0D0 ,133.0D0 ,162.0D0 ,&
             7.726D0, 20.292D0, 36.830D0, 55.200D0, 79.9D0 ,103.0D0 ,139.0D0 ,166.0D0 ,&
             9.394D0, 17.964D0, 39.722D0, 59.400D0, 82.6D0 ,108.0D0 ,134.0D0 ,174.0D0 ,&
             6.000D0,  20.509D0,   30.700D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, & 
             7.89944D0,15.93462D0, 34.058D0, 45.715D0,99.99D0,99.99D0,99.99D0,99.99D0,&    
             9.7887D0, 18.5892D0,  28.351D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             9.750D0,21.500D0, 32.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,  &  
            11.839D0,21.600D0, 35.900D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
            13.995D0,24.559D0, 36.900D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             4.175D0,27.500D0, 40.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             5.692D0,11.026D0, 43.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             6.2171D0,12.2236D0, 20.5244D0,60.607D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             6.63390D0,13.13D0,23.17D0,34.418D0,80.348D0,99.99D0,99.99D0,99.99D0, &   
             6.879D0,14.319D0, 25.039D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,&   
             7.099D0,16.149D0, 27.149D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             7.280D0,15.259D0, 30.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,&   
             7.364D0,16.759D0, 28.460D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             7.460D0,18.070D0, 31.049D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,&   
             8.329D0,19.419D0, 32.920D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             7.574D0,21.480D0, 34.819D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             8.990D0,16.903D0, 37.470D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.784D0,18.860D0, 28.029D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             7.342D0,14.627D0, 30.490D0,72.3D0,99.99D0,99.99D0,99.99D0,99.99D0,  & 
             8.639D0,16.500D0, 25.299D0,44.2D0,55.7D0,99.99D0,99.99D0,99.99D0,   &
             9.0096D0,18.600D0, 27.96D0, 37.4D0,58.7D0,99.99D0,99.99D0,99.99D0,   &
            10.454D0,19.090D0, 32.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
            12.12984D0,20.975D0,31.05D0,45.D0,54.14D0,99.99D0,99.99D0,99.99D0,   &
             3.893D0,25.100D0, 35.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.210D0,10.000D0, 37.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.580D0,11.060D0, 19.169D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.650D0,10.850D0, 20.080D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,  & 
             5.419D0,10.550D0, 23.200D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.490D0,10.730D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.550D0,10.899D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.629D0,11.069D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.680D0,11.250D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             6.159D0,12.100D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.849D0,11.519D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.930D0,11.670D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             6.020D0,11.800D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             6.099D0,11.930D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             6.180D0,12.050D0, 23.700D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             6.250D0,12.170D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             6.099D0,13.899D0, 19.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             7.000D0,14.899D0, 23.299D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             7.879D0,16.200D0, 24.00D00, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             7.86404D0,17.700D0, 25.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             7.870D0,16.600D0, 26.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,  & 
             8.500D0,17.000D0, 27.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             9.100D0,20.000D0, 28.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             8.95868D0,18.563D0,33.227D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             9.220D0,20.500D0, 30.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,  & 
            10.430D0,18.750D0, 34.200D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,  & 
             6.10829D0,20.4283D0,29.852D0,50.72D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             7.416684D0,15.0325D0,31.9373D0,42.33D0,69.0D0,99.99D0,99.99D0,99.99D0, &  
             7.285519D0,16.679D0, 25.563D0,45.32D0,56.0D0,88.D0,99.99D0,99.99D0,   &
             8.430D0,19.000D0, 27.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,  & 
             9.300D0,20.000D0, 29.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0,  & 
            10.745D0,20.000D0, 30.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             4.000D0,22.000D0, 33.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             5.276D0,10.144D0, 34.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             6.900D0,12.100D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &  
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &   
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0, &    
             6.000D0,12.000D0, 20.000D0, 99.99D0,99.99D0,99.99D0,99.99D0,99.99D0/
!
	DO i=1,matom

	  adyp(i)=dyp(i)

	  DO j=1,8

	    axi(j,i)=xi(j,i)
	
	  ENDDO

	  DO j=1,3

	    ad(j,i)=d(j,i)
	
	  ENDDO

	ENDDO

END SUBROUTINE state0

SUBROUTINE eldat(ichemc,iat,iion,dyp,d,xi,abhyd,abhel,&
     wm,nion,abund,hmc,zip,nline,ielnd,necod)
!		extract data for our particular element and  
!	change abundances
!	input: 
!		ichem!switch to change built in chemical composition
!		iat-atomic number
!		iion-ionization degree
!		dyp,d,xi 
!		ielnd-switch to calculate electron n.e. and READ 3.column
!	output: 
!		nion - number of ions of our element considered 
!		abund - abundance, N(EL)/N(H)
!    	abhyd - H abundance N/N(TOT)
!		abhel - He abundance, N(EL)/N(H) 
! 		hmc - mass number
!		wm -mean molecular weight= sum ai*mi/sum ai
!		zip - first 8 ionization potentials [EV]
 USE param
 IMPLICIT NONE

 CHARACTER(LEN=4),DIMENSION(matom)::dyp
 CHARACTER(LEN=4),DIMENSION(mline)::elem
 INTEGER,DIMENSION(mline)::nion,iat,iion
 INTEGER,DIMENSION(matom)::necod
 REAL(KIND=8),DIMENSION(3,matom)::d
 REAL(KIND=8),DIMENSION(8,matom)::xi
 REAL(KIND=8),DIMENSION(mline)::abund,hmc
 REAL(KIND=8),DIMENSION(mion-1,mline)::zip
 REAL(KIND=8)::abhyd,abhel,abii
 REAL(KIND=8)::wm
 INTEGER::i,j,nichem,nline,ielnd,ii,ichemc



	IF(ielnd==1)ichemc=1
	IF(ichemc==1)THEN
	  	OPEN(7,file='abundances',status='old')
          	READ(7,*)nichem
	  		DO i=1,nichem
	  		  IF(ielnd==1)THEN	
          		   	READ(7,*)ii,abii,necod(i)
	  		  ELSE
              		  	 READ(7,*)ii,abii
	    		  ENDIF
	    			d(2,ii)=abii
	  		ENDDO
	  	CLOSE(7)	
	ENDIF

	abhyd=0.d0
	wm=0.d0
	DO i=1,matom
	  abhyd=abhyd+d(2,i)
	  wm=wm+d(1,i)*d(2,i)
	ENDDO
	abhyd=1.d0/abhyd
	wm=wm*abhyd
	abhel=d(2,2)
  		WRITE(2,*)' lines of these elements are considered:',nline
	DO i=1,nline
  	nion(i)=int(d(3,iat(i))+0.5d0)  
  		IF(iion(i)>nion(i).or.iion(i)>mion)THEN
    			WRITE(*,*)i,'-th sp. line cannot be calculated,',&
			    ' error iion>nion'
	                RETURN
	        ENDIF
  		IF(iion(i)>5)THEN
    			WRITE(*,*)i,'-th sp. line, partition function not available',&
		     ', p.f. will be set=1, warning'
  		ENDIF
	  abund(i)=d(2,iat(i))
	  hmc(i)=d(1,iat(i))
		  DO j=1,8
		    zip(j,i)=xi(j,iat(i))	
		  ENDDO
	  elem(i)=dyp(iat(i))
	  WRITE(2,'(a4,i3,f8.4,i3,e9.3)')elem(i),iat(i),hmc(i),nion(i),abund(i)
	  ENDDO
	WRITE(2,'(f5.3,f5.3,e9.3)')abhyd,abhel,wm

END SUBROUTINE eldat


 SUBROUTINE lindat(iat,iion,wlab,elo,eup,glo,gr0,gs0,gw0,bij,&
     nline)
! 		Line data as in Kurucz linelist or in the SYNSPEC code
!    dll -wavelength [nm]
!    cod -element.ion cod, e.g. 26.02. It is interpreted as:
!      26=atomic number=iron, 02=2xtimes ionized i.e. FeIII line
!    gf -log_10 (gf) 
!    elo,eup -energy of the lower and upper level in [1/cm]
!    qlo -quantum number -J of the lower level[=>stat.weight=2*J+1] 
!    qup -quantum number -J of the upper level[=>stat.weight=2*J+1] 
!    gr0,gs0,gw0-radiative, Stark, Van der Waals damping constants
! 	iat - element atomic number
! 	wlab - lambda [A] 
!freq0 -frequency [1/s]
! 	stio-ionization degree of the ion [1.-neutral,...] 
! 	bij-transition probability [1/s]=Einstein coef. per unit solid
!	angle
 USE param
 IMPLICIT NONE

 INTEGER,DIMENSION(mline)::iat,iion
 INTEGER::i,nline,io
 REAL(KIND=8),DIMENSION(mline)::wlab,elo,eup,glo,gr0,gs0,gw0,bij
 REAL(KIND=8)::stio,dll,cod,gf,qlo,qups,qup

	OPEN(8,FILE='line.dat',STATUS='old')
	i=1
	DO
	READ(8,*,err=20,end=20)dll,cod,gf,elo(i),qlo,eup(i),qup,&
       gr0(i),gs0(i),gw0(i)
	wlab(i)=dll*10.d0
	iat(i)=int(cod)
	stio=(cod-int(cod))*100.d0+1.d0 
	iion(i)=int(stio+0.5d0)
	glo(i)=2.d0*qlo+1.d0
	bij(i)=exp(2.302585d0*gf)/glo(i)*wlab(i)/7.484d-7
	i=i+1
	ENDDO
20	nline=i-1
	CLOSE(8)
	WRITE(*,*)' No. of sp. lines READ from line.dat=',nline

 END SUBROUTINE lindat



 SUBROUTINE medop(nivac,alam1,alamn,alam,&
     ophbf,ophbf1,ophbf2,ophff,ophff1,ophff2,&
     ophrs,ophrs1,ophrs2)	
!	calculates the continuum opacities at the particular wavelength
!	via a simple linear interpolation in between the boundaries
!	input:  ndim3,nivac,alam1,alamn
!		alam -actual value of lambda[A]
!		ophbf1,ophbf2,ophff1,ophff2,ophrs1,ophrs2
!	output: ophbf,ophff,ophrs
 USE param
 IMPLICIT NONE

 REAL(KIND=8),DIMENSION(ndim3)::ophbf,ophbf1,ophbf2,& 
			ophff,ophff1,ophff2,ophrs,ophrs1,ophrs2
 REAL(KIND=8)::falam,alam1,alamn,alam    				
 INTEGER::nivac,kk

 falam=(alam-alam1)/(alamn-alam1)

 DO kk=1,nivac
	  ophbf(kk)=(ophbf2(kk)-ophbf1(kk))*falam+ophbf1(kk)
	  ophff(kk)=(ophff2(kk)-ophff1(kk))*falam+ophff1(kk)
	  ophrs(kk)=(ophrs2(kk)-ophrs1(kk))*falam+ophrs1(kk)
 ENDDO
 
 END SUBROUTINE medop

!----------------------------------------------------------------------
 SUBROUTINE fend(temp,ekonc,entot,zip,stavs,nion,endf,endfp)
!subroutine calculates the electron number density function
!and its first parcial derivative (after electron num.dens.) 
!for one chemical element (valid also for H without H-)
!input:  temp-temperature
!	ekon!electron number density
!	entot-number density of particular chemical element
!	zip-ionization potentials
!	stavs-partition functions
!	nion-the highest ion of the element considered
!output:
!	endf-electron number density function
!	endfp-  \parc endf / \parc Ne
!notation:
!    rr(j)- population of the J-th ion/total element population
!rt(j)= Ne*N(j)/N(j-1)
!rt1(j)= Ne**(j-1)*N(j)/N(1)
!sahan	-calling function 
!	(Saha equation without el.num. dens. term)
!
 USE param
 IMPLICIT NONE

 REAL(KIND=8),DIMENSION(mion-1)::zip
 REAL(KIND=8),DIMENSION(mion)::stavs,rt,rt1
 REAL(KIND=8)::temp,ekonc,entot,endf,endfp
 REAL(KIND=8)::sahan,b,a,pom,rr,rrp
 INTEGER::nion
 INTEGER:: j,nj1

	IF(nion<2)THEN
	  WRITE(*,*)'error in subroutine: fend'
	  RETURN
	ENDIF
	rt1(1)=1.d0
	a=ekonc**(nion)
	b=dble(nion)*ekonc**(nion-1)
	DO j=2,nion
	  rt(j)=sahan(temp,zip(j-1),stavs(j-1),stavs(j))
          rt1(j)=rt1(j-1)*rt(j)
	  nj1=nion-j+1
	  a=a+ekonc**nj1*rt1(j)
	  b=b+dble(nj1)*ekonc**(nion-j)*rt1(j)
	ENDDO
	endf=0.d0
	endfp=0.d0
	DO j=2,nion
	  nj1=nion-j+1
	  rr=ekonc**nj1*rt1(j)/a
	  rrp=(dble(nj1)*ekonc**(nion-j)*rt1(j)-rr*b)/a
	  pom=dble(j-1)*entot
	  endf=endf+pom*rr
	  endfp=endfp+pom*rrp
	ENDDO

 END SUBROUTINE fend
!----------------------------------------------------------------------
 SUBROUTINE fendh(temp,ekonc,entot,zip,stavs,endfh,endfhp)
!subroutine calculates the electron number density function
!and its first parcial derivative (after electron num.dens.) 
!for Hydrogen including H-
!input:  temp-temperature
!	ekon!electron number density
!	entot-number density of particular chemical element
!	zip-ionization potentials
!	stavs-partition functions
!output:
!	endfh-electron number density function
!	endfhp-  \parc endfh / \parc Ne
!notation:
!    rr(j)- population of the J-th ion/total element population
!rt(j)= Ne*N(j)/N(j-1)
!sahan	-calling function 
!	(Saha equation without el.num. dens. term)
!
 USE param
 IMPLICIT NONE

 REAL(KIND=8),DIMENSION(mion-1)::zip
 REAL(KIND=8),DIMENSION(mion)::stavs
 REAL(KIND=8),PARAMETER::    pothm=0.7552D0
 REAL(KIND=8),PARAMETER::  stavhm=1.d0
 REAL(KIND=8)::temp,ekonc,entot,endfh,endfhp
 REAL(KIND=8)::rt2,rt3,bot,rr1,rr3,rr1p,rr3p,sahan
!H- ionization potential	and partition function
	 endfhp=0.d0
	 endfh=0.d0
	rt2=sahan(temp,pothm,stavhm,stavs(1))
	rt3=sahan(temp,zip(1),stavs(1),stavs(2))	
	bot=ekonc*ekonc+ekonc*rt2+rt3*rt2
	rr1=ekonc*ekonc/bot
!rr2=ekonc*rt2/bot
	rr3=rt3*rt2/bot
        endfh=(rr3-rr1)*entot
	rr1p=(ekonc**2*rt2+2.d0*ekonc*rt3*rt2)/bot**2
	rr3p=-rt3*rt2*(2.d0*ekonc+rt2)/bot**2
        endfhp=(rr3p-rr1p)*entot
 
 END SUBROUTINE	fendh
!----------------------------------------------------------------------

 SUBROUTINE elnd(d,xi,necod,wm,abhyd,ftemp,fdens,fne,&
     nbod1,nbod2,nbod3,ane0)
!    calculates electron number density from the temperature, density
!and abundances using Newton-Raphson iteration method 
!input:  
!	d(2,*) -abundance with respect to H
!	xi -first 8 ionization potentials
!	necod=1 -code for taking the element into account
!	wm -mean molecular weight in [hjed]
!	abhyd -H abundance relative to total elem.num.dens.
!	ftemp,fdens -temperature,density
!	nbod1,nbod2,nbod3 -number of xyz grid points
!	denvac,dcut1 -density limits for vacuum and opaque obj.
!	ane0 -fake electron num.dens beyond the limits
!output:
!	fne-electron number density
!notation:

 USE param
 USE constants
 USE pfdwor
 USE sun

 IMPLICIT NONE

 INTEGER,DIMENSION(matom)::necod
 INTEGER,PARAMETER::niter=35
 INTEGER::nbod1,nbod2,nbod3
 INTEGER::l,m,n,nion,j,iat,i
 REAL(KIND=8),DIMENSION(mion-1)::zip
 REAL(KIND=8),DIMENSION(mion)::stavs
 REAL(KIND=8),DIMENSION(5)::u
 REAL(KIND=8),DIMENSION(3,matom)::d
 REAL(KIND=8),DIMENSION(8,matom)::xi
 REAL(KIND=8),DIMENSION(ndim1,ndim2,ndim3)::fdens,fne,ftemp
 REAL(KIND=8)::wm
 REAL(KIND=8)::abhyd,ane0
 REAL(KIND=8)::wmhj,pom,theta,endf,endfp,temp,ekonc,entot,&
		dx,endfi,endfpi,eplow
 REAL(KIND=8),PARAMETER::xacc=1.d-3

 wmhj=hjed*wm
	
 DO l=1,nbod1
	DO m=1,nbod2
		DO n=1,nbod3
	  		IF(fdens(l,m,n)<denvac.or.fdens(l,m,n)>dcut1)THEN
	    		fne(l,m,n)=ane0
	    		CYCLE
	  		ENDIF
	  		temp=ftemp(l,m,n)
	  		ekonc=fdens(l,m,n)/wmhj
	  		pom=ekonc*abhyd
 	  		theta=5039.77d0/temp
!  	iteration cycle
	  		DO i=1,niter
 	    			CALL potlow(temp,ekonc,eplow)
	    			endf=0.d0
			        endfp=0.d0
		                DO iat=1,matom
				      IF(necod(iat)/=1)CYCLE
				      nion=int(d(3,iat)+0.5d0)
				              DO j=1,8
				  	        zip(j)=xi(j,iat)
					      ENDDO	
!     PARTFN is in 'pfdwor.inc'
!   mind, for temperatures over a few 1.d5 deg. partition f.
!   might DO strange things 
 	      				      CALL partfn(iat,theta,u,eplow) 
				 	      DO j=1,nion
 	        				IF(j<=5)THEN
 	          				stavs(j)=u(j)
 	        				ELSE
 	          				stavs(j)=1.d0
 	        				ENDIF
	      				      ENDDO
	      				      entot=pom*d(2,iat)
					      IF(iat==1)THEN	
	        			      CALL fendh(temp,ekonc,entot,zip,stavs,endfi,endfpi)
	      				      ELSE
               				      CALL fend(temp,ekonc,entot,zip,stavs,nion,endfi,endfpi)
              				      ENDIF
	      				      endf=endf+endfi
					      endfp=endfp+endfpi
	 			ENDDO
!  	heart of iteration
	    			dx=(endf-ekonc)/(endfp-1.d0)
	    			IF((ekonc-dx)<1.d-3*ekonc)THEN
	      			ekonc=1.d-3*ekonc
	    			ELSE
	      			ekonc=ekonc-dx
	    			ENDIF
	    			IF(abs(dx/ekonc)<xacc)EXIT
	    			IF(i==niter)THEN
	      				WRITE(*,*)'slow or no convergence in subroutine: elnd'
	      				RETURN
	    			ENDIF
		ENDDO
	  	fne(l,m,n)=ekonc
	ENDDO
    ENDDO
 ENDDO
 END SUBROUTINE elnd

!-----------------------------------------------------------------------
 SUBROUTINE trans(x,y,z,trmt)
!	Assumes coordinate system e1,e2,e3.
!	Defines new coordinate system ep1,ep2,ep3 such that ep(3)
!	is along the chosen direction ep3=(x,y,z).
!	Optionaly defines ep2 axes such that ep2 is perpendicular 
!	to e3,ep3 i.d. ep2=e3xep3.
!	Calculates the transformation matrix trmt between 
!	the old and new rotated coordinates such that the new 
!	coordinates (Vp) are related to old coordinates (V):
!	Vp=trmt \times V.  
!	input: x,y,z   
! 	output:trmt
	IMPLICIT NONE
	REAL(KIND=8),DIMENSION(3,3)::trmt,e,ep
	REAL(KIND=8)::x,y,z,pom1,pom2,pom3
	INTEGER::i,j
	e(1,1)=1.d0
	e(1,2)=0.d0
	e(1,3)=0.d0
	e(2,1)=0.d0
	e(2,2)=1.d0
	e(2,3)=0.d0
	e(3,1)=0.d0
	e(3,2)=0.d0
	e(3,3)=1.d0
	pom1=x*x+y*y
	pom2=sqrt(pom1)
	pom3=sqrt(x*x*z*z+y*y*z*z+pom1*pom1)
	IF(pom2<1.d-30)THEN
	  ep(1,1)=1.d0
	  ep(1,2)=0.d0
	  ep(1,3)=0.d0
          ep(2,1)=0.d0
          ep(2,2)=1.d0
          ep(2,3)=0.d0
 	ELSE
	  ep(1,1)=x*z/pom3
	  ep(1,2)=y*z/pom3
	  ep(1,3)=-pom1/pom3
	  ep(2,1)=-y/pom2
	  ep(2,2)=x/pom2
	  ep(2,3)=0.d0
	ENDIF
	ep(3,1)=x
	ep(3,2)=y
	ep(3,3)=z
	DO i=1,3
	  DO j=1,3
	    trmt(i,j)=ep(i,1)*e(j,1)+ep(i,2)*e(j,2)+ep(i,3)*e(j,3)
	  ENDDO
	ENDDO  
 END SUBROUTINE trans

 SUBROUTINE ophyd(nbod,temp,ekonc,akonc,hi,abhyd,&
     	          ophbf,ophff,ophrs,hipf,wlab) 
!	calculates HI bound-free, free-free, Rayleigh opacity
!	input: nbod,temp,ekonc,akonc,hi,abhyd,hipf,wlab
!	output: ophbf,ophff,ophrs
 USE param
 USE constants

 IMPLICIT NONE

 INTEGER::i,j,nbod,inn 
 REAL(KIND=8),DIMENSION(ndim)::temp,ekonc,hi,ophbf,ophff,ophrs
 REAL(KIND=8),DIMENSION(ndim)::akonc
 REAL(KIND=8),DIMENSION(ndim,2)::hipf
 REAL(KIND=8),PARAMETER:: frray=2.463d15
 REAL(KIND=8)::frjump,tk,secor,en,chi,en3,gau,xe,gau3,hii,abhyd,freq0,pot,wlab,gaunt,gfree
  
 freq0=clight/wlab*1.d8
 pot=clight*plank*rydbh

	DO i=1,16

	  frjump=clight*rydbh*1.d0/dble(i*i)

	  IF(frjump<freq0)EXIT 

	ENDDO	

	inn=i 
 
	DO i=1,nbod

!		HI bound-free opacity
	  hii=akonc(i)*abhyd-hi(i)
	  tk=bol*temp(i)
	  secor=(1.d0-exp(-plank*freq0/tk))


	  IF(inn<16)THEN
	    ophbf(i)=0.d0

	    DO j=1,3
	      en=dble(inn+j-1)
	      chi=pot*(1.d0-1.d0/(en*en))
	      en3=en*en*en
	      gau=gaunt(inn+j-1,freq0)
	      ophbf(i)=ophbf(i)+gau/en3*exp(-chi/tk)
	    ENDDO

	    en=dble(inn+3)
	    chi=pot*(1.d0-1.d0/(en*en))
	    ophbf(i)=tk/pot*0.5d0*(exp(-chi/tk)-exp(-pot/tk))&
            +ophbf(i)
!	    ophbf(i)=ophbf(i)*1.044d-26*wlab*wlab*wlab*2.d0/hipf(i,1)
!  &        *secor*hi(i)
	    ophbf(i)=ophbf(i)*2.8154d29/freq0/freq0/freq0*2.d0&
            /hipf(i,1)*secor*hi(i)

	  ELSE
	    ophbf(i)=0.d0
	  ENDIF

!            HI free-free opacity
	  gau3=gfree(temp(i),freq0)
	  ophff(i)=3.69d8*gau3/(freq0*freq0*freq0*sqrt(temp(i)))&
          *ekonc(i)*hii*secor
!            HI Rayleigh scattering on neutral hydrogen
!		Kurucz R.L., 1970, SAO Special Report 309
	  xe=1.d-16*(min(freq0,frray)/clight)**2
	  ophrs(i)=(5.799d-13+(1.422d-6+2.784d0*xe)*xe)*xe*xe
	  ophrs(i)=ophrs(i)*2.d0*hi(i)/hipf(i,1)

	ENDDO

 END SUBROUTINE ophyd
!--------------------------------------------------------------
 SUBROUTINE hihei(ipart,nbod,temp,ekonc,akonc,hipf,hi,hneg,hei,&
            abhyd,abhel)
!	calculates HI,HeI number densities and partition functions
!	input: ipart,nbod,temp,ekonc,akonc
!    	abhyd - H abundance N/N(TOT)
!		abhel - He abundance, N(EL)/N(H) 
!	output: hipf - HI partition function
!		hi,hneg,hei - HI, H-, HeI number densities
 USE param
 IMPLICIT NONE

 INTEGER::i,j,nbod,ipart
 REAL(KIND=8),DIMENSION(ndim,2)::hipf
 REAL(KIND=8),DIMENSION(ndim,3)::hepf
 REAL(KIND=8),DIMENSION(ndim)::hi,hneg,hei,temp,ekonc
 REAL(KIND=8),DIMENSION(ndim)::akonc
 REAL(KIND=8),DIMENSION(6,2)::stahi
 REAL(KIND=8),DIMENSION(6,3)::stahei
 REAL(KIND=8)::rt2,rt3,rr1,rr2,bot,sala,abhyd,abhel
 REAL(KIND=8)::sahan,sah
 REAL(KIND=8),PARAMETER::pothm=0.7552D0,stavhm=1.d0,pothi=13.598D0

!		POLYNOMIAL PARTITION FUNCTION COEFFICIENTS FOR 
!		HI,HII   A    HEI,HEII,HEIII 
!		AFTER IRWIN, 1981, ApJS 45,621	
        DATA stahi/-2.61655891D2,1.63428326D2,-4.06133526D1,5.03282928D0,&
     -3.10998364D-1,7.66654594D-3,0.0D0,0.0D0,0.0D0,0.0D0,0.0D0,&
     0.0D0/
        DATA stahei/-3.76575219D-1,2.33951687D-1,-5.79755525D-2,&
     7.16333160D-3,-4.41302573D-4,1.08442997D-5,6.93147179D-1,&
     9.29636701D-10,-2.30049742D-10,2.83829746D-11,-1.74590774D-12,&
     4.28355287D-14,0.0D0,0.0D0,0.0D0,0.0D0,0.0D0,0.0D0/

	IF(ipart==1)THEN
          CALL pfdwir(nbod,2,2,1,temp,ekonc,hipf)
          CALL pfdwir(nbod,3,3,2,temp,ekonc,hepf)
	ELSE
	  DO i=1,nbod
	    sala=log(temp(i))
!		HIPF - H PARTITION FUNCTION
	    DO j=1,2
	      hipf(i,j)=exp(stahi(1,j)+stahi(2,j)*sala&
             +stahi(3,j)*sala*sala+stahi(4,j)*sala**3&
             +stahi(5,j)*sala**4+stahi(6,j)*sala**5)
	    ENDDO
!            HEPF - HE PARTITION FUNCTION
	    DO j=1,3
	      hepf(i,j)=exp(stahei(1,j)+stahei(2,j)*sala+stahei(3,j)&
     	      *sala*sala+stahei(4,j)*sala**3+stahei(5,j)*sala**4&
     	      +stahei(6,j)*sala**5)
	    ENDDO
	  ENDDO
	ENDIF

!		HNEG - H- NUMBER DENSITY
!            HI - HI NUMBER DENSITY
!            HEI - HeI NUMBER DENSITY	

  DO i=1,nbod
    rt2=sahan(temp(i),pothm,stavhm,hipf(i,1))
    rt3=sahan(temp(i),pothi,hipf(i,1),hipf(i,2))
    bot=ekonc(i)*ekonc(i)+ekonc(i)*rt2+rt3*rt2
    rr1=ekonc(i)*ekonc(i)/bot
    rr2=ekonc(i)*rt2/bot
!   rr3=rt3*rt2/bot
    hneg(i)=rr1*akonc(i)*abhyd
    hi(i)=rr2*akonc(i)*abhyd
!   HI(I)=1.d0/(1.d0+SAH(TEMP(I),EKONC(I),13.598,HIPF(i,1)
!  P	  ,HIPF(i,2)))*AKONC(I)*ABHYD
    hei(i)=1.d0/(1.d0+sah(temp(i),ekonc(i),24.587D0,hepf(i,1),&
    hepf(i,2))*(1.d0+sah(temp(i),ekonc(i),54.416D0,hepf(i,2),&
    hepf(i,3))))*akonc(i)*abhel*abhyd
  ENDDO

 END SUBROUTINE hihei

 SUBROUTINE hunt(xx,n,x,jlo)
!taken from Numerical recipes
!given field XX and a value X subroutine returns JLO such that
!XX(JLO)<X<XX(JLO+1)
 IMPLICIT NONE
	
 REAL(KIND=8),DIMENSION(n):: xx
 REAL(KIND=8)::x
 INTEGER::n,inc,jhi,jm,jlo
 LOGICAL::ascnd

 !n=size(xx) 
 
 ascnd=(xx(n)>=xx(1))

  IF(jlo<=0.or.jlo>n)THEN
        jlo=0
        jhi=n+1
  ELSE 
 	inc=1	
  IF(x>=xx(jlo).eqv.ascnd)THEN
	DO
	jhi=jlo+inc
	IF(jhi>n)THEN
	jhi=n+1
	EXIT
	ELSE 
	IF(x<xx(jhi).eqv.ascnd) EXIT
          jlo=jhi
          inc=inc+inc
	ENDIF
	ENDDO
  ELSE
         jhi=jlo
	DO
	       jlo=jhi-inc
        IF(jlo<1)THEN
	                jlo=0
	EXIT
        ELSE
	IF(x>=xx(jlo).eqv.ascnd)EXIT
	   jhi=jlo
	   inc=inc+inc
	ENDIF
	ENDDO
  ENDIF
  ENDIF
  DO
     IF(jhi-jlo<=1)THEN
	IF(x==xx(n)) jlo=n-1
	IF(x==xx(1)) jlo=1
	exit
     ELSE
      jm=(jhi+jlo)/2
     	 IF(x>=xx(jm).eqv.ascnd)THEN
    	    jlo=jm
      	  ELSE
             jhi=jm
          ENDIF
      ENDIF
  ENDDO
 END SUBROUTINE hunt


 SUBROUTINE rotat1(theta,alpha,temp0,ane0,&
            vxst,vyst,vzst,vxstr,vystr,vzstr,&
            xcp,ycp,zcp,xcpr,ycpr,zcpr,&
            vxcp,vycp,vzcp,vxcpr,vycpr,vzcpr,&
            nbodf1,nbodf2,nbodf3,nbod1,nbod2,nbod3,&
            ar,at,az,far,fat,faz,&
            atemp,apress,adens,ane,avr,avt,avz,avtrb,&
            ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb)
!to transform (rotate) the line of sight grid (x,y,z) to the body
!frozen grid (x'',y'',z'') and interpolate state quantities from 
!the rotating body frozen coordinates (RBFC) to the line of sight 
!coordinates (LSC) and transform the velocity field vectors 
!from RBFC to LSC
!input: 	theta,alpha,dcut1
!	temp0,ane0 - out of grid (sphere) reasonable values
!            vxst,vyst,vzst - net velocity of the primary
!	xcp,ycp,zcp - position of the secondary
!	vxcp,vycp,vzcp - velocity of the secondary
!	ar,at,az -coord. in which the model is necessary (LSC)
!	far,fat,faz -coord. in which the model is defined (RBFC)
!	ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb -model
!output:
!	atemp,apress,adens,ane,avr,avt,avz,avtrb- rotated model
!            vxstr,vystr,vzstr- rotated net velocity of the primary
!	xcpr,ycpr,zcpr -rotated position of the secondary
!	vxcpr,vycpr,vzcpr - rotated velocity of the secondary 
!parameter (ndim1=100,ndim2=100,ndim3=200)
 USE sun
 USE param

 IMPLICIT NONE

 INTEGER::i,j,k,nbod1,nbod2,nbod3,ii1,ii,jj1,jj,kk1,kk,nbodf1,nbodf2,nbodf3
 REAL(KIND=8),DIMENSION(ndim1)::ar,far
 REAL(KIND=8),DIMENSION(ndim2)::at,fat
 REAL(KIND=8),DIMENSION(ndim3)::az,faz
 REAL(KIND=8),DIMENSION(ndim1,ndim2,ndim3)::atemp,apress,adens,ane,avr,avt,&
                         avz,avtrb,ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb
 REAL(KIND=8)::theta,alpha,temp0,ane0,vxst,vyst,vzst,vxstr,&
	vystr,vzstr,xcp,ycp,zcp,xcpr,ycpr,zcpr,vxcp,vycp,vzcp,vxcpr,&
	vycpr,vzcpr,ctheta,stheta,calpha,salpha,fx,fy,fz,&
	dx,dy,dz,dist,tt,uu,vv,tt1,uu1,vv1
 REAL(KIND=8)::dist1		

 ctheta=cos(theta)
 stheta=sin(theta)
 calpha=cos(alpha)
 salpha=sin(alpha)
!initialization of first values to ii,jj,kk
 ii=1
 jj=1
 kk=1
 DO i=1,nbod1
 DO j=1,nbod2
 DO k=1,nbod3
!  calculation of RBFC coordinates corresponding to the grid 
!  points of the line of sight grid 

 CALL rotz(ar(i),at(j),az(k),fx,fy,fz,&
      ctheta,stheta,calpha,salpha)

!  	interpolation from body frozen grid to required 
!  line of sight grid  :
   IF(fx>far(1).and.fx<far(nbodf1).and.&
      fy>fat(1).and.fy<fat(nbodf2).and.&
      fz>faz(1).and.fz<faz(nbodf3))THEN
	    CALL hunt(far,nbodf1,fx,ii)    
     IF(ii<1.or.ii>nbodf1)THEN
      IF(ii==nbodf1.and.fx==far(nbodf1))THEN
	ii=nbodf1-1
      ELSE		
        WRITE(*,'(a,a,4i4)')' compiler/maschine dependent error',&
     ' -out of grid at ii,i,j,k',ii,i,j,k
      ENDIF
    ENDIF

	    CALL hunt(fat,nbodf2,fy,jj)       

    IF(jj<1.or.jj>nbodf2)THEN
      IF(jj==nbodf2.and.fy==fat(nbodf2))THEN
	jj=nbodf2-1
      ELSE		
       WRITE(*,'(a,a,4i4)')' compiler/maschine dependent error',&
     ' -out of grid at jj,i,j,k',jj,i,j,k
      ENDIF
    ENDIF

	    CALL hunt(faz,nbodf3,fz,kk)		
    IF(kk<1.or.kk>nbodf3)THEN
      IF(kk==nbodf3.and.fz==faz(nbodf3))THEN
       	kk=nbodf3-1
      ELSE		
        WRITE(*,'(a,a,4i4)')' compiler/maschine dependent error',&
     ' -out of grid at kk,i,j,k',kk,i,j,k
      ENDIF
    ENDIF

!	abnormal interpolation-nearest neighbour value
!	in the untransparent object or on its edge.
!	density of an untransparent object dens>dcut1
    IF(fdens(ii,jj,kk)>dcut1&
      .or.fdens(ii,jj,kk+1)>dcut1&
      .or.fdens(ii,jj+1,kk)>dcut1&
      .or.fdens(ii,jj+1,kk+1)>dcut1&
      .or.fdens(ii+1,jj,kk)>dcut1&
      .or.fdens(ii+1,jj,kk+1)>dcut1&
      .or.fdens(ii+1,jj+1,kk)>dcut1&
      .or.fdens(ii+1,jj+1,kk+1)>dcut1)THEN

     dist1=1.d60
      DO ii1=ii,ii+1
      DO jj1=jj,jj+1
      DO kk1=kk,kk+1	
	dx=fx-far(ii1)
        dy=fy-fat(jj1)
        dz=fz-faz(kk1)
	dist=dx*dx+dy*dy+dz*dz
	IF(dist<dist1)THEN
          atemp(i,j,k)=ftemp(ii1,jj1,kk1)
          apress(i,j,k)=fpress(ii1,jj1,kk1)
          adens(i,j,k)=fdens(ii1,jj1,kk1)
          ane(i,j,k)=fne(ii1,jj1,kk1)
          avr(i,j,k)=fvr(ii1,jj1,kk1)
          avt(i,j,k)=fvt(ii1,jj1,kk1)
          avz(i,j,k)=fvz(ii1,jj1,kk1)
          avtrb(i,j,k)=fvtrb(ii1,jj1,kk1)
	  dist1=dist
        ENDIF

      ENDDO
      ENDDO
      ENDDO
      CYCLE  
    ENDIF
!	normal interpolation
    tt=(fx-far(ii))/(far(ii+1)-far(ii))
    uu=(fy-fat(jj))/(fat(jj+1)-fat(jj))
    vv=(fz-faz(kk))/(faz(kk+1)-faz(kk))
    tt1=1.d0-tt
    uu1=1.d0-uu
    vv1=1.d0-vv	
    atemp(i,j,k)=tt1*uu1*vv1*ftemp(ii,jj,kk)&
             +tt*uu1*vv1*ftemp(ii+1,jj,kk)&
     	      +tt*uu*vv1*ftemp(ii+1,jj+1,kk)&
     	      +tt1*uu*vv1*ftemp(ii,jj+1,kk)&
            +tt1*uu1*vv*ftemp(ii,jj,kk+1)&
            +tt*uu1*vv*ftemp(ii+1,jj,kk+1)&
            +tt*uu*vv*ftemp(ii+1,jj+1,kk+1)&
            +tt1*uu*vv*ftemp(ii,jj+1,kk+1)
    apress(i,j,k)=tt1*uu1*vv1*fpress(ii,jj,kk)&
            +tt*uu1*vv1*fpress(ii+1,jj,kk)&
     	      +tt*uu*vv1*fpress(ii+1,jj+1,kk)&
     	      +tt1*uu*vv1*fpress(ii,jj+1,kk)&
            +tt1*uu1*vv*fpress(ii,jj,kk+1)&
            +tt*uu1*vv*fpress(ii+1,jj,kk+1)&
            +tt*uu*vv*fpress(ii+1,jj+1,kk+1)&
            +tt1*uu*vv*fpress(ii,jj+1,kk+1)
    adens(i,j,k)=tt1*uu1*vv1*fdens(ii,jj,kk)&
            +tt*uu1*vv1*fdens(ii+1,jj,kk)&
     	      +tt*uu*vv1*fdens(ii+1,jj+1,kk)&
     	      +tt1*uu*vv1*fdens(ii,jj+1,kk)&
            +tt1*uu1*vv*fdens(ii,jj,kk+1)&
            +tt*uu1*vv*fdens(ii+1,jj,kk+1)&
            +tt*uu*vv*fdens(ii+1,jj+1,kk+1)&
            +tt1*uu*vv*fdens(ii,jj+1,kk+1)
    ane(i,j,k)=tt1*uu1*vv1*fne(ii,jj,kk)&
            +tt*uu1*vv1*fne(ii+1,jj,kk)&
     	      +tt*uu*vv1*fne(ii+1,jj+1,kk)&
     	      +tt1*uu*vv1*fne(ii,jj+1,kk)&
            +tt1*uu1*vv*fne(ii,jj,kk+1)&
            +tt*uu1*vv*fne(ii+1,jj,kk+1)&
            +tt*uu*vv*fne(ii+1,jj+1,kk+1)&
            +tt1*uu*vv*fne(ii,jj+1,kk+1)
    avr(i,j,k)=tt1*uu1*vv1*fvr(ii,jj,kk)&
            +tt*uu1*vv1*fvr(ii+1,jj,kk)&
     	      +tt*uu*vv1*fvr(ii+1,jj+1,kk)&
     	      +tt1*uu*vv1*fvr(ii,jj+1,kk)&
            +tt1*uu1*vv*fvr(ii,jj,kk+1)&
            +tt*uu1*vv*fvr(ii+1,jj,kk+1)&
            +tt*uu*vv*fvr(ii+1,jj+1,kk+1)&
            +tt1*uu*vv*fvr(ii,jj+1,kk+1)
    avt(i,j,k)=tt1*uu1*vv1*fvt(ii,jj,kk)&
            +tt*uu1*vv1*fvt(ii+1,jj,kk)&
     	      +tt*uu*vv1*fvt(ii+1,jj+1,kk)&
     	      +tt1*uu*vv1*fvt(ii,jj+1,kk)&
            +tt1*uu1*vv*fvt(ii,jj,kk+1)&
            +tt*uu1*vv*fvt(ii+1,jj,kk+1)&
            +tt*uu*vv*fvt(ii+1,jj+1,kk+1)&
            +tt1*uu*vv*fvt(ii,jj+1,kk+1)
    avz(i,j,k)=tt1*uu1*vv1*fvz(ii,jj,kk)&
            +tt*uu1*vv1*fvz(ii+1,jj,kk)&
     	      +tt*uu*vv1*fvz(ii+1,jj+1,kk)&
     	      +tt1*uu*vv1*fvz(ii,jj+1,kk)&
            +tt1*uu1*vv*fvz(ii,jj,kk+1)&
            +tt*uu1*vv*fvz(ii+1,jj,kk+1)&
            +tt*uu*vv*fvz(ii+1,jj+1,kk+1)&
            +tt1*uu*vv*fvz(ii,jj+1,kk+1)
    avtrb(i,j,k)=tt1*uu1*vv1*fvtrb(ii,jj,kk)&
            +tt*uu1*vv1*fvtrb(ii+1,jj,kk)&
     	      +tt*uu*vv1*fvtrb(ii+1,jj+1,kk)&
     	      +tt1*uu*vv1*fvtrb(ii,jj+1,kk)&
            +tt1*uu1*vv*fvtrb(ii,jj,kk+1)&
            +tt*uu1*vv*fvtrb(ii+1,jj,kk+1)&
            +tt*uu*vv*fvtrb(ii+1,jj+1,kk+1)&
            +tt1*uu*vv*fvtrb(ii,jj+1,kk+1)
  ELSE
    atemp(i,j,k)=temp0
    apress(i,j,k)=0.d0
    adens(i,j,k)=0.d0
    ane(i,j,k)=ane0
    avr(i,j,k)=0.d0
    avt(i,j,k)=0.d0
    avz(i,j,k)=0.d0
    avtrb(i,j,k)=0.d0
  ENDIF
 ENDDO
 ENDDO
 ENDDO
!back transform of vector quantities
 DO i=1,nbod1
 DO j=1,nbod2
 DO k=1,nbod3
   CALL rotzb(avr(i,j,k),avt(i,j,k),avz(i,j,k),&			  
    avr(i,j,k),avt(i,j,k),avz(i,j,k),ctheta,stheta,calpha,salpha)
 ENDDO
 ENDDO
 ENDDO
!back transform of the velocity of the center of mass 
!of the primary explicitely because of the scattered light 
!treatment
 CALL rotzb(vxst,vyst,vzst,vxstr,vystr,vzstr,&		  
     ctheta,stheta,calpha,salpha)
!back transform of the center of the secondary explicitely 
!because	of the limb darkening treatement
 CALL rotzb(xcp,ycp,zcp,xcpr,ycpr,zcpr,&		  
     ctheta,stheta,calpha,salpha)
!    back transform of the velocity of the secondary explicitely 
!because of the scattered light treatment
 CALL rotzb(vxcp,vycp,vzcp,vxcpr,vycpr,vzcpr,&		 
     ctheta,stheta,calpha,salpha)
 END SUBROUTINE rotat1

 SUBROUTINE rotat2(theta,alpha,temp0,ane0,&
     vxst,vyst,vzst,vxstr,vystr,vzstr,&
     xcp,ycp,zcp,xcpr,ycpr,zcpr,&
     vxcp,vycp,vzcp,vxcpr,vycpr,vzcpr,&
     nbodf1,nbodf2,nbodf3,nbod1,nbod2,nbod3,&
     ar,at,az,far,fat,faz,&
     atemp,apress,adens,ane,avr,avt,avz,avtrb,&
     ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb)
!to transform (rotate) the line of sight grid (x,y,z) to the body
!frozen grid (x'',y'',z'') and interpolate state quantities from 
!the rotating body frozen coordinates (RBFC) to the line of sight 
!coordinates (LSC) and transform the velocity field vectors 
!from RBFC to LSC

!input: 	theta,alpha,dcut1
!	temp0,ane0 - out of grid (sphere) reasonable values
!            vxst,vyst,vzst - net velocity of the primary
!	xcp,ycp,zcp - position of the secondary
!	vxcp,vycp,vzcp - velocity of the secondary
!	ar,at,az -coord. in which the model is necessary (LSC)
!	far,fat,faz -coord. in which the model is defined (RBFC)
!	ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb -model
!output:
!	atemp,apress,adens,ane,avr,avt,avz,avtrb- rotated model
!            vxstr,vystr,vzstr-rotated net velocity of the primary
!	xcpr,ycpr,zcpr -rotated position of the secondary
!	vxcpr,vycpr,vzcpr - rotated velocity of the secondary 
!parameter (ndim1=100,ndim2=100,ndim3=200)
 USE sun
 USE param

 IMPLICIT NONE

 INTEGER::i,ii,j,jj,k,kk,nbod1,nbod2,nbod3,ii1,jj1,kk1,nbodf1,nbodf2,nbodf3
 REAL(KIND=8),DIMENSION(ndim1)::ar,far
 REAL(KIND=8),DIMENSION(ndim2)::at,fat
 REAL(KIND=8),DIMENSION(ndim3)::az,faz
 REAL(KIND=8),DIMENSION(ndim1,ndim2,ndim3)::atemp,apress,&
     adens,ane,avr,avt,avz,avtrb,ftemp,fpress,&
     fdens,fne,fvr,fvt,fvz,fvtrb
 REAL(KIND=8)::theta,alpha,temp0,ane0,&
     vxst,vyst,vzst,vxstr,vystr,vzstr,&
     xcp,ycp,zcp,xcpr,ycpr,zcpr,&
     vxcp,vycp,vzcp,vxcpr,vycpr,vzcpr,&     
     ctheta,stheta,calpha,salpha,&
     dx,dy,dz,dist,fx,fy,fz
 REAL(KIND=8)::dist1	

	ctheta=cos(theta)
	stheta=sin(theta)
	calpha=cos(alpha)
	salpha=sin(alpha)
!initialization of first values to ii,jj,kk
	ii=1
	jj=1
	kk=1
	DO i=1,nbod1
	DO j=1,nbod2
	DO k=1,nbod3
!  calculation of RBFC coordinates corresponding to the grid 
!  points of the line of sight grid

          CALL rotz(ar(i),at(j),az(k),fx,fy,fz,&
          ctheta,stheta,calpha,salpha)
!  	interpolation from body frozen grid to required 
!  line of sight grid  :
	  IF(fx>far(1).and.fx<far(nbodf1).and.&
             fy>fat(1).and.fy<fat(nbodf2).and.&
             fz>faz(1).and.fz<faz(nbodf3))THEN

	    CALL hunt(far,nbodf1,fx,ii)
	
	    IF(ii<1.or.ii>=nbodf1)THEN
	        WRITE(*,'(a,a,4i4)')' compiler/maschine dependent error'&
              ,' -out of grid at ii,i,j,k',ii,i,j,k
	    ENDIF
	    CALL hunt(fat,nbodf2,fy,jj)
	    IF(jj<1.or.jj>=nbodf2)THEN
	        WRITE(*,'(a,a,4i4)')' compiler/maschine dependent error'&
              ,' -out of grid at jj,i,j,k',jj,i,j,k
	    ENDIF
	    CALL hunt(faz,nbodf3,fz,kk)
	    IF(kk<1.or.kk>=nbodf3)THEN
	        WRITE(*,'(a,a,4i4)')' compiler/maschine dependent error'&
              ,' -out of grid at kk,i,j,k',kk,i,j,k
	    ENDIF
!	nearest neighbour value approximation
	    dist1=1.d60
	    DO ii1=ii,ii+1
            DO jj1=jj,jj+1
            DO kk1=kk,kk+1	
	      dx=fx-far(ii1)
              dy=fy-fat(jj1)
              dz=fz-faz(kk1)
	      dist=dx*dx+dy*dy+dz*dz
	      IF(dist<dist1)THEN
	        atemp(i,j,k)=ftemp(ii1,jj1,kk1)
	        apress(i,j,k)=fpress(ii1,jj1,kk1)
	        adens(i,j,k)=fdens(ii1,jj1,kk1)
	        ane(i,j,k)=fne(ii1,jj1,kk1)
	        avr(i,j,k)=fvr(ii1,jj1,kk1)
	        avt(i,j,k)=fvt(ii1,jj1,kk1)
	        avz(i,j,k)=fvz(ii1,jj1,kk1)
	        avtrb(i,j,k)=fvtrb(ii1,jj1,kk1)
		dist1=dist
	      ENDIF
	    ENDDO
	    ENDDO
	    ENDDO
	      ELSE
	        atemp(i,j,k)=temp0
	        apress(i,j,k)=0.d0
	        adens(i,j,k)=0.d0
	        ane(i,j,k)=ane0
	        avr(i,j,k)=0.d0
	        avt(i,j,k)=0.d0
	        avz(i,j,k)=0.d0
	        avtrb(i,j,k)=0.d0
	     ENDIF
	ENDDO
	ENDDO
	ENDDO
!back transform of vector quantities

	DO i=1,nbod1
		DO j=1,nbod2
			DO k=1,nbod3
 	  			CALL rotzb(avr(i,j,k),avt(i,j,k),avz(i,j,k),&
			          avr(i,j,k),avt(i,j,k),avz(i,j,k),ctheta,stheta,calpha,salpha)
			ENDDO
		ENDDO
	ENDDO
!back transform of the velocity of the center of mass 
!of the primary explicitely because of the scattered light 
!treatment
  	CALL rotzb(vxst,vyst,vzst,vxstr,vystr,vzstr,&
     ctheta,stheta,calpha,salpha)
!back transform of the center of the secondary explicitely 
!because	of the limb darkening treatement
	CALL rotzb(xcp,ycp,zcp,xcpr,ycpr,zcpr,&
     ctheta,stheta,calpha,salpha)
!    back transform of the velocity of the secondary explicitely 
!because of the scattered light treatment
        CALL rotzb(vxcp,vycp,vzcp,vxcpr,vycpr,vzcpr,&
     ctheta,stheta,calpha,salpha)
 END SUBROUTINE rotat2

 FUNCTION planck(freq,tep,ier)
!
!	calculates intensity of the black body radiation
!		IF   ier<0  freq [cm], planck[erg/S/cm**2/cm]
!		     ier=0  freq [Angst.], planck [erg/s/cm**2/Angstrom]
!		     ier>0  freq [1/S], planck [erg/s/cm**2/Hz]
!
 IMPLICIT NONE
 
 INTEGER:: ier
 REAL(KIND=8):: planck
 REAL(KIND=8):: freq,tep

  IF (ier < 0) THEN
     planck=1.1904397d-5/freq**5/(EXP(1.438769d0/freq/tep)-1.D0)
  ELSE IF (ier == 0) THEN
     planck=1.1904397d27/freq**5/(EXP(1.438769d8/freq/tep)-1.D0)
  ELSE
      planck=1.474501d-47*freq*freq*freq/(EXP(4.799216d-11*freq/tep)-1.D0)
  ENDIF

 END FUNCTION planck

 FUNCTION voigt(x,y)

!	taken from the old Synspec code:
! Hubeny I., Lanz T., Jeffery C.S., 1994, in Newsletter on Analysis
! of Astronomical spectra No.20, ed. C.S. Jeffery (CCP7; St. Andrews:
! St. Andrews Univ.), 30
!   	HUMLICEK VOIGT FUNCTION ALGORITHM  (LOWER PRECISION VERSION)

  IMPLICIT NONE
  REAL(KIND=8):: voigt
  REAL(KIND=8):: x,y

  REAL(KIND=8), DIMENSION(6):: t=(/0.314240376D0,0.947788391D0,1.59768264D0,2.27950708D0,3.02063703D0,3.8897249D0/),&
                       c=(/1.01172805D0,-.75197147D0,1.2557727D-2,1.00220082D-2,-2.42068135D-4,5.00848061D-7/),&
                       s=(/1.393237D0,.231152406D0,-.155351466D0,6.21836624D-3,9.19082986D-5,-6.27525958D-7/)

  REAL(KIND=8):: wr,y1,y2,d,d1,d2,d3,d4,r
  INTEGER:: i 

   wr=0.0D0
   y1 = y + 1.5D0
   y2 = y1*y1

   DO i=1,6
      r=x-t(i)
      d=1.D0/(r*r+y2)
      d1=y1*d
      d2=r*d
      r=x+t(i)
      d=1.D0/(r*r+y2)
      d3=y1*d
      d4=r*d
      wr=wr+c(i)*(d1+d3)-s(i)*(d2-d4)
   END DO

    voigt=DBLE(wr)
 END FUNCTION voigt


 FUNCTION bolt(temp,g1,g2,pot,iriad)
 
 USE unitchange	
 
 IMPLICIT NONE
!     temp -temperature [K]			     
!     g1   -statistical weight of the lower level -m	     
!     g2   -statistical weight of the upper level -n	     
!     pot  -energy diference (upper-lower level) [ev], pot>0  
!     output - population ratio N(n)/N(m) 		       
!     
! OR IF:            
!     
!     temp -temperature [K]			       
!     g2   -statistical weight of the level-n		       
!     g1   -partition function of the ion                      
!     pot  -excitation potencial of the n-th level [eV]	       
!           (with respect to ground state), pot>0	       
!     output -ratio N(n)/N, (n-th level population to	       
!          -the ion population	
!	       
! OR IF:
!     
!     iriad=1  pot is in [1/cm]         		       !
!

 REAL(KIND=8):: bolt
 REAL(KIND=8):: temp,pot
 REAL(KIND=8):: g1,g2
 INTEGER:: iriad


 
    IF (iriad == 1) THEN
	  bolt=g2/(g1*EXP(2.302585d0*5039.77d0/temp*pot/evcm))
    ELSE
	  bolt=g2/(g1*EXP(2.302585d0*5039.77d0/temp*pot))
    ENDIF
	
 END FUNCTION bolt

 SUBROUTINE utlm(dl0,ep,iat,am,iion,chi,HI,HeI,ekonc,temp,vt,gr,gr0,gs,gs0,gw,gw0,a,dop)
 
 USE constants
 USE unitchange 
 
 IMPLICIT NONE
!
!	     dl0   -line center [Angstrom]			       
!	     ep    -excit. potential of the upper level [1/cm]	       
!	     s	   -atom number 				       
!	     am    -mass number 				       
!	     z	   -degree of ionization, z=1. for neutral...	       
!	     chi   -ioniz. potentional of the ion [EV] 		       
!	     HI    -neutral hydrogen number densityY [CGS]	       
!	     HeI   -neutral helium number density [CGS]		       
!	     ekonc -electron number density [CGS]		       
!	     temp   temperature [K]			       
!	     vt    -turbulence velocity [CGS]    		       
!	     gr    -radiative damping				       
!	     gs    -stark damping				       
!	     gw    -van der waals damping			       
!								       
!	     OUTPUT:                                                   
!            
!             a   - damping parameter                                
!	      dop - doppler. half-width [1/S]                         
!

  REAL(KIND=8):: dl0,ep,s,am,z,chi,HI,HeI,ekonc,temp,vt,gr,gr0,gs,gs0,gw,gw0
  REAL(KIND=8):: a,dop
  REAL(KIND=8):: dll0,eff2,r2
  INTEGER::iat,iion

  dll0=1.d-8*dl0
  s=iat
  z=iion
!	     Radiative damping
	IF (gr0 > 0.d0) THEN
	  gr=EXP(2.302585d0*gr0)
	ELSE
	  gr=2.4734d-22*clight*clight/dll0/dll0
!	     V PRIPADE AK POZNAS SIRKU AUTOIONIZACNEJ HLADINY GRA
!	     V [EV] => GR=1.52E12*GRA
	ENDIF

  eff2=z*z*13.595d0/(chi-ep/evcm)

	IF (eff2 < 0.d0.OR.eff2 > 25.d0) eff2=25.d0

!	     Stark damping

	IF (gs0 /= 0.d0) THEN
	  gs=EXP(2.302585d0*gs0)
	ELSE
	  gs=1.0d-8*eff2**2.5d0
	ENDIF

!	     Van der Waals damping
	IF (INT(s) < 21.OR.INT(s) > 28) THEN
          r2=2.5d0*eff2*eff2/z/z
	ELSE
	  r2=(45.d0-s)/z
	ENDIF

	IF (gw0 /= 0.d0) THEN
	  gw=EXP(2.302585d0*gw0)
	ELSE
	  gw=4.5d-9*r2**0.4d0
	ENDIF

!	     Doppler halfwidth in frequency
  dop=SQRT(2.d0*bol*temp/(am*1.672614d-24)+vt*vt)/dll0
!	     Frame damping parameter
  a=(gr+gs*ekonc+gw*(HI+0.42d0*HeI)*(temp/1.d4)**0.3d0)/(4.d0*3.1415926d0*dop)
 END SUBROUTINE utlm



 SUBROUTINE rotzb(xpp,ypp,zpp,x,y,z,ctheta,stheta,calpha,salpha) 
!
!       transformation from body frozen (x'',y'',z'') to line of sight  
!       coordinates (x,y,z) 
!       i.d. rotation about  z'=z'' and THEN about x=x'
!       input: xpp,ypp,zpp   
!       output:x,y,z

  IMPLICIT NONE

  REAL(KIND=8):: z,x,y
  REAL(KIND=8):: zp,xp,yp
  REAL(KIND=8):: zpp,xpp,ypp
  REAL(KIND=8):: ctheta,stheta,calpha,salpha
        
  zp=zpp
  xp=xpp*calpha-ypp*salpha
  yp=ypp*calpha+xpp*salpha
  x=xp
  y=yp*ctheta+zp*stheta
  z=zp*ctheta-yp*stheta
        
 END SUBROUTINE rotzb
 

 SUBROUTINE rotz(x,y,z,xpp,ypp,zpp,ctheta,stheta,calpha,salpha)  
!
!       transformation from line of sight (x,y,z) to body frozen 
!       coordinates (x'',y'',z'')
!       i.d. rotation about x=x' and THEN about z'=z''
!       input: x,y,z   
!       output:xpp,ypp,zpp
  IMPLICIT NONE
 
  REAL(KIND=8):: z,x,y
  REAL(KIND=8):: zp,xp,yp
  REAL(KIND=8):: zpp,xpp,ypp
  REAL(KIND=8):: ctheta,stheta,calpha,salpha
 
  xp=x
  yp=y*ctheta-z*stheta
  zp=z*ctheta+y*stheta
  zpp=zp
  xpp=xp*calpha+yp*salpha
  ypp=yp*calpha-xp*salpha
       
 END SUBROUTINE rotz

 SUBROUTINE surf(nbod1,nbod2,ar,at,area)
!
!	to calculate area(i,j)-area of a projected surface element
!	input: ar,at
!	output: area
!
 USE param

  IMPLICIT NONE

  INTEGER::nbod1,nbod2
  INTEGER:: i,j
  REAL(KIND=8),DIMENSION(ndim1):: ar
  REAL(KIND=8),DIMENSION(ndim2):: at
  REAL(KIND=8),DIMENSION(ndim1,ndim2):: area
  REAL(KIND=8):: pomi12,pominn,pom
  REAL(KIND=8):: rshell
   DO i=2,nbod1-1

     DO j=2,nbod2-1
        area(i,j)=(ar(i+1)-ar(i-1))*(at(j+1)-at(j-1))*0.25d0
     END DO

   END DO

   pomi12=(ar(2)-ar(1))*0.5d0
   pominn=(ar(nbod1)-ar(nbod1-1))*0.5d0

   DO j=2,nbod2-1
	  area(1,j)=pomi12*(at(j+1)-at(j-1))*0.5d0
	  area(nbod1,j)=pominn*(at(j+1)-at(j-1))*0.5d0
   END DO

   DO  i=2,nbod1-1
	  area(i,1)=(ar(i+1)-ar(i-1))*(at(2)-at(1))*0.25d0
	  area(i,nbod2)=(ar(i+1)-ar(i-1))*(at(nbod2)-at(nbod2-1))*0.25d0
   END DO

   area(1,1)=pomi12*(at(2)-at(1))*0.5d0
   area(1,nbod2)=pomi12*(at(nbod2)-at(nbod2-1))*0.5d0
   area(nbod1,1)=pominn*(at(2)-at(1))*0.5d0
   area(nbod1,nbod2)=pominn*(at(nbod2)-at(nbod2-1))*0.5d0

!od ted se lisi 07.f od 17.f
 !  sum=0.d0
!	
 !  DO i=1,nbod1
!
!	DO j=1,nbod2
!	  pom=ar(i)*ar(i)+at(j)*at(j)
!	  IF (pom < rshell*rshell) sum=sum+area(i,j)
 !       END DO
!
 !  END DO

!       Je treba zjistit kam se to zapisuje a jestli je to vubec nutne to psat
!	WRITE(2,*)' sum area(ij)/shelldisc=',sum/pi/rshell/rshell

 
 END SUBROUTINE surf

 SUBROUTINE tlac1(nbod3,az,vr,vt,vz,&
    temp,ekonc,akonc,hi,hneg,hei,hipf,&
     ophbf,ophff,ophrs)
!	output of selected quantities for selected ray
!	along the light of sight
 USE param
 IMPLICIT NONE

 REAL(KIND=8),DIMENSION(ndim3)::az,hi,hneg,hei,ophbf,ophff,ophrs,vr,vt,vz,temp,ekonc
 REAL(KIND=8),DIMENSION(ndim3)::akonc
 REAL(KIND=8),DIMENSION(ndim3,2)::hipf
 INTEGER::kk,nbod3

        WRITE(2,'(A)')' velocity   vx        vy        vz'
        DO kk=1,nbod3
          WRITE(2,'(i4,3E11.3)')kk,vr(kk),vt(kk),vz(kk)
	ENDDO

        WRITE (2,'(a,a)')'       akonc      ekonc      temp',&
     				     '       az'
        DO kk=1,nbod3
          WRITE(2,'(i4,4e11.3)')kk,akonc(kk),ekonc(kk),temp(kk),az(kk)
	ENDDO
        WRITE(2,'(a,a)')'       AKONC      HI         H-         HEI',&
				    '        HIPF       HIIPF'
        DO kk=1,nbod3
	  WRITE(2,'(i4,6E11.3)')kk,akonc(kk),hi(kk),hneg(kk),hei(kk),&
     			hipf(kk,1),hipf(kk,2) 
	ENDDO
        WRITE(2,'(a)')'        ophbf1     ophff1     ophrs1'
        DO kk=1,nbod3
	  WRITE(2,'(i4,3E11.3)')kk,ophbf(kk),ophff(kk),ophrs(kk)
	ENDDO

 END SUBROUTINE tlac1
!-----------------------------------------------------------------------
 SUBROUTINE tlac2(nbod3,&
     nion,iion,az,zip,eup,&
     temp,stavs,rr,gr,gs,gw)
!	output of selected quantities for selected ray
!	along the light of sight
 USE unitchange
 USE param
 IMPLICIT NONE

 INTEGER::kk,jj1
 INTEGER::nbod3,iion,nion
 REAL(KIND=8),DIMENSION(ndim3)::az,temp
 REAL(KIND=8),DIMENSION(ndim3,mion):: stavs,rr
 REAL(KIND=8),DIMENSION(mion-1)::zip
 REAL(KIND=8)::gr,gs,gw,eup

             WRITE (2,'(A)')' PARTITION FUNCTIONS'
             WRITE (2,'(A,20I10)')' TEMP',(kk,kk=1,NION)
             DO kk=1,nbod3
                WRITE(2,'(F7.0,20E10.3)')TEMP(kk),(STAVS(kk,jj1),jj1=1,NION)
	     ENDDO

             WRITE(2,*)'   N(J)/N  ION POPULATIONS'
             WRITE(2,'(A,40I11)')' z',(kk,kk=1,NION)
             DO kk=1,nbod3
                WRITE(2,'(40(D10.3,1X))')az(kk),(RR(kk,jj1),jj1=1,NION)
	     ENDDO

	     WRITE(2,'(a,a)')' i   GR        GS        GW        '
!     p        'damp      dophw'
             DO  kk=1,nbod3
	        IF(ZIP(IION)<EUP/EVCM)THEN
	          WRITE(2,'(i3,3E10.3)')kk,GR,GS,GW !vynechali jsem nepovinny parametr auto
	        ELSE
	          WRITE(2,'(i3,3E10.3)')kk,GR,GS,GW
	        ENDIF
	     ENDDO
 END SUBROUTINE tlac2

 SUBROUTINE smod1(nbodf1,nbodf2,nbodf3,rmodfx,rmodfy,rmodfz,&
gainfx,gainfy,gainfz,&
rstar,tstar,emstar,xstar,ystar,zstar,vrotst,vxst,vyst,vzst,&
istar,dgst,ffst,irrst,albst,htst,htsta,&
icomp,dgcp,ffcp,qq,vrxcp,vrycp,vrzcp,vrotcp,rcp,&
xcp,ycp,zcp,vxcp,vycp,vzcp,tempcp,irrcp,albcp,htcp,htcpa,&
ispot,vrxsp,vrysp,vrzsp,vrotsp,rsp,&
xsp,ysp,zsp,vxsp,vysp,vzsp,tempsp,denssp,anesp,vtrbsp,&
xdisc,ydisc,zdisc,adisc,rindc,routdc,idisc,edendc,itdc,&
vxdc,vydc,vzdc,tempdc,densdc,anedc,vtrbdc,&
x1sm,y1sm,z1sm,x2sm,y2sm,z2sm,v1sm,v2sm,rsm,ism,&
tempsm,denssm,anesm,vtrbsm,&
xjet,yjet,zjet,ajet,rinjt,routjt,vjt,&
tempjt,densjt,anejt,vtrbjt,ijet,&
rinsh,routsh,vsh,&
tempsh,denssh,anesh,vtrbsh,ishell,evelsh,rcsh,&
v0,temp0,dens0,ane0,&
far,fat,faz,ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb)
!
!	Calculates model of a shell (behaviour of state quantities and
!	velocity field in Cartesian coordinates x,y,z=far,fat,faz. 
!	frv,ftv,fzv are corresponding components of velocity vector.
!	ftemp,fpress,fdens,fne are temperature,pressure,density
!	and electron number density, respectively.
!	fvtrb- is parameter to characterize microturbulence
!	Reasonable vaules of temp and ekonc must be defined 
!	everywhere but you can use dens<=0.d0 to define empty space
!	or dcut1<dens<dcut2 for central nontransparent objects (star),
!	dcut2<dens<dcut3 for second nontransparent object (companion),
!       dcut3<dens<dcutn for 3. nontransparent object (3.body),
!	dcutn<dens for any dark nontransparent object.
!	nbod1,nbod2,nbod3>=1
!	nbod3 should be large enough so that a huge leaps in opt. depth
!	DO not occure because of large velocity and opacity change 
!	between the grid points
!
   USE sun
   USE param
   USE constants
   USE unitchange
   IMPLICIT NONE


   INTEGER     :: i,j,k,n1,n2
   INTEGER     :: itdc
   INTEGER     :: istar,icomp,ispot,idisc,ijet,ishell,ism,irrst,irrcp
   INTEGER     :: nbod1,nbod2,nbod3,nbodf1,nbodf2,nbodf3
  
   REAL(KIND=8), DIMENSION(ndim1)::far
   REAL(KIND=8), DIMENSION(ndim2)::fat
   REAL(KIND=8), DIMENSION(ndim3)::faz
   REAL(KIND=8), DIMENSION(ndim1,ndim2,ndim3)::ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb
   REAL(KIND=8), DIMENSION(ndim1,ndim2)::rosup,rosus,rotemp,rotems
   REAL(KIND=8),DIMENSION(3,3)::trmt,trmtst

   REAL(KIND=8):: rmodfx,rmodfy,rmodfz
   REAL(KIND=8):: gainfx,gainfy,gainfz
   REAL(KIND=8):: rstar,tstar,emstar,xstar,ystar,zstar,vrotst,vxst,vyst,vzst
   REAL(KIND=8):: dgst,ffst,albst,htst,htsta
   REAL(KIND=8)::dgcp,ffcp,qq,vrxcp,vrycp,vrzcp,vrotcp,rcp
   REAL(KIND=8):: xcp,ycp,zcp,vxcp,vycp,vzcp,tempcp,albcp,htcp,htcpa
   REAL(KIND=8):: vrxsp,vrysp,vrzsp,vrotsp,rsp
   REAL(KIND=8):: xsp,ysp,zsp,vxsp,vysp,vzsp,tempsp,denssp,anesp,vtrbsp
   REAL(KIND=8):: xdisc,ydisc,zdisc,adisc,rindc,routdc,edendc
   REAL(KIND=8):: vxdc,vydc,vzdc,tempdc,densdc,anedc,vtrbdc
   REAL(KIND=8):: x1sm,y1sm,z1sm,x2sm,y2sm,z2sm,v1sm,v2sm,rsm
   REAL(KIND=8):: tempsm,denssm,anesm,vtrbsm
   REAL(KIND=8):: xjet,yjet,zjet,ajet,rinjt,routjt,vjt
   REAL(KIND=8):: tempjt,densjt,anejt,vtrbjt
   REAL(KIND=8):: rinsh,routsh,vsh
   REAL(KIND=8):: tempsh,denssh,anesh,vtrbsh,evelsh,rcsh
   REAL(KIND=8):: v0,temp0,dens0,ane0
   REAL(KIND=8)::a1,a2,angle,areasp,areass,teffp,cosir,coslat,dunt1,dunt2
   REAL(KIND=8)::dxdc3,dxsm3,dxsm,dydc3,dysm,dysm3,dzdc3,dzsm,dzsm3
   REAL(KIND=8)::er,er2,er2dc,er2sm,ermcp,ermsp,farcp,farsp,fatcp,fatsp,fazcp
   REAL(KIND=8)::fazsp,ffx,ffy,ffz,firr,fnlat,htstb,omega,omgcp,omgst
   REAL(KIND=8)::pom,porb,r1,ratio,rback,rpole,rside,rmean,rfront,rn,rr2vv
   REAL(KIND=8)::rshell,rstep,st1,t04,t1,teffs,tirr4,tn,tsm,tstep,undisc,unjet,unspot
   REAL(KIND=8)::unstar,vdc,vshin,vshr,vsm,xdc3,xsm3,ydc3,ysm3
   REAL(KIND=8)::tdc,unsm,vvsh,z1,zdc3,zn,zsm3,zstep

 	dunt1=(dcut1+dcut2)*0.5d0
	dunt2=(dcut2+dcut3)*0.5d0
	IF(gainfx==1.d0)THEN
          r1=-rmodfx
          rn=rmodfx
          rstep=(rn-r1)/dble(nbodf1-1)
	DO i=1,nbodf1
	    far(i)=r1+rstep*dble(i-1)
	ENDDO
	ELSE
	  n2=nbodf1/2
	  st1=rmodfx*(1.d0-gainfx)/(1.d0-gainfx**n2)
	  far(n2+1)=0.d0
	 DO i=1,n2
	    far(n2+1+i)=st1*(1.d0-gainfx**i)/(1.d0-gainfx)
	    far(n2+1-i)=-st1*(1.d0-gainfx**i)/(1.d0-gainfx)
	ENDDO
	ENDIF
	IF(gainfy==1.d0)THEN
	  t1=-rmodfy
	  tn=rmodfy
	  tstep=(tn-t1)/dble(nbodf2-1)
	  DO i=1,nbodf2
	    fat(i)=t1+tstep*dble(i-1)
	  ENDDO
	ELSE
	  n2=nbodf2/2
	  st1=rmodfy*(1.d0-gainfy)/(1.d0-gainfy**n2)
	  fat(n2+1)=0.d0
	  DO i=1,n2
	    fat(n2+1+i)=st1*(1.d0-gainfy**i)/(1.d0-gainfy)
	    fat(n2+1-i)=-st1*(1.d0-gainfy**i)/(1.d0-gainfy)
	  ENDDO
	ENDIF
	IF(gainfz==1.d0)THEN
	  z1=-rmodfz
	  zn=rmodfz
	  zstep=(zn-z1)/dble(nbodf3-1)
	  DO i=1,nbodf3
	    faz(i)=z1+zstep*dble(i-1)	  
	  ENDDO
	ELSE
	  n2=nbodf3/2
	  st1=rmodfz*(1.d0-gainfz)/(1.d0-gainfz**n2)
	  faz(n2+1)=0.d0
	  DO i=1,n2
	    faz(n2+1+i)=st1*(1.d0-gainfz**i)/(1.d0-gainfz)
	    faz(n2+1-i)=-st1*(1.d0-gainfz**i)/(1.d0-gainfz)
	ENDDO
	ENDIF
 

   IF (istar > 1 .or.icomp > 1) THEN
      a1=xcp*qq/(1.d0+qq)
      a2=xcp-a1
      porb=2.d0*pi*SQRT(xcp**3/(grav*emstar*(1.d0+qq)))
      omgst=2.d0*pi/porb
      omgcp=omgst
      vxst=0.d0
      vyst=-omgst*a1
      vzst=0.d0
      vxcp=0.d0
      vycp=omgst*a2
      vzcp=0.d0
      ycp=0.d0
      zcp=0.d0
   END IF

   IF (istar == 1) THEN
	  unstar=xstar*xstar+ystar*ystar+zstar*zstar
	  unstar=SQRT(unstar)
	  xstar=xstar/unstar
	  ystar=ystar/unstar
	  zstar=zstar/unstar
	  omgst=vrotst/rstar/unstar
	  call trans(xstar,ystar,zstar,trmtst)
   ELSE IF (istar == 2) THEN
	  xstar=0.d0
	  ystar=0.d0
	  zstar=1.d0
!		rosup>=0 -Roche surface of the primary
!		rogrvp-normalized Roche gravity on its surface
!
	  CALL roche(rosup,rotemp,areasp,teffp,&
     rfront,rback,rpole,rside,rmean ,&
     tstar,rstar,dgst,irrst,albst,htst,htsta,&
     tempcp,rcp,dgcp,irrcp,albcp,htcp,htcpa,&
     far,fat,xcp,qq,ffst,nbodf1,nbodf2,1)
!

   ELSE IF (istar == 3) THEN
	  xstar=0.d0
	  ystar=0.d0
	  zstar=1.d0
	  icomp=0
!		rosup>=0 -Roche surface of the contact system
!		rogrvp-normalized Roche gravity on its surface

	  CALL roche(rosus,rotems,areass,teffs,&
     rfront,rback,rpole,rside,rmean ,&
     tstar,rstar,dgst,irrst,albst,htst,htsta,&
     tempcp,rcp,dgcp,irrcp,albcp,htcp,htcpa,&
     far,fat,xcp,qq,ffst,nbodf1,nbodf2,3)
	ENDIF

	IF(icomp==2)THEN
	  vrxcp=0.d0
	  vrycp=0.d0
	  vrzcp=1.d0
	  CALL roche(rosus,rotems,areass,teffs,&
     rfront,rback,rpole,rside,rmean ,&
     tstar,rstar,dgst,irrst,albst,htst,htsta,&
     tempcp,rcp,dgcp,irrcp,albcp,htcp,htcpa,&
     far,fat,xcp,qq,ffcp,nbodf1,nbodf2,2)
!

  	 END IF


    IF (idisc == 1.or.idisc == 2) THEN
	  undisc=xdisc*xdisc+ydisc*ydisc+zdisc*zdisc
	  undisc=SQRT(undisc)
	  xdisc=xdisc/undisc
	  ydisc=ydisc/undisc
	  zdisc=zdisc/undisc
	ELSE IF(idisc==3)THEN
	  undisc=xdisc*xdisc+ydisc*ydisc+zdisc*zdisc
	  undisc=sqrt(undisc)
	  xdisc=xdisc/undisc
	  ydisc=ydisc/undisc
	  zdisc=zdisc/undisc
!		calculates rotation matrix trmt
	  call trans(xdisc,ydisc,zdisc,trmt)
    END IF

    IF (ism == 1) THEN
	  dxsm=x2sm-x1sm
	  dysm=y2sm-y1sm
	  dzsm=z2sm-z1sm	
	  unsm=dxsm*dxsm+dysm*dysm+dzsm*dzsm
	  unsm=SQRT(unsm)
	  dxsm=dxsm/unsm
	  dysm=dysm/unsm
	  dzsm=dzsm/unsm	
    END IF

    IF (ijet == 1.or.ijet == 2) THEN	
	  unjet=xjet*xjet+yjet*yjet+zjet*zjet
	  unjet=SQRT(unjet)
    END IF


	WRITE(2,*)' Porb[d]  istar  icomp'
	WRITE(2,*)porb/8.64d4,istar,icomp
	WRITE(2,'(a,a)')'     xstar      ystar      zstar      ','vxst       vyst       vzst'
       WRITE(2,'(6e11.3)')xstar,ystar,zstar,vxst,vyst,vzst
	WRITE(2,'(a,a)')'     vrxcp      vrycp      vrzcp      ','vxcp       vycp       vzcp'
       WRITE(2,'(6e11.3)')vrxcp,vrycp,vrzcp,vxcp,vycp,vzcp
        WRITE(2,'(a,a)')'       xcp        ycp        zcp      '
        WRITE(2,'(3e11.3)')xcp,ycp,zcp
        IF(istar>1)THEN
          WRITE(2,'(a,a)')' Roche S.  Teffprim '
          WRITE(2,'(2e12.4)')areasp,teffp
          WRITE(2,'(a,a)')'  Rsub     Rback    Rpole    Rside    Rmean',&
     '    Rsub/Rpole in units of a'
          WRITE(2,'(6f9.5)')rfront,rback,rpole,rside,rmean,rfront/rpole
        ENDIF  
        IF(icomp==2)THEN
          WRITE(2,'(a,a)')' Roche S. Teffsec'
          WRITE(2,'(2e12.4)')areass,teffs        
          WRITE(2,'(a,a)')'  Rsub     Rback    Rpole    Rside    Rmean',&
    '    Rsub/Rpole in units of a'
          WRITE(2,'(6f9.5)')rfront,rback,rpole,rside,rmean,rfront/rpole          
        ENDIF
    DO i=1,nbodf1
       DO j=1,nbodf2
          DO k=1,nbodf3
	      er2=far(i)*far(i)+fat(j)*fat(j)+faz(k)*faz(k)
	      er=SQRT(er2)
!	      out of shell grid
 !             IF (er >= rshell) THEN
!	        ftemp(i,j,k)=temp0
!	        fpress(i,j,k)=0.d0
!	        fdens(i,j,k)=0.d0
!	        fne(i,j,k)=ane0
!	        fvr(i,j,k)=0.d0
!	        fvt(i,j,k)=0.d0
!	        fvz(i,j,k)=0.d0
!	        fvtrb(i,j,k)=0.d0	
!	        CYCLE
!	      END IF
!	      inside the star
              IF (istar == 1) THEN 
                 IF (er < rstar) THEN
	          ftemp(i,j,k)=tstar
		  IF(er2<=0.d0)EXIT
	          IF(irrst==1.and.icomp>0)THEN
	            farcp=far(i)-xcp		
	            fatcp=fat(j)-ycp
	            fazcp=faz(k)-zcp
		    cosir=far(i)*xcp+fat(j)*ycp+faz(k)*zcp
		    cosir=cosir/sqrt(xcp**2+ycp**2+zcp**2)/er
!			new coordinates aligned with the rot.axis		
		    ffx=trmtst(1,1)*far(i)+trmtst(1,2)*fat(j)&
      		    +trmtst(1,3)*faz(k)
		    ffy=trmtst(2,1)*far(i)+trmtst(2,2)*fat(j)&
     		    +trmtst(2,3)*faz(k)
		    ffz=trmtst(3,1)*far(i)+trmtst(3,2)*fat(j)&
     		    +trmtst(3,3)*faz(k)
     		    coslat=(ffx**2+ffy**2)/(ffx**2+ffy**2+ffz**2)
		    coslat=sqrt(coslat)
    		    htstb=4.d0*(1.d0-htsta)/pi
    		    fnlat=htsta+htstb*coslat
    		    pom=rcp**2/(xcp**2+ycp**2+zcp**2)	
                    t04=0.25d0*htst*(1.d0-albst)*pom*tempcp**4
		    IF(cosir>0.d0)THEN
!		      dayside
		      pom=rcp**2/(farcp**2+fatcp**2+fazcp**2)	
	              firr=pom*tempcp**4*cosir
	              tirr4=(1.d0-htst)*(1.d0-albst)*firr
	              ftemp(i,j,k)=(tirr4+t04*fnlat+tstar**4)**0.25
	            ELSE  
!		      nightside	            
                      ftemp(i,j,k)=(t04*fnlat+tstar**4)**0.25
		    ENDIF
	          ENDIF

	          fpress(i,j,k)=0.d0
	          fdens(i,j,k)=dunt1
	          fne(i,j,k)=ane0
	          fvr(i,j,k)=omgst*(ystar*faz(k)-zstar*fat(j))+vxst
	          fvt(i,j,k)=omgst*(zstar*far(i)-xstar*faz(k))+vyst
	          fvz(i,j,k)=omgst*(xstar*fat(j)-ystar*far(i))+vzst
	          fvtrb(i,j,k)=0.d0
	          CYCLE
                ENDIF  
              elseif(istar==2 .or. istar==3)THEN
                IF(abs(faz(k))<rosup(i,j))THEN
	          ftemp(i,j,k)=rotemp(i,j)
	          fpress(i,j,k)=0.d0
	          fdens(i,j,k)=dunt1
	          fne(i,j,k)=ane0
	          fvr(i,j,k)=omgst*(ystar*faz(k)-zstar*fat(j))+vxst
	          fvt(i,j,k)=omgst*(zstar*far(i)-xstar*faz(k))+vyst
	          fvz(i,j,k)=omgst*(xstar*fat(j)-ystar*far(i))+vzst
	          fvtrb(i,j,k)=0.d0
	          CYCLE
                ENDIF  
	      ENDIF	      


!	      inside the companion
	      IF (icomp == 1) THEN
                farcp=far(i)-xcp		
	        fatcp=fat(j)-ycp
	        fazcp=faz(k)-zcp
	        ermcp=farcp*farcp+fatcp*fatcp+fazcp*fazcp
	        ermcp=SQRT(ermcp)	

                IF (ermcp < rcp) THEN
	 	  unspot=vrxcp*vrxcp+vrycp*vrycp+vrzcp*vrzcp
		  unspot=SQRT(unspot)
		  omgcp=vrotcp/rcp/unspot
	          ftemp(i,j,k)=tempcp
	          fpress(i,j,k)=0.d0
	          fdens(i,j,k)=dunt2
	          fne(i,j,k)=ane0
	          fvr(i,j,k)=omgcp*(vrycp*fazcp-vrzcp*fatcp)+vxcp
	          fvt(i,j,k)=omgcp*(vrzcp*farcp-vrxcp*fazcp)+vycp
	          fvz(i,j,k)=omgcp*(vrxcp*fatcp-vrycp*farcp)+vzcp
	          fvtrb(i,j,k)=0.d0
	          CYCLE
	        END IF	     

	      ELSE IF (icomp == 2) THEN

   	        IF (ABS(faz(k)) < rosus(i,j)) THEN
   	          farcp=far(i)-xcp                
   	          fatcp=fat(j)-ycp
   	          fazcp=faz(k)-zcp
	          ftemp(i,j,k)=rotems(i,j)
	          fpress(i,j,k)=0.d0
	          fdens(i,j,k)=dunt2
	          fne(i,j,k)=ane0
	          fvr(i,j,k)=omgcp*(vrycp*fazcp-vrzcp*fatcp)+vxcp
	          fvt(i,j,k)=omgcp*(vrzcp*farcp-vrxcp*fazcp)+vycp
	          fvz(i,j,k)=omgcp*(vrxcp*fatcp-vrycp*farcp)+vzcp
	          fvtrb(i,j,k)=0.d0
	          CYCLE
		END IF

	      END IF

!	      inside the spot
	      IF (ispot == 1) THEN
	        farsp=far(i)-xsp		
	        fatsp=fat(j)-ysp
	        fazsp=faz(k)-zsp
	        ermsp=farsp*farsp+fatsp*fatsp+fazsp*fazsp
	        ermsp=SQRT(ermsp)
	
                IF (ermsp < rsp) THEN
	 	  unspot=vrxsp*vrxsp+vrysp*vrysp+vrzsp*vrzsp
		  unspot=SQRT(unspot)
		  omega=vrotsp/rsp/unspot
	          ftemp(i,j,k)=tempsp
	          fpress(i,j,k)=0.d0
	          fdens(i,j,k)=denssp
	          fne(i,j,k)=anesp
	          fvr(i,j,k)=omega*(vrysp*fazsp-vrzsp*fatsp)+vxsp
	          fvt(i,j,k)=omega*(vrzsp*farsp-vrxsp*fazsp)+vysp
	          fvz(i,j,k)=omega*(vrxsp*fatsp-vrysp*farsp)+vzsp
	          fvtrb(i,j,k)=vtrbsp
	          CYCLE
	        END IF
	      
	      END IF

!	      in the disc
	      IF (idisc == 1) THEN
	        angle=(xdisc*far(i)+ydisc*fat(j)+zdisc*faz(k))/er
	        angle=ACOS(angle)
                IF (ABS(angle-pi/2.d0) < adisc.and.er > rindc.and.er < routdc) THEN
	          vdc=SQRT(grav*emstar/er)/er

		  IF (itdc == 1) THEN
	            ftemp(i,j,k)=tempdc
		  ELSE
		  !tady bylo driv rstar/er
		    ratio=er/rstar
	            ftemp(i,j,k)=0.8D0*tstar*(ratio)**(-0.75D0)
	            WRITE(35,*)ftemp(i,j,k),ratio,er/rsol
		  ENDIF

		  ratio=er/rindc
	          fdens(i,j,k)=densdc*ratio**edendc
	          write(36,*)fdens(i,j,k),er/rsol,ratio
	          fne(i,j,k)=anedc*ratio**edendc
	          fpress(i,j,k)=0.d0
	          fvr(i,j,k)=vdc*(ydisc*faz(k)-zdisc*fat(j))+vxdc
	          fvt(i,j,k)=vdc*(zdisc*far(i)-xdisc*faz(k))+vydc
	          fvz(i,j,k)=vdc*(xdisc*fat(j)-ydisc*far(i))+vzdc
	          fvtrb(i,j,k)=vtrbdc
	          CYCLE
		END IF

	      ELSE IF (idisc == 2) THEN	
		tdc=xdisc*far(i)+ydisc*fat(j)+zdisc*faz(k)
		xdc3=xdisc*tdc
		ydc3=ydisc*tdc
		zdc3=zdisc*tdc
		dxdc3=far(i)-xdc3
		dydc3=fat(j)-ydc3
		dzdc3=faz(k)-zdc3
!		er2dc=dxdc3*dxdc3+dydc3*dydc3+dzdc3*dzdc3

                IF (ABS(tdc) < adisc.and.er > rindc.and.er < routdc)THEN
	          vdc=SQRT(grav*emstar/er)/er

		  IF (itdc == 1) THEN
	            ftemp(i,j,k)=tempdc
		  ELSE
		   !tady bylo driv rstar/er
		    ratio=er/rstar
	            ftemp(i,j,k)=0.8D0*tstar*(ratio)**(-0.75D0)
	            WRITE(35,*)ftemp(i,j,k)
		  END IF

		  ratio=er/rindc
	          fdens(i,j,k)=densdc*ratio**edendc
	          fne(i,j,k)=anedc*ratio**edendc
	          fpress(i,j,k)=0.d0
	          fvr(i,j,k)=vdc*(ydisc*faz(k)-zdisc*fat(j))+vxdc
	          fvt(i,j,k)=vdc*(zdisc*far(i)-xdisc*faz(k))+vydc
	          fvz(i,j,k)=vdc*(xdisc*fat(j)-ydisc*far(i))+vzdc
	          fvtrb(i,j,k)=vtrbdc
	          CYCLE
		END IF


		ELSE IF (idisc == 3) THEN
!		rotation of the coordinates to align with the disc		
		ffx=trmt(1,1)*far(i)+trmt(1,2)*fat(j)+trmt(1,3)*faz(k)
		ffy=trmt(2,1)*far(i)+trmt(2,2)*fat(j)+trmt(2,3)*faz(k)
		ffz=trmt(3,1)*far(i)+trmt(3,2)*fat(j)+trmt(3,3)*faz(k)
	        er2dc=ffx*ffx/routdc/routdc+ffy*ffy/routdc/routdc&
     	        +ffz*ffz/adisc/adisc
                IF(er2dc<1.d0.and.er>rindc)THEN
	          vdc=sqrt(grav*emstar/er)/er
		  IF(itdc==1)THEN
	            ftemp(i,j,k)=tempdc
		  ELSE
		    !tady bylo driv rstar/er
		    ratio=er/rstar
	            ftemp(i,j,k)=0.8D0*tstar*(ratio)**(-0.75D0)
		  ENDIF
		  ratio=er/rindc
	          fdens(i,j,k)=densdc*ratio**edendc
	          fne(i,j,k)=anedc*ratio**edendc
	          fpress(i,j,k)=0.d0
	          fvr(i,j,k)=vdc*(ydisc*faz(k)-zdisc*fat(j))+vxdc
	          fvt(i,j,k)=vdc*(zdisc*far(i)-xdisc*faz(k))+vydc
	          fvz(i,j,k)=vdc*(xdisc*fat(j)-ydisc*far(i))+vzdc
	          fvtrb(i,j,k)=vtrbdc
	          CYCLE
		ENDIF
	      ENDIF	  
!	      in the stream
	      IF (ism == 1) THEN
		tsm=dxsm*(far(i)-x1sm)+dysm*(fat(j)-y1sm)+dzsm*(faz(k)-z1sm)
		xsm3=dxsm*tsm+x1sm
		ysm3=dysm*tsm+y1sm
		zsm3=dzsm*tsm+z1sm
		dxsm3=xsm3-far(i)
		dysm3=ysm3-fat(j)
		dzsm3=zsm3-faz(k)
		er2sm=dxsm3*dxsm3+dysm3*dysm3+dzsm3*dzsm3

		IF (tsm > 0.d0.and.tsm < unsm.and.er2sm < rsm*rsm) THEN
	          ftemp(i,j,k)=tempsm
	          fpress(i,j,k)=0.d0
	          fdens(i,j,k)=denssm*v1sm/vsm
	          fne(i,j,k)=anesm
	          fvr(i,j,k)=vsm*dxsm
	          fvt(i,j,k)=vsm*dysm
	          fvz(i,j,k)=vsm*dzsm
	          fvtrb(i,j,k)=vtrbsm
                  CYCLE
		END IF
              
              END IF
            
!	      in the jet 
	      IF (ijet ==1) THEN	
	        angle=(xjet*far(i)+yjet*fat(j)+zjet*faz(k))/unjet/er
	        angle=ACOS(angle)

                IF (ABS(angle-0.d0) < ajet.and.er >= rinjt.and.er < routjt) THEN
	          ftemp(i,j,k)=tempjt
	          fpress(i,j,k)=0.d0
	          fdens(i,j,k)=densjt
	          fne(i,j,k)=anejt
	          fvr(i,j,k)=vjt*far(i)/er
	          fvt(i,j,k)=vjt*fat(j)/er
	          fvz(i,j,k)=vjt*faz(k)/er
	          fvtrb(i,j,k)=vtrbjt
	          CYCLE
		END IF

	      ELSE IF (ijet == 2) THEN
	        angle=(xjet*far(i)+yjet*fat(j)+zjet*faz(k))/unjet/er
	        angle=ACOS(angle)

                IF ((ABS(angle-0.d0) < ajet.or.ABS(angle-pi) < ajet)&
                .and.er >= rinjt.and.er < routjt) THEN
	          ftemp(i,j,k)=tempjt
	          fpress(i,j,k)=0.d0
	          fdens(i,j,k)=densjt
	          fne(i,j,k)=anejt
	          fvr(i,j,k)=vjt*far(i)/er
	          fvt(i,j,k)=vjt*fat(j)/er
	          fvz(i,j,k)=vjt*faz(k)/er
	          fvtrb(i,j,k)=vtrbjt
	          CYCLE
		END IF

	      END IF

!	      in the shell
              IF (ishell == 1.and.er >=rinsh.and.er < routsh) THEN
                ftemp(i,j,k)=tempsh
                fpress(i,j,k)=0.d0
                fdens(i,j,k)=denssh
                fne(i,j,k)=anesh   
                fvr(i,j,k)=vsh*far(i)/er
                fvt(i,j,k)=vsh*fat(j)/er
                fvz(i,j,k)=vsh*faz(k)/er
                fvtrb(i,j,k)=vtrbsh
                CYCLE

              ELSE IF (ishell == 2.and.er >= rinsh.and.er < routsh) THEN
		vvsh=(er/rinsh)**(-evelsh)
		vshr=vsh/vvsh
		rr2vv=rinsh*rinsh/(er*er)*vvsh
	        fvr(i,j,k)=vshr*far(i)/er
	        fvt(i,j,k)=vshr*fat(j)/er
	        fvz(i,j,k)=vshr*faz(k)/er
	        fvtrb(i,j,k)=vtrbsh
	        ftemp(i,j,k)=tempsh
	        fpress(i,j,k)=0.d0
	        fdens(i,j,k)=denssh*rr2vv
	        fne(i,j,k)=anesh*rr2vv
	        CYCLE 

	      ELSE IF (ishell == 3.and.er >= rinsh.and.er < routsh) THEN
		vshr=vsh*(1.d0-rcsh/er)**evelsh
		vshin=vsh*(1.d0-rcsh/rinsh)**evelsh
		rr2vv=rinsh*rinsh/(er*er)*vshin/vshr
	        fvr(i,j,k)=vshr*far(i)/er
	        fvt(i,j,k)=vshr*fat(j)/er
	        fvz(i,j,k)=vshr*faz(k)/er
	        fvtrb(i,j,k)=vtrbsh
	        ftemp(i,j,k)=tempsh
	        fpress(i,j,k)=0.d0
	        fdens(i,j,k)=denssh*rr2vv
	        fne(i,j,k)=anesh*rr2vv
	        CYCLE
	      END IF

!	      in the background	
	      ftemp(i,j,k)=temp0
	      fpress(i,j,k)=0.d0
	      fdens(i,j,k)=dens0
	      fne(i,j,k)=ane0
	      fvr(i,j,k)=v0*far(i)/er
	      fvt(i,j,k)=v0*fat(j)/er
	      fvz(i,j,k)=v0*faz(k)/er
	      fvtrb(i,j,k)=0.d0	
          END DO
       END DO
    END DO

 END SUBROUTINE smod1


SUBROUTINE smod2(nbodf1,nbodf2,nbodf3,&
     far,fat,faz,ftemp,fpress,fdens,fne,fvr,fvt,fvz,fvtrb)

 USE param

 IMPLICIT NONE

 REAL(KIND=8),DIMENSION(ndim1):: far
 REAL(KIND=8),DIMENSION(ndim2)::fat
 REAL(KIND=8),DIMENSION(ndim3)::faz
 REAL(KIND=8),DIMENSION(ndim1,ndim2,ndim3)::ftemp,fpress,&
     fdens,fne,fvr,fvt,fvz,fvtrb
 INTEGER::nbodf1,nbodf2,nbodf3,i,j,k

!	READs the input model of the shell from the file
!	nbodf1,nbodf2,nbodf3 -number of x,y,z grid points
!	far,fat,faz -define the x,y,z grid points [cm]
!	ftemp, fpress, fdens, fne -temperature,pressure, density, 
!		electron number density, respectively [cgs]
!	fvr,fvt,fvz,fvtrb -x,y,z components of the velocity field [cm/s]
!	fvtrb -turbulence [cm/s]
	READ(10,*)nbodf1,nbodf2,nbodf3
	IF(nbodf1>ndim1.or.nbodf2>ndim2.or.nbodf3>ndim3)THEN
	  WRITE(*,*)' error: space dimension exceeded, STOP'
	  STOP
	ENDIF
	READ(10,*)(far(i),i=1,nbodf1)
	READ(10,*)(fat(i),i=1,nbodf2)
	READ(10,*)(faz(i),i=1,nbodf3)
	DO  i=1,nbodf1
		DO  j=1,nbodf2
			DO  k=1,nbodf3
	  		READ(10,*)ftemp(i,j,k),fpress(i,j,k),fdens(i,j,k),fne(i,j,k),&
     			fvr(i,j,k),fvt(i,j,k),fvz(i,j,k),fvtrb(i,j,k)
			ENDDO
		ENDDO
	ENDDO
END SUBROUTINE smod2

SUBROUTINE deflsg(nbod1,nbod2,nbod3,&
                rmodx,rmody,rmodz,gainx,gainy,gainz,ar,at,az)
!      definition of the line of sight grid

 USE param

 IMPLICIT NONE

 INTEGER::nbod1,nbod2,nbod3
 INTEGER::i,n2
 REAL(KIND=8),DIMENSION(ndim1)::ar
 REAL(KIND=8),DIMENSION(ndim2)::at
 REAL(KIND=8),DIMENSION(ndim3)::az
 REAL(KIND=8):: rmodx,rmody,rmodz,gainx,gainy,gainz
 REAL(KIND=8)::r1,rn,rstep,t1,tn,tstep,z1,zn,st1,zstep

   IF(gainx==1.d0)THEN
          r1=-rmodx
 	  rn=rmodx 	
          rstep=(rn-r1)/dble(nbod1-1)
         DO  i=1,nbod1
            ar(i)=r1+rstep*dble(i-1)
	 ENDDO
   ELSE
   	  n2=nbod1/2
   	  st1=rmodx*(1.d0-gainx)/(1.d0-gainx**n2)
   	  ar(n2+1)=0.d0
   	  DO i=1,n2  
   	    ar(n2+1+i)=st1*(1.d0-gainx**i)/(1.d0-gainx)
   	    ar(n2+1-i)=-st1*(1.d0-gainx**i)/(1.d0-gainx)
	  ENDDO
   ENDIF 	
   IF(gainy==1.d0)THEN
          t1=-rmody
          tn=rmody
          tstep=(tn-t1)/dble(nbod2-1)
          DO i=1,nbod2
            at(i)=t1+tstep*dble(i-1)
	  ENDDO	
   ELSE
          n2=nbod2/2
          st1=rmody*(1.d0-gainy)/(1.d0-gainy**n2)
          at(n2+1)=0.d0
          DO i=1,n2  
            at(n2+1+i)=st1*(1.d0-gainy**i)/(1.d0-gainy)
            at(n2+1-i)=-st1*(1.d0-gainy**i)/(1.d0-gainy)
	  ENDDO
    ENDIF
     IF(gainz==1.d0)THEN
          z1=-rmodz
          zn=rmodz
          zstep=(zn-z1)/dble(nbod3-1)
          DO i=1,nbod3
            az(i)=z1+zstep*dble(i-1)       
         ENDDO
      ELSE
          n2=nbod3/2
          st1=rmodz*(1.d0-gainz)/(1.d0-gainz**n2)
          az(n2+1)=0.d0
          DO i=1,n2  
            az(n2+1+i)=st1*(1.d0-gainz**i)/(1.d0-gainz)
            az(n2+1-i)=-st1*(1.d0-gainz**i)/(1.d0-gainz)
	  ENDDO
      ENDIF
END SUBROUTINE deflsg


 FUNCTION gaunt(i,fr)
!	taken from the Syspec code:
!       Hubeny I., Lanz T., Jeffery C.S., 1994, in Newsletter on Analysis
!       of Astronomical spectra No.20, ed. C.S. Jeffery (CCP7; St. Andrews:
!       St. Andrews Univ.), 30
!
!       Hydrogenic bound-free Gaunt factor for the principal quantum
!     number I and frequency FR
!
  IMPLICIT NONE

  INTEGER:: i
  REAL(KIND=8):: gaunt
  REAL(KIND=8):: x,fr

  x=fr/2.99793D14
        gaunt=1.D0
  SELECT CASE (i)
     CASE (1)
        gaunt=1.2302628D0+x*(-2.9094219D-3+x*(7.3993579D-6-8.7356966D-9*x))&
              +(12.803223D0/x-5.5759888D0)/x
     CASE (2)
        gaunt=1.1595421D0+x*(-2.0735860D-3+2.7033384D-6*x)+(-1.2709045D0+&
              (-2.0244141D0/x+2.1325684D0)/x)/x
     CASE (3)
        gaunt=1.1450949D0+x*(-1.9366592D-3+2.3572356D-6*x)+(-0.55936432D0+&
              (-0.23387146D0/x+0.52471924D0)/x)/x
     CASE (4)
        gaunt=1.1306695D0+x*(-1.3482273D-3+x*(-4.6949424D-6+2.3548636D-8*x))&
              +(-0.31190730D0+(0.19683564D0-5.4418565D-2/x)/x)/x
     CASE (5)
        gaunt=1.1190904D0+x*(-1.0401085D-3+x*(-6.9943488D-6+2.8496742D-8*x))&
              +(-0.16051018D0+(5.5545091D-2-8.9182854D-3/x)/x)/x
     CASE (6)
        gaunt=1.1168376D0+x*(-8.9466573D-4+x*(-8.8393133D-6+3.4696768D-8*x))&
              +(-0.13075417D0+(4.1921183D-2-5.5303574D-3/x)/x)/x
     CASE (7)
        gaunt=1.1128632D0+x*(-7.4833260D-4+x*(-1.0244504D-5+3.8595771D-8*x))&
              +(-9.5441161D-2+(2.3350812D-2-2.2752881D-3/x)/x)/x
     CASE (8)
        gaunt=1.1093137D0+x*(-6.2619148D-4+x*(-1.1342068D-5+4.1477731D-8*x))&
              +(-7.1010560D-2+(1.3298411D-2 -9.7200274D-4/x)/x)/x
     CASE (9)
        gaunt=1.1078717D0+x*(-5.4837392D-4+x*(-1.2157943D-5+4.3796716D-8*x))&
              +(-5.6046560D-2+(8.5139736D-3-4.9576163D-4/x)/x)/x
     CASE (10)
        gaunt=1.1052734D0+x*(-4.4341570D-4+x*(-1.3235905D-5+4.7003140D-8*x))&
              +(-4.7326370D-2+(6.1516856D-3-2.9467046D-4/x)/x)/x
     CASE DEFAULT
  END SELECT
    
 END FUNCTION gaunt


 FUNCTION gfree(t,fr)
!	taken from the Syspec code:
!       Hubeny I., Lanz T., Jeffery C.S., 1994, in Newsletter on Analysis
!       of Astronomical spectra No.20, ed. C.S. Jeffery (CCP7; St. Andrews:
!       St. Andrews Univ.), 30
!
!	Hydrogenic free-free Gaunt factor, for temperature T and
!	frequency FR
!
  IMPLICIT NONE

  REAL(KIND=8):: t,fr,gfree
  REAL(KIND=8):: x,thet,c1,c2,c3,c4

  thet=5040.4D0/t
  
  IF (thet < 4.d-2) thet=4.d-2
 	x=fr/2.99793D14
  
  IF (x < 1) THEN
     
     IF(x < 0.2D0) X=0.2D0 
     gfree=(1.0823D0+2.98D-2/thet)+(6.7D-3+1.12D-2/thet)/x
  
  ELSE 
     c1=(3.9999187D-3-7.8622889D-5/thet)/thet+1.070192D0
     c2=(6.4628601D-2-6.1953813D-4/thet)/thet+2.6061249D-1
     c3=(1.3983474D-5/thet+3.7542343D-2)/thet+5.7917786D-1
     c4=3.4169006D-1+1.1852264D-2/thet
     gfree=((c4/x-c3)/x+c2)/x+c1
  END IF

 END FUNCTION gfree  

 SUBROUTINE locate(xx,n,x,j)
!
!	taken from Numerical recipes, 
!	search an ordered table by bisection
  IMPLICIT NONE
 
  INTEGER:: j,n,jl,ju,jm
  REAL(KIND=8), DIMENSION(n):: xx
  REAL(KIND=8):: x
 
  jl=0
  ju=n+1

  DO WHILE (ju - jl > 1)

        jm=(ju+jl)/2
        IF ((xx(n) > xx(1)).EQV.(x > xx(jm))) THEN
          jl=jm
        ELSE
          ju=jm
        ENDIF

  END DO

  j=jl
     
 END SUBROUTINE locate

 SUBROUTINE locate2(XX,N,X,J,k)
!	taken from Numerical recipes, 
!	search an ordered table by bisection
 USE param
 
 IMPLICIT NONE

 REAL(KIND=8),DIMENSION(mstarx,mspecx)::xx
 REAL(KIND=8)::x
 INTEGER::j,k,n
 INTEGER::jl,ju,jm

 JL=0
 JU=N+1

 DO WHILE (JU-JL>1)
        JM=(JU+JL)/2
        IF((XX(N,k)>XX(1,k)).EQV.(X>XX(JM,k)))THEN
          JL=JM
        ELSE
          JU=JM
        ENDIF
 ENDDO

 J=JL
 END SUBROUTINE locate2

 SUBROUTINE bcond1(iunt,darkl,cdelta,vdif,nbod,dens,&
     vr,vt,vz,&
     ari,ati,azn,dinc,alpha,&
     istar,rstar,vxstr,vystr,vzstr,dlst,dlst2,irrst,&
     icomp,rcp,xcpr,ycpr,zcpr,vxcpr,vycpr,vzcpr,&
     dlcp,dlcp2,irrcp,xcp,qq,iprint)
!       identifies the last untransparent point along the ray and
!	calculates the limb darkening factor IF applicable
!	The limb darkening applies only to 
!	'star' and 'companion' and you must set dlst=dlcp=0.e0 IF you
!	want to use their density intervals for other objects
!	(without limb dakening). 
!       input:  nbod -number of points
!               dens - density
!       	dcut1,2,3,n -density interval boundaries
!		ari,ati -x,y coordinates of the ray (line of sight)
!		rstar -radius of the star
!		dlst -limb darkening coefficient of the 'star'
!		rcp -radius of the 'companion'
!		xcpr,ycpr -position of the center of the 'companion'
!			corrected for the rotation
!		dlcp -limb darkening coef. of the 'companion'
!       output: iunt -number of the last untransparent point
!		darkl -limb darkening factor
!              vxstr,vystr,vzstr
 USE sun
 USE param

 IMPLICIT NONE

  INTEGER:: i,irrst,icomp,irrcp,iprint,istar
  INTEGER:: nbod,iunt
  REAL(KIND=8), DIMENSION(ndim):: dens,azn,vr,vt,vz
  REAL(KIND=8)::dlst,dlcp,dlcp2,xcpr,ycpr,zcpr,ari,ati
  REAL(KIND=8)::rstar,rcp,darkl
  REAL(KIND=8):: dati,dari,ctheta,sth2
  REAL(KIND=8)::cdelta,vdif,vxstr,vystr,vzstr,vxcpr,vycpr,vzcpr,xcp,qq
  REAL(KIND=8)::cdinc,sdinc,calpha,salpha,dinc,alpha,dlst2
  REAL(KIND=8)::fx,fy,fz,grav,grmin,potx,poty,potz,rr2,rvec,rvecx,rvecy,rvecz
  REAL(KIND=8)::vdif1,vdif2,vdif3,xlos,ylos,zlos

        iunt=0
        darkl=1.d0
        cdelta=0.d0
        vdif=0.d0
!    icut=0
!    grmin -optional small value??
	grmin=1.d-20
        DO i=nbod,1,-1
          IF(dens(i)>dcut1.and.dens(i)<=dcut2)THEN
            iunt=i
	    IF(istar>1)THEN
              cdinc=cos(dinc)
              sdinc=sin(dinc)
              calpha=cos(alpha)
              salpha=sin(alpha)
!	coordinates of the last untransparent point 
!	in the body frozen frame
              CALL rotz(ari,ati,azn(iunt),fx,fy,fz,&
                   cdinc,sdinc,calpha,salpha)
	      fx=fx/xcp
	      fy=fy/xcp
	      fz=fz/xcp	
!	line of sight coordinates in the body frozen frame
              CALL rotz(0.d0,0.d0,1.d0,xlos,ylos,zlos,&
                   cdinc,sdinc,calpha,salpha)
!	gravity vector in the last untransparent point
	      CALL gravn(fx,fy,fz,qq,potx,poty,potz,grav)
	      IF(grav<grmin)THEN
		ctheta=0.d0
	      ELSE
 	        ctheta=-(xlos*potx+ylos*poty+zlos*potz)/grav
		IF(ctheta<0.d0)ctheta=0.d0
	        ENDIF
! 		reflection off the roche surface
	      IF(irrst==1.and.istar==2)THEN
		rvec=(fx-1.d0)**2+fy**2+fz**2
		IF(grav<grmin)THEN
		  cdelta=1.d0
		ELSE
		  cdelta=((fx-1.d0)*potx+fy*poty+fz*potz)
		  cdelta=cdelta/sqrt(rvec)/grav
		ENDIF
	        cdelta=cdelta*(rcp/xcp)**2/rvec
	      ELSE
	        cdelta=0.d0
	      ENDIF	
	      vdif=0.d0
	    ELSE	
              sth2=(ari*ari+ati*ati)/(rstar*rstar)
              IF(sth2<1.d0)THEN
                ctheta=sqrt(1.d0-sth2)
              ELSE
                ctheta=0.d0
              ENDIF
!	reflection off the sphere
	      IF(irrst==1.and.istar==1)THEN
	        cdelta=ari*xcpr+ati*ycpr+azn(iunt)*zcpr
	        cdelta=cdelta/sqrt(ari**2+ati**2+azn(iunt)**2)
	        cdelta=cdelta/sqrt(xcpr**2+ycpr**2+zcpr**2)
	        rr2=(ari-xcpr)**2+(ati-ycpr)**2+(azn(iunt)-zcpr)**2
	        cdelta=cdelta*rcp**2/rr2
     	        vdif1=(ari-xcpr)*(vr(iunt)-vxcpr)
     	        vdif2=(ati-ycpr)*(vt(iunt)-vycpr)
     	        vdif3=(azn(iunt)-zcpr)*(vz(iunt)-vzcpr)
     	        vdif=-(vdif1+vdif2+vdif3)/sqrt(rr2)	        
	      ELSE
	        cdelta=0.d0
		vdif=0.d0
	      ENDIF
	    ENDIF
            darkl=1.d0-dlst*(1.d0-ctheta)-dlst2*(1.d0-ctheta)**2
	RETURN
!        icut=1
          ELSEIF(dens(i)>dcut2.and.dens(i)<=dcut3)THEN
            iunt=i
	    IF(icomp>1)THEN
              cdinc=cos(dinc)
              sdinc=sin(dinc)
              calpha=cos(alpha)
              salpha=sin(alpha)
!	coordinates of the last untransparent point 
!	in the body frozen frame
              CALL rotz(ari,ati,azn(iunt),fx,fy,fz,&
     cdinc,sdinc,calpha,salpha)
	      fx=fx/xcp
	      fy=fy/xcp
	      fz=fz/xcp	
!	line of sight coordinates in the body frozen frame
              CALL rotz(0.d0,0.d0,1.d0,xlos,ylos,zlos,&
     cdinc,sdinc,calpha,salpha)
!	gravity vector in the last untransparent point
              CALL gravn(fx,fy,fz,qq,potx,poty,potz,grav)
	      IF(grav<grmin)THEN
		ctheta=0.d0
	      ELSE
 	        ctheta=-(xlos*potx+ylos*poty+zlos*potz)/grav
		IF(ctheta<0.d0)ctheta=0.d0
	      ENDIF
! 		reflection off the roche surface
	      IF(irrcp==1.and.icomp==2)THEN
		rvec=fx**2+fy**2+fz**2
		IF(grav<grmin)THEN
		  cdelta=1.d0
		ELSE
		  cdelta=(fx*potx+fy*poty+fz*potz)/sqrt(rvec)/grav
		ENDIF
	        cdelta=cdelta*(rstar/xcp)**2/rvec
	      ELSE
	        cdelta=0.d0
	      ENDIF	
	      vdif=0.d0
	    ELSE	
	      dari=ari-xcpr
	      dati=ati-ycpr
	      sth2=(dari*dari+dati*dati)/(rcp*rcp)
	      IF(sth2<1.d0)THEN
	        ctheta=sqrt(1.d0-sth2)
	      ELSE
	        ctheta=0.d0
	      ENDIF
!	reflection off the sphere
	    IF(irrcp==1.and.icomp==1)THEN
	        rvecx=ari-xcpr
	        rvecy=ati-ycpr
	        rvecz=azn(iunt)-zcpr
	        cdelta=-rvecx*xcpr-rvecy*ycpr-rvecz*zcpr
	        cdelta=cdelta/sqrt(rvecx**2+rvecy**2+rvecz**2)
	        cdelta=cdelta/sqrt(xcpr**2+ycpr**2+zcpr**2)
	        rr2=ari**2+ati**2+azn(iunt)**2	
	        cdelta=cdelta*rstar**2/rr2
     	        vdif1=ari*(vr(iunt)-vxstr)
     	        vdif2=ati*(vt(iunt)-vystr)
     	        vdif3=azn(iunt)*(vz(iunt)-vzstr)
     	        vdif=-(vdif1+vdif2+vdif3)/sqrt(rr2)	      	        
	      ELSE
	        cdelta=0.d0
	        vdif=0.d0
	      ENDIF
	    ENDIF
	    darkl=1.d0-dlcp*(1.d0-ctheta)-dlcp2*(1.d0-ctheta)**2	    
	  RETURN
!        icut=2
          ELSEIF(dens(i)>dcut3.and.dens(i)<=dcutn)THEN
            iunt=i
	    darkl=1.d0
	   RETURN
!        icut=3
          ELSEIF(dens(i)>dcutn)THEN
            iunt=i
	    darkl=1.d0
	  RETURN
!        icut=4
          ENDIF
	ENDDO
 END SUBROUTINE bcond1

 SUBROUTINE bcond2(iunt,dens,temp,vz,vdif,freq,&
    darkl,cdelta,&
     lunt1,nstar1,xstar1,star1,nspec1,tspec1,alb1x,alb1y,nalb1,&
     tstar,dlst,dlst2,irrst,albst,wstar1,fstar1,jstar1,&
     lunt2,nstar2,xstar2,star2,nspec2,tspec2,alb2x,alb2y,nalb2,&
     tempcp,dlcp,dlcp2,irrcp,albcp,wstar2,fstar2,jstar2,&
     lunt3,nstar3,xstar3,star3,aint0,aintb)
!Calculation of the boundary condition of intensity behind 
!the last untransparent object along the line of sight
!input:  
!      iunt-last untransparent point
!  dens,temp -density, temperature
!  vz - radial velocity field
!      vdif -differential radial velocity between the reflecting 
!            surface and the source of light
!  freq - frequency at which we are about to solve RTE
!  dcut1,dcut2,dcut3,dcutn -boundaries of density intervals
!  darkl-limb darkening factor of the particular ray
!      cdelta -cos(irradiation angle)*R**2/separation**2
!  lunt1,nstar1,xstar1,star1,nspec1,tspec1 -refere to 
!	'star' i.e. <dcut1,dcut2>
!  tstar -polar temperature of the 'star'
!  dlst - limb darkening coefficients of the `star`
!  irrst -irradiation switch
!  albst -Bond albedo of the `star`
!  alb1x,alb1y,nalb1 -monochromatic albedo of the 'star'
!  wstar1,fstar1,jstar1 -spectrum of the star with temp=tstar
!  lunt2,nstar2,xstar2,star2,nspec2,tspec2 -refere to 
! 		'companion' i.e. <dcut2,dcut3>
!  tempcp -polar temperature of the 'companion'
!      dlcp - limb darkening coefficients of the 'companion'
!  irrcp -irradiation switch
!      albcp -Bond albedo of the `companion`
!  alb2x,alb2y,nalb2 -monochromatic albedo of the 'companion'
!      wstar2,fstar2,jstar2 -spectrum of `companion` with temp=tempcp
!  lunt3,nstar3,xstar3,star3 -refere to <dcut3,dcutn>
!  aint0 -incident intensity from behind the shell
!output:
!  aintb- boundary condition for intensity, intensity heading 
! towards you along the line of sight after the untransparent,star2 
! object
	USE sun
	USE param
	USE constants
	IMPLICIT NONE


	REAL(KIND=8),DIMENSION(ndim):: temp,dens,vz
	REAL(KIND=8),DIMENSION(mstarx,mspecx)::xstar1,star1,xstar2,star2
	REAL(KIND=8),DIMENSION(mstarx):: xstar3,star3, wstar1,fstar1,wstar2,fstar2,alb1x,alb1y,alb2x,alb2y
	REAL(KIND=8),DIMENSION(mspecx)::tspec1,yy,tspec2
	INTEGER,DIMENSION(mspecx)::nstar1,nstar2
	INTEGER::j,jj,iunt,nspec1,nalb1,irrst,jstar1,nspec2,nalb2,irrcp,jstar2,nstar3,lunt3,lunt2,lunt1
	REAL(KIND=8)::aintb,vdif,freq,darkl,cdelta
	REAL(KIND=8)::tstar,dlst,dlst2,albst,tempcp,dlcp,dlcp2
	REAL(KIND=8)::albcp,aint0,alamr,albnu,bunt,der,freqr,freqs,alamf,buntr,planck
	aintb=aint0
!    Four types of untransparent objects can be recognized here:
!    dcut1<dens<dcut2 -central star,
!    dcut2<dens<dcut3 -secondary star(=companion) 
!    dcut3<dens<dcutn -3.body (it can be anything)
!    dcutn<dens -any opaque dark matter
!Note that lunt1, lunt2, lunt3 are in fact associated with 
!particular density intervals rather THEN with objects. 
!    They can be used to ascribe the spectrum to any untransparent 
!    object setting its density within a particular density interval.
!Ba cautious to turn off the limb darkening in that case.
	IF(iunt>0)THEN
  	  freqs=freq*(1.d0-vz(iunt)/clight)
	  freqr=freq*(1.d0-(vdif+vz(iunt))/clight)
!  behind the primary
	  IF(dens(iunt)>dcut1.and.dens(iunt)<=dcut2)THEN
	    IF(lunt1>0)THEN
	      alamf=clight/freqs*1.d8	
	      DO j=1,nspec1
	        CALL locate2(xstar1,nstar1(j),alamf,jj,j)	        
	        IF(jj<1)THEN
	          yy(j)=star1(1,j)
	        ELSEIF(jj>=nstar1(j))THEN
	          yy(j)=star1(nstar1(j),j)
	        ELSE
	          der=(star1(jj+1,j)-star1(jj,j))&
     	          /(xstar1(jj+1,j)-xstar1(jj,j))
	          yy(j)=(alamf-xstar1(jj,j))*der+star1(jj,j)
	        ENDIF
              ENDDO
              CALL locate(tspec1,nspec1,temp(iunt),jj)
	      IF(jj<1)THEN
                bunt=yy(1)*darkl
              ELSEIF(jj>=nspec1)THEN 
                bunt=yy(nspec1)*darkl
              ELSE
                der=(yy(jj+1)-yy(jj))/(tspec1(jj+1)-tspec1(jj))
                bunt=(temp(iunt)-tspec1(jj))*der+yy(jj)
                bunt=bunt*darkl                
              ENDIF
            ELSE
              bunt=planck(freqs,temp(iunt),1)*darkl
              bunt=bunt/(1.d0-dlst/3.d0-dlst2/6.d0)
	    ENDIF
!	reflection off the primary
	    IF(irrst==1.and.albst>0.d0.and.cdelta>0.d0)THEN
	      IF(nalb1>1)THEN
                alamf=clight/freqs*1.d8
!             WRITE(*,*)'before locate alb1x',(alb1x(i),i=1,nalb1)
	        CALL locate(alb1x,nalb1,alamf,jj)
	        IF(jj<1)THEN
	          albnu=alb1y(1)
	        ELSEIF(jj>=nalb1)THEN
	          albnu=alb1y(nalb1)
	        ELSE
	          der=alb1y(jj+1)-alb1y(jj)
	          der=der/(alb1x(jj+1)-alb1x(jj))
	          albnu=(alamf-alb1x(jj))*der+alb1y(jj)
	        ENDIF
	      ELSE
	        albnu=albst
	      ENDIF
	    ENDIF
            IF(lunt2>0)THEN
              IF(irrst==1.and.albst>0.d0.and.cdelta>0.d0)THEN
	        alamr=clight/freqr*1.d8              
                CALL locate(wstar2,jstar2,alamr,jj)
                IF(jj<1)THEN
                  buntr=fstar2(1)
                ELSEIF(jj>=jstar2)THEN
                  buntr=fstar2(jstar2)
                ELSE
                  der=(fstar2(jj+1)-fstar2(jj))&
                /(wstar2(jj+1)-wstar2(jj))
                  buntr=(alamr-wstar2(jj))*der+fstar2(jj)
                ENDIF
                buntr=albnu*cdelta*buntr*(1.d0-dlcp/3.d0-dlcp2/6.d0)
              ELSE
                buntr=0.d0
              ENDIF  
            ELSE
              IF(irrst==1.and.albst>0.d0.and.cdelta>0.d0)THEN
                buntr=albnu*cdelta*planck(freqr,tempcp,1)
              ELSE
                buntr=0.d0
              ENDIF  
            ENDIF
            bunt=bunt+buntr
!      behind the secondary
	  ELSEIF(dens(iunt)>dcut2.and.dens(iunt)<=dcut3)THEN
	    IF(lunt2>0)THEN
	      alamf=clight/freqs*1.d8	
! 	      darkg=planck(freqs,temp(iunt),1)/planck(freqs,tempcp,1)
              DO j=1,nspec2
  	        CALL locate2(xstar2,nstar2(j),alamf,jj,j)
	        IF(jj<1)THEN
	          yy(j)=star2(1,j)
	        ELSEIF(jj>=nstar2(j))THEN
	          yy(j)=star2(nstar2(j),j)
	        ELSE
	          der=(star2(jj+1,j)-star2(jj,j))&
               /(xstar2(jj+1,j)-xstar2(jj,j))
	          yy(j)=(alamf-xstar2(jj,j))*der+star2(jj,j)
	        ENDIF
              ENDDO
              CALL locate(tspec2,nspec2,temp(iunt),jj)
              IF(jj<1)THEN
!          IF(temp(iunt).lt.tspec2(1))THEN              
                bunt=yy(1)*darkl
              ELSEIF(jj>=nspec2)THEN
!          elseif(temp(iunt).ge.tspec2(nspec2))THEN              
                bunt=yy(nspec2)*darkl
              ELSE
                der=(yy(jj+1)-yy(jj))/(tspec2(jj+1)-tspec2(jj))
                bunt=(temp(iunt)-tspec2(jj))*der+yy(jj)
                bunt=bunt*darkl
              ENDIF              
	    ELSE
              bunt=planck(freqs,temp(iunt),1)*darkl
              bunt=bunt/(1.d0-dlcp/3.d0-dlcp2/6.d0)
	    ENDIF
!	reflection off the secoondary
	    IF(irrcp==1.and.albcp>0.d0.and.cdelta>0.d0)THEN
	      IF(nalb2>1)THEN
                alamf=clight/freqs*1.d8
	        CALL locate(alb2x,nalb2,alamf,jj)
	        IF(jj<1)THEN
	          albnu=alb2y(1)
	        ELSEIF(jj>=nalb2)THEN
	          albnu=alb2y(nalb2)
	        ELSE
	          der=alb2y(jj+1)-alb2y(jj)
	          der=der/(alb2x(jj+1)-alb2x(jj))
	          albnu=(alamf-alb2x(jj))*der+alb2y(jj)
	        ENDIF
	      ELSE
	        albnu=albcp
	      ENDIF
	    ENDIF
            IF(lunt1>0)THEN
              IF(irrcp==1.and.albcp>0.d0.and.cdelta>0.d0)THEN
	        alamr=clight/freqr*1.d8              
                CALL locate(wstar1,jstar1,alamr,jj)
                IF(jj<1)THEN
                  buntr=fstar1(1)
                ELSEIF(jj>=jstar1)THEN
                  buntr=fstar1(jstar1)
                ELSE
                  der=(fstar1(jj+1)-fstar1(jj))&
                /(wstar1(jj+1)-wstar1(jj))
                  buntr=(alamr-wstar1(jj))*der+fstar1(jj)
                ENDIF
                buntr=albnu*cdelta*buntr*(1.d0-dlst/3.d0-dlst2/6.d0)
              ELSE
                buntr=0.d0
              ENDIF  
            ELSE
              IF(irrcp==1.and.albcp>0.d0.and.cdelta>0.d0)THEN
                buntr=albnu*cdelta*planck(freqr,tstar,1)
              ELSE
                buntr=0.d0
              ENDIF  
            ENDIF
            bunt=bunt+buntr
!      behind the third body
	  ELSEIF(dens(iunt)>dcut3.and.dens(iunt)<=dcutn)THEN
	    IF(lunt3>0)THEN
	      alamf=clight/freqs*1.d8	
	      CALL locate(xstar3,nstar3,alamf,jj)
	      IF(jj<1)THEN
	        bunt=star3(1)
	      ELSEIF(jj>=nstar3)THEN
	        bunt=star3(nstar3)
	      ELSE
	        der=(star3(jj+1)-star3(jj))/(xstar3(jj+1)-xstar3(jj))
	        bunt=(alamf-xstar3(jj))*der+star3(jj)
	      ENDIF
	    ELSE
              bunt=planck(freqs,temp(iunt),1)
	    ENDIF
	  ELSE
            bunt=0.d0
	  ENDIF	
          aintb=bunt
	ENDIF
END SUBROUTINE bcond2

SUBROUTINE jnu(rstar,tstar,ari,ati,az,&
     vr,vt,vz,freq,xstar1,star1,nstar1,lunt1,&
     vxstr,vystr,vzstr,jj,ejnu,iprint,xcpr,ycpr,zcpr)
!
!	input:
!	  rstar,tstar,lunt1,ari,ati,az,vr,vt,vz,freq
!	  xstar1,star1,nstar1,vxstr,vystr,vzstr,iprint
!	output:
!		ejnu = mean intensity = I_nu*omega/4/pi
!		jj- input for subseqent CALL of hunt
!
 USE constants
 USE param
 IMPLICIT NONE 


 INTEGER:: jj,nstar1,iprint,lunt1  
 REAL(KIND=8)::ari,ati,az,vr,vt,vz
 REAL(KIND=8)::freq,freqs,omega4,ratio2,er2,dx,dz,dy
 REAL(KIND=8)::rdotv,alamf,bstar,der,dvel,ejnu
 REAL(KIND=8)::vxstr,vystr,vzstr,rstar,tstar
 REAL(KIND=8)::xcpr,ycpr,zcpr
 REAL(KIND=8):: planck
 REAL(KIND=8), DIMENSION(mstarx):: xstar1,star1       
 dx=ari-xcpr
 dy=ati-ycpr
 dz=az-zcpr
  er2=ari*ari+ati*ati+az*az
  ratio2=rstar*rstar/er2

  IF (ratio2 < 1.d0) THEN	
	  omega4=0.5d0*(1.d0-SQRT(1.d0-ratio2))
  ELSE
          omega4=0.d0
  ENDIF
 
  rdotv=(ari*(vr-vxstr)+ati*(vt-vystr)+az*(vz-vzstr))/SQRT(er2)
  dvel=-rdotv+vz
!		freqs -frequency of alam in comoving frame
  freqs=-dvel/clight*freq+freq
!		in ejnu the rotation of the star is ignored so far
  IF (lunt1 > 0) THEN
      alamf=clight/freqs*1.d8	
      CALL hunt(xstar1,nstar1,alamf,jj)

      IF (jj < 1) THEN
	      bstar=star1(1)
      ELSE IF (jj >= nstar1) THEN
	      bstar=star1(nstar1)
           ELSE
	      der=(star1(jj+1)-star1(jj))/(xstar1(jj+1)-xstar1(jj))
	      bstar=(alamf-xstar1(jj))*der+star1(jj)
           ENDIF
      ELSE
	  bstar=planck(freqs,tstar,1)
      ENDIF
   ejnu=bstar*omega4

 END SUBROUTINE jnu 

SUBROUTINE roche(rosu,rotem,areas,teff,&
     rfront,rback,rpole,rside,rmean,&	
     tstar,rstar,dgst,irrst,albst,htst,htsta,&
     tempcp,rcp,dgcp,irrcp,albcp,htcp,htcpa,&
     x,y,xcomp,q,ff,nbod1,nbod2,ipsc)
!
!	Calculates the Roche surface (RS) and normalized gravity on RS
!	of the primary, secondary or contact system. 
!	Code works in normalized scale where center of 
!	mass of the primary is at x=0, center of mass of the secondary 
!	at x=1 but the input/output is scaled/unscaled. 
!		input: 
!	x,y -grid points
!	nbod1,nbod2 -number of grid points
!	xcomp -is the scale=x coordinate of the secondary 
!	q -mass ratio (q<1 -more massive star is in the center,
!		q>1 -less massive star is in the center)
!	ff -Roche lobe filling factor of the primary. Its is 
!	   the distance of the inner substellar point of the primary 
!	   (between the stars) from the center of the primary relative 
!	   to the distance to L1, 
!	   ff<=1, the Roche lobe is reproduced IF ff=1 
!	ff -Roche lobe filling factor of the secondary is the distance 
!	   of the inner substellar point of the secondary 
!	   from the center of the secondary relative to 1-L1, 
!	   ff<=1, the Roche lobe is reproduced IF ff=1 
!	ff -Roche lobe fill-out factor of the contact system
!		ff=(P1-P)/(P1-P2)+1
!	ipsc=1 -calculates Roche surface of detached primary
!	ipsc=2 -calculates Roche surface of detached secondary
!	ipsc=3 -calculates Roche surface of contact system
!		output: 
!	rosu(x,y) -Roche surface
!	rogrv(x,y) -normalized gravity on the RS relative to the gravity
!		on the rotation pole (of more massive star in case of
!		contact system)
 USE param
 USE constants
 IMPLICIT NONE


 INTEGER:: ipsc,i,irrcp,irrst
 INTEGER:: nbod1,nbod2
 INTEGER:: j
 REAL(KIND=8):: prc,pinn,pout,rpot,xcomp,ff,el1,el2,el3,q
 REAL(KIND=8):: ystart,xpole,ypole,zpole 
 REAL(KIND=8):: pot,dxpot,dypot,dzpot,dxxpot
 REAL(KIND=8):: raphx,raphy,raphz,radlob
 REAL(KIND=8)::teff,rfront,rback,rpole,rside,rmean,&	
   				  tstar,rstar,dgst,albst,htst,htsta,&
			          tempcp,rcp,dgcp,albcp,htcp,htcpa,areas
 REAL(KIND=8)::coslat,firr,fnlat,fxy2,gravec,gravnp,htb,htbnew,&
				pom,potel1,potel2,potel3,sum1,sum2,surf1,&
				potx,poty,potz,rr2,rvec,sinn,sout,surf2,t04,tirr4,vol,&		
				t04new
 REAL(KIND=8), DIMENSION(ndim1):: x,xn,rosuy
 REAL(KIND=8), DIMENSION(ndim2):: y,yn
 REAL(KIND=8), DIMENSION(ndim1,ndim2):: rosu,rogrv,rocos,rotem,area,areaz,arear 

 EXTERNAL pot,dxpot,dypot,dzpot,dxxpot

  IF ((ipsc == 1 .or. ipsc == 2).and.(ff > 1.d0 .or. ff < 1.d-2)) THEN
	  WRITE(*,*)' error: inconsistent input in subroutine: roche1'
          STOP	 
  ENDIF 

  IF (ipsc == 3 .and. (ff <= 1.d0 .or. ff > 2.d0)) THEN
	  WRITE(*,*)' error: inconsistent input in subroutine: roche2'
	  STOP
  ENDIF
	
  IF (.not.(ipsc == 1 .or. ipsc == 2 .or. ipsc == 3)) THEN
	  WRITE(*,*)' error: inconsistent input in subroutine: roche3'
	  STOP
  ENDIF
  IF(htst<0.d0 .or. htst>1.d0 .or.&
     	  htsta<0.d0 .or. htsta>1.d0 .or.&
     	  htcp<0.d0 .or. htcp>1.d0 .or.&
     	  htcpa<0.d0 .or. htcpa>1.d0)THEN
	  WRITE(*,*)' error: inconsistent input in subroutine: roche4'
	  STOP
  ENDIF
!	precision required
  prc=1.d-5
!	normalization of coordinates
  DO i=1,nbod1
          xn(i)=x(i)/xcomp
  END DO
        
  DO i=1,nbod2
          yn(i)=y(i)/xcomp
  END DO
!
!		el1- the x coordinate of the L1 point 
!	it may be quicker to start with 0.5 THEN with radlob function

  el1=raphx(dxpot,dxxpot,0.d0,q,0.5d0,prc)

!       	el2,el3 -the L2,L3 points (they are reversed IF q>1)

  el2=raphx(dxpot,dxxpot,0.d0,q,2.0d0-el1,prc)
  el3=raphx(dxpot,dxxpot,0.d0,q,-el1,prc)

!
!			primary, ff<=1
!
  IF (ipsc == 1.and.ff > 0.d0.and.ff <= 1.d0) THEN
     pinn=ff*el1
     pout=-pinn
     rpot=pot(pinn,0.d0,0.d0,q)

!	  pinn-the inner substellar point of the surface of the primary
!	  pout-the outer point of the primary on the averted hemisphere
	  IF (pout > el3) THEN
   	     pout=pout
          ELSE
	     pout=el3*0.5d0
	  END IF
   
  pout=raphx(pot,dxpot,rpot,q,pout,prc)
!
!	  rosuy(x)-the shape of the primary Roche surface for x=x(i),z=0
!
 	  ystart=pinn*0.5d0	 
 DO i=1,nbod1
	    IF(xn(i)>pout.and.xn(i)<pinn)THEN
	      rosuy(i)=abs(raphy(pot,dypot,rpot,q,xn(i),ystart,prc))
	      ystart=rosuy(i)
	    ELSE
	      rosuy(i)=0.d0
	    ENDIF
 ENDDO
!	  coordinates of the rotation pole
	  xpole=0.d0
	  ypole=0.d0
!	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,radlob(1.d0/q),prc)
	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,abs(pout),prc)	  
	  rfront=pinn
	  rback=-pout
	  rpole=zpole
	  rside=abs(raphy(pot,dypot,rpot,q,0.d0,pout,prc))
 ENDIF
!
!			contact, 1<ff<=2
 IF(ipsc==3.and.ff>1.d0.and.ff<2.d0)THEN
	  potel1=pot(el1,0.d0,0.d0,q)	
	  potel2=pot(el2,0.d0,0.d0,q)	
	  potel3=pot(el3,0.d0,0.d0,q)	
	  IF(potel2<potel3)THEN
	    pom=potel2
	    potel2=potel3
	    potel3=pom
	  ENDIF
	  rpot=potel1-(ff-1.d0)*(potel1-potel2)
	  pout=raphx(pot,dxpot,rpot,q,el3*0.5d0,prc)
	  pinn=raphx(pot,dxpot,rpot,q,0.5d0+el2*0.5d0,prc)
!	  pinn-the right hand side edge of the contact Roche surface
!	  pout-the left hand side edge of the contact Roche surface
!	  rosuy(x)-the shape of the contact Roche surface for x=x(i),z=0
	  ystart=radlob(1.d0/q)*0.2d0
 	DO i=1,nbod1
	    IF(xn(i)>pout.and.xn(i)<pinn)THEN
	      rosuy(i)=abs(raphy(pot,dypot,rpot,q,xn(i),ystart,prc))
	      ystart=rosuy(i)	
	    ELSE
	      rosuy(i)=0.d0
	    ENDIF
!	    WRITE(3,*)xn(i),rosuy(i),pout
	 ENDDO
!	  coordinates of the rotation pole of the more massive star
 	IF(q<1.d0)THEN
	  xpole=0.d0
	  ypole=0.d0
	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,radlob(1.d0/q),prc)
	  rside=abs(raphy(pot,dypot,rpot,q,xpole,radlob(1.d0/q),prc))	  	  
  	ELSE
	  xpole=1.d0
	  ypole=0.d0
	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,radlob(q),prc)
	  rside=abs(raphy(pot,dypot,rpot,q,xpole,radlob(q),prc))	 
 	ENDIF
	  rfront=pinn
	  rback=-pout
	  rpole=zpole
ENDIF
!
!			secondary, ff<=1
	IF(ipsc==2.and.ff>0.d0.and.ff<1.d0)THEN
	  sout=ff*(1.d0-el1)
	  sinn=1.d0-sout
	  rpot=pot(sinn,0.d0,0.d0,q)
!	  sinn-inner substellar point of the surface of the secondary
!	  sout-outer point of the secondary on the averted hemisphere
	  IF((1.d0+sout)<el2)THEN
   	    sout=1.d0+sout
	  ELSE
	    sout=1.d0+(el2-1.d0)*0.5d0
	  ENDIF
	  sout=raphx(pot,dxpot,rpot,q,sout,prc)
!	  rosuy(x)-the shape of the Roche surface for x=x(i),z=0
	  ystart=(sout-1.d0)*0.2d0
	  DO i=1,nbod1
	    IF(xn(i)>sinn.and.xn(i)<sout)THEN
	      rosuy(i)=abs(raphy(pot,dypot,rpot,q,xn(i),ystart,prc))
	      ystart=rosuy(i)
	    ELSE
	      rosuy(i)=0.d0
	    ENDIF
	  ENDDO
!	  coordinates of the rotation pole
	  xpole=1.d0
	  ypole=0.d0
	  zpole=radlob(q)*ff
	  zpole=raphz(pot,dzpot,rpot,q,xpole,ypole,zpole,prc)
	  rfront=1.d0-sinn
	  rback=sout-1.d0
	  rpole=zpole
	  rside=abs(raphy(pot,dypot,rpot,q,1.d0,zpole,prc))
	ENDIF
!	check mainly this
  	DO i=1,nbod1
	  IF(rosuy(i)>0.d0.and.&
         dypot(xn(i),rosuy(i),0.d0,q)>=0.d0)THEN
	    WRITE(*,*)' Error: Newton-Raphson slipped in subr.: Roche,',&
           ' try to modify slightly: nbod,q,ff'
	  STOP
	  ENDIF
	ENDDO
!
!	gravnp-normalized gravity at the rotation pole
	call gravn(xpole,ypole,zpole,q,potx,poty,potz,gravnp)
!		rosu(x,y)-the shape of the Roche surface
!		rogrv(x,y) -normalized gravity relative to the pole
!		rocos(i,j) -cosine of the irradiation angle
	DO  i=1,nbod1
          DO  j=1,nbod2
            IF(abs(yn(j))<rosuy(i))THEN
	      rosu(i,j)=raphz(pot,dzpot,rpot,q,xn(i),yn(j),rosuy(i),prc)
	      call gravn(xn(i),yn(j),rosu(i,j),q,potx,poty,potz,gravec)
	      rogrv(i,j)=gravec/gravnp
	      areaz(i,j)=abs(potz/gravec)
	      IF(ipsc==1)THEN
	        rvec=sqrt((xn(i)-1.d0)**2+yn(j)**2+rosu(i,j)**2)
	        rocos(i,j)=((xn(i)-1.d0)*potx+yn(j)*poty+rosu(i,j)*potz)&
     	        /rvec/gravec
       	        IF(rogrv(i,j)<1.d-4)THEN
	          rogrv(i,j)=1.d-4	
	          rocos(i,j)=0.84d0
	        ENDIF
		rotem(i,j)=tstar*rogrv(i,j)**dgst
	      ELSEIF(ipsc==2)THEN
	        rvec=sqrt(xn(i)*xn(i)+yn(j)*yn(j)+rosu(i,j)*rosu(i,j))
	        rocos(i,j)=(xn(i)*potx+yn(j)*poty+rosu(i,j)*potz)/rvec&
     	        /gravec
!		take care of the unimportant case of singularity
     	        IF(rogrv(i,j)<1.d-4)THEN
	          rogrv(i,j)=1.d-4	
	          rocos(i,j)=0.84d0
	        ENDIF
		rotem(i,j)=tempcp*rogrv(i,j)**dgcp
              ELSE
		IF(rogrv(i,j)<1.d-4)rogrv(i,j)=1.d-4	
                rocos(i,j)=0.d0
		rotem(i,j)=tstar*rogrv(i,j)**dgst
	      ENDIF	 
    	      IF(rocos(i,j)<0.d0)rocos(i,j)=0.d0	
            ELSE
	      rosu(i,j)=0.d0
	      rogrv(i,j)=0.d0
	      rocos(i,j)=0.d0
	      rotem(i,j)=0.d0
	      areaz(i,j)=0.d0
            ENDIF
!		unscale the result
	    rosu(i,j)=xcomp*rosu(i,j)
	  ENDDO
	ENDDO
!	
!		irradiation and heat redistribution
	CALL surf(nbod1,nbod2,x,y,area)
!	area-projection of the surface element to xy
!	arear-area of the surface element
!	vol-volume under the Roche surface
        areas=0.d0
	vol=0.d0
        sum1=0.d0
        sum2=0.d0        
        DO i=1,nbod1  
          DO j=1,nbod2
            IF(abs(yn(j))<rosuy(i))THEN  
              arear(i,j)=area(i,j)/areaz(i,j)
              IF(irrst==1.and.ipsc==1)THEN
                fxy2=x(i)**2+y(j)**2
                coslat=fxy2/(fxy2+rosu(i,j)**2)
                coslat=sqrt(coslat)
                surf2=coslat*arear(i,j)
                IF(rocos(i,j)>0.d0)THEN
!                 dayside
                  rr2=rcp**2/((x(i)-xcomp)**2+y(j)**2+rosu(i,j)**2)
                  firr=stefb*rr2*tempcp**4*rocos(i,j)
		  surf1=firr*arear(i,j)
                ENDIF
              ENDIF
              IF(irrcp==1.and.ipsc==2)THEN
                fxy2=(x(i)-xcomp)**2+y(j)**2
                coslat=fxy2/(fxy2+rosu(i,j)**2)
                coslat=sqrt(coslat)
                surf2=coslat*arear(i,j)
                IF(rocos(i,j)>0.d0)THEN
!                 day-side
                  rr2=rstar*rstar/(x(i)**2+y(j)**2+rosu(i,j)**2)
                  firr=stefb*rr2*tstar**4*rocos(i,j)
                  surf1=firr*arear(i,j)
                ENDIF    
              ENDIF              
              areas=areas+arear(i,j)
              vol=vol+area(i,j)*rosu(i,j)
	      sum1=sum1+surf1
	      sum2=sum2+surf2
            ENDIF
          ENDDO
        ENDDO
        areas=areas*2.d0
        rmean=(vol*2.d0/(4.d0/3.d0*pi))**(1.d0/3.d0)/xcomp
        sum1=sum1*2.d0
        sum2=sum2*2.d0
        IF(irrst==1.and.ipsc==1)THEN
          rr2=rcp**2/xcomp**2
          t04=0.25d0*htst*(1.d0-albst)*rr2*tempcp**4
          t04new=htst*(1.d0-albst)*sum1/areas/stefb
!            t04=htst*(1.d0-albst)*sum1/areas/stefb          
          htb=4.d0*(1.d0-htsta)/pi
          htbnew=(1.d0-htsta)*areas/sum2
!            htb=(1.d0-htsta)*areas/sum2          
        ENDIF  
        IF(irrcp==1.and.ipsc==2)THEN
          rr2=rstar*rstar/xcomp**2  
          t04=0.25d0*htcp*(1.d0-albcp)*rr2*tstar**4            
          t04new=htcp*(1.d0-albcp)*sum1/areas/stefb
!            t04=htcp*(1.d0-albcp)*sum1/areas/stefb          
          htb=4.d0*(1.d0-htcpa)/pi
          htbnew=(1.d0-htcpa)*areas/sum2
!            htb=(1.d0-htcpa)*areas/sum2          
        ENDIF  
!	sum1,t04new may not be precise enough
!	(polar or cylindrical coordinates might be better).
!	However, IF you want to use them simply uncomment 
!	the four lines above and replace
!	t04 by t04new and htb by htbnew.
!	WRITE(*,*)'T0=',t04new**0.25d0,t04**0.25d0
!	WRITE(*,*)'b=',htbnew,htb
!	WRITE(*,*)'sum1,2=',sum1,sum2
	teff=0.d0
	DO i=1,nbod1
          DO j=1,nbod2
            IF(abs(yn(j))<rosuy(i))THEN
              IF(irrst==1.and.ipsc==1)THEN
                fxy2=x(i)**2+y(j)**2
                coslat=fxy2/(fxy2+rosu(i,j)**2)
                coslat=sqrt(coslat)
                fnlat=htsta+htb*coslat
                IF(rocos(i,j)==0.d0)THEN
!                 dayside
                  rr2=rcp**2/((x(i)-xcomp)**2+y(j)**2+rosu(i,j)**2)
                  firr=stefb*rr2*tempcp**4*rocos(i,j)
                  tirr4=(1.d0-htst)*(1.d0-albst)*firr/stefb
                  rotem(i,j)=(tirr4+t04*fnlat+rotem(i,j)**4)
		  rotem(i,j)=sqrt(sqrt(rotem(i,j)))	
                ELSE 
!                nightside             
                  rotem(i,j)=(t04*fnlat+rotem(i,j)**4)
                  rotem(i,j)=sqrt(sqrt(rotem(i,j)))
                ENDIF
              ENDIF
              IF(irrcp==1.and.ipsc==2)THEN
                fxy2=(x(i)-xcomp)**2+y(j)**2
                coslat=fxy2/(fxy2+rosu(i,j)**2)
                coslat=sqrt(coslat)
                fnlat=htcpa+htb*coslat  
                IF(rocos(i,j)>0.d0)THEN
!                 day-side
                  rr2=rstar*rstar/(x(i)**2+y(j)**2+rosu(i,j)**2)
                  firr=stefb*rr2*tstar**4*rocos(i,j)
                  tirr4=(1.d0-htcp)*(1.d0-albcp)*firr/stefb
                  rotem(i,j)=(tirr4+t04*fnlat+rotem(i,j)**4)
                  rotem(i,j)=sqrt(sqrt(rotem(i,j)))
                ELSE
!                 night-side
                  rotem(i,j)=(t04*fnlat+rotem(i,j)**4)
		  rotem(i,j)=sqrt(sqrt(rotem(i,j)))
                ENDIF    
              ENDIF
              teff=teff+arear(i,j)*rotem(i,j)**4
            ENDIF
	  ENDDO
	ENDDO
	teff=(teff*2.d0/areas)**0.25
!
!     	OPEN(4,file='roche.out',status='unknown')
!	DO i=1,nbod1
!          DO j=1,nbod2
!            WRITE(4,180)x(i),y(j),rosu(i,j),rogrv(i,j),rocos(i,j)
!     p      ,rotem(i,j)
!	  ENDDO
!	  WRITE(4,*)
!	ENDDO
!180	format(6e14.5)
!	close(4)	
!


 END SUBROUTINE roche

 FUNCTION raphx(fun,dfun,fun0,q,xstart,xacc)
!	Newton-Raphson Method in 1D (x coordinate)
!	to solve the equation fun(x)=fun0
!	input:  fun-external function of x and parameters: y,z,q
!		dfun-external function, partial derivative df/dx
!		fun0-value of the function 
!		q-mass ratio
!	        xstart-first estimate
!		xacc-accuracy
!	output:  raphx-root for which fun=fun0
 IMPLICIT NONE
   
 INTEGER:: i

 REAL(KIND=8):: raphx
 REAL(KIND=8):: fun,dfun
 REAL(KIND=8):: q,fun0,xstart,xacc,dx

 EXTERNAL fun,dfun  
 
  raphx=xstart
 
  DO i=1,20
	  dx=(fun(raphx,0.d0,0.d0,q)-fun0)/dfun(raphx,0.d0,0.d0,q)
	  raphx=raphx-dx
	  IF (ABS(dx) < xacc) RETURN
  END DO
 
 END FUNCTION raphx

 FUNCTION raphy(fun,dfun,fun0,q,x0,ystart,yacc)
!
!	Newton-Raphson Method in 1D (y coordinate)
!	to solve the equation fun(y)=fun0
!	input:  fun-external function of y and parameters: x,z,q
!		dfun-external function, partial derivative df/dy
!		fun0-value of the function 
!		q-mass ratio
!	        ystart-first estimate
!		yacc-accuracy
!	output:  raphy-root for which fun=fun0
 IMPLICIT NONE

 EXTERNAL fun,dfun

 INTEGER:: i

 REAL(KIND=8):: raphy
 REAL(KIND=8):: fun,dfun
 REAL(KIND=8):: q,x0,ystart,yacc,dy,fun0

  raphy=ystart

  DO i=1,20
     dy=(fun(x0,raphy,0.D0,q)-fun0)/dfun(x0,raphy,0.D0,q)
     raphy=raphy-dy
	  IF (ABS(dy) < yacc) RETURN
  END DO
	
 END FUNCTION raphy

 FUNCTION raphz(fun,dfun,fun0,q,x0,y0,zstart,zacc)
!
!	Newton-Raphson Method in 1D (z coordinate)
!	to solve the equation fun(z)=fun0
!	input:  fun-external function of z and parameters: x,y,q
!		dfun-external function, partial derivative df/dz
!		fun0-value of the function 
!		q-mass ratio
!	        zstart-first estimate
!		zacc-accuracy
!	output:  raphz-root for which fun=fun0
 IMPLICIT NONE 
 
 EXTERNAL fun,dfun

 INTEGER:: i

 REAL(KIND=8):: raphz
 REAL(KIND=8):: fun,dfun
 REAL(KIND=8):: zstart,q,dz,zacc,fun0,x0,y0
	
 raphz=zstart
 
 DO i=1,20
    dz=(fun(x0,y0,raphz,q)-fun0)/dfun(x0,y0,raphz,q)
    raphz=raphz-dz
    IF (ABS(dz) < zacc) RETURN
 END DO

END FUNCTION raphz

FUNCTION pot(x,y,z,q)
!
!	input: x,y,z,q
!	output: normalized Roche potential 
 IMPLICIT NONE

 REAL(KIND=8):: pot
 REAL(KIND=8):: x,y,z,q
 REAL(KIND=8):: q1,x1,r1,r2
  q1=1.0D0+q
  x1=x-1.0D0
  r1=SQRT(x*x+y*y+z*z)
  r2=SQRT(x1*x1+y*y+z*z)
  pot=2.0D0/q1/r1+2.0D0*q/q1/r2+(x-q/q1)*(x-q/q1)+y*y
	
END FUNCTION pot

FUNCTION dxpot(x,y,z,q)
!
!	input: x,y,z,q
!	output: partial derivation of the Roche potential after x
!
 IMPLICIT NONE

 REAL(KIND=8):: dxpot
 REAL(KIND=8):: x,y,z,q
 REAL(KIND=8):: q1,x1,r1,r2
  q1=1.d0+q
  x1=x-1.d0
  r1=SQRT(x*x+y*y+z*z)
  r2=SQRT(x1*x1+y*y+z*z)
  dxpot=-x/q1/r1**3-q*x1/q1/r2**3+x-q/q1
  dxpot=2.d0*dxpot

END FUNCTION dxpot


FUNCTION dypot(x,y,z,q)
!
!	input: x,y,z,q
!	output: partial derivation of the Roche potential after y
!
 IMPLICIT NONE

 REAL(KIND=8):: dypot
 REAL(KIND=8):: x,y,z,q
 REAL(KIND=8):: x1,q1,r1,r2

  q1=1.d0+q
  x1=x-1.d0
  r1=SQRT(x*x+y*y+z*z)
  r2=SQRT(x1*x1+y*y+z*z)
  dypot=-2.d0/q1/r1**3-2.d0*q/q1/r2**3+2.d0
  dypot=dypot*y

END FUNCTION dypot

FUNCTION dzpot(x,y,z,q)
!
!	input: x,y,z,q
!	output: partial derivation of the Roche potential after z
!
 IMPLICIT NONE

 REAL(KIND=8):: dzpot
 REAL(KIND=8):: x,y,z,q
 REAL(KIND=8):: q1,x1,r1,r2
 q1=1.d0+q
 x1=x-1.d0
 r1=SQRT(x*x+y*y+z*z)
 r2=SQRT(x1*x1+y*y+z*z)
 dzpot=-2.d0/q1/r1**3-2.d0*q/q1/r2**3
 dzpot=dzpot*z

END FUNCTION dzpot

FUNCTION dxxpot(x,y,z,q)
!
!	input: x,y,z,q
!	output: second partial derivation of the Roche potential after x
!
 IMPLICIT NONE
 
 REAL(KIND=8):: dxxpot
 REAL(KIND=8):: q,x,y,z
 REAL(KIND=8):: q1,x1,r1,r2
  q1=1.d0+q
  x1=x-1.d0
  r1=SQRT(x*x+y*y+z*z)
  r2=SQRT(x1*x1+y*y+z*z)
  dxxpot=6.d0*x*x/q1/r1**5+6.d0*q*x1*x1/q1/r2**5-2.d0/q1/r1**3&
         -2.d0*q/q1/r2**3+2.d0
	
END FUNCTION dxxpot

FUNCTION radlob(q)
!
!	returns the volumee radius of the Roche lobe, 0<q<infty
!	q<1 -radius of the less massive star
!	q>1 -radius of the more massive star
! 
 IMPLICIT NONE
 
 REAL(KIND=8):: radlob
 REAL(KIND=8):: q,q13

  q13=q**(1.d0/3.d0)
  radlob=0.49d0*q13*q13/(0.69d0*q13*q13+LOG(1.d0+q13))
	
END FUNCTION radlob

SUBROUTINE gravn(x,y,z,q,potx,poty,potz,pot) 
!
!	returns the normalized magnitude of the gravity vector
!       input: x,y,z,q
 IMPLICIT NONE

 REAL(KIND=8):: x,y,z,q
 REAL(KIND=8):: potx,poty,potz,pot
 REAL(KIND=8):: dxpot,dypot,dzpot
 
 potx=dxpot(x,y,z,q)
 poty=dypot(x,y,z,q)
 potz=dzpot(x,y,z,q)
 pot=sqrt(potx*potx+poty*poty+potz*potz)
END SUBROUTINE gravn

!----------------------------------------------------------------------
 SUBROUTINE untsp(xstar1,star1,xstar2,star2,xstar3,star3,&
     wstar1,fstar1,tstar1,jstar1,wstar2,fstar2,tstar2,jstar2 ,&
     xunt1,yunt1,xunt2,yunt2,xunt3,yunt3,&
     nstar1,nstar2,nstar3,nspec1,nspec2,tspec1,tspec2,&
     lunt1,lunt2,lunt3,dlst,dlst2,dlcp,dlcp2)
!reading the input spectra of nontransparent objects
!input: lunt1,lunt2,lunt3,xunt1,yunt1,xunt2,yunt2,xunt3,yunt3,
!	dlst,dlst2,dlcp,dlcp2,tstar1,tstar2
!output:xstar1,star1,xstar2,star2,xstar3,star3,
!	nstar1,nstar2,nstar3,nspec1,nspec2,tspec1,tspec2,
!	wstar1,fstar1,jstar1,wstar2,fstar2,jstar2
 USE param
 USE constants
 IMPLICIT NONE

 INTEGER,DIMENSION(mspecx)::nstar1,nstar2
 INTEGER::nstar3,nspec1,nspec2,lunt2,lunt3,lunt1
 INTEGER::i,j,jstar1,jstar2,jj,idummy
 CHARACTER(LEN=50),DIMENSION(mspecx):: in1,in2
 REAL(KIND=8),DIMENSION(mstarx,mspecx)::xstar1,star1,xstar2,star2
 REAL(KIND=8),DIMENSION(mstarx)::wstar1,fstar1,wstar2,fstar2,xstar3,star3,fjj,wjj1,fjj1,fjj1n
 REAL(KIND=8),DIMENSION(mspecx):: tspec1,tspec2
 REAL(KIND=8)::xunt1,yunt1,xunt2,yunt2,xunt3,yunt3,dlst,dlst2,dlcp,dlcp2
 REAL(KIND=8)::tstar1,tstar2,der,dummy
!our units will be:	
! 	star1 -intrinsic nonrotated intensity spectrum of the central 
!	star in	[erg/cm^2/s/Hz/sterad]
!xstar1(i) -lambda in [A]
	IF(lunt1>0)THEN
	  OPEN(12,file='starspec1',STATUS='OLD')
	  READ(12,*)
	  READ(12,*)nspec1
	  READ(12,*)
	  DO i=1,nspec1
	    READ(12,*)tspec1(i)
	  ENDDO
	  READ(12,*)	  
	  DO i=1,nspec1
	    READ(12,'(a)')in1(i)
	  ENDDO	  
	  CLOSE(12)
  	  IF(nspec1>1)THEN
	    DO i=1,nspec1-1
	      IF(tspec1(i+1)<=tspec1(i))THEN 
	        WRITE(*,*)'error: tspec1, STOP'
	        STOP
	      ENDIF  
            ENDDO
	  ENDIF
	  DO j=1,nspec1
	    OPEN(12,file=in1(j),status='old')	  
	    i=1
	    DO
            IF(lunt1==3)THEN
              READ(12,*,end=20)
              READ(12,*,end=20)
            ENDIF
10	    IF(lunt1==1)THEN
	      READ(12,*,end=20)xstar1(i,j),star1(i,j)
	      IF(i>1)THEN
	        IF(.not.(xstar1(i,j)>xstar1(i-1,j)))&
              WRITE(*,*)' input error in starspec1 on line ',i,j
	        IF(i+1>mstarx) WRITE(*,*)' error in starspec1, ',&
              'dimension mstarx almost exceeded in starspec1'
	      ENDIF	    
!	convert input of Hlambda from synspec to our units
	      xstar1(i,j)=xstar1(i,j)*xunt1
	      star1(i,j)=star1(i,j)*yunt1/(1.d0-dlst/3.d0-dlst2/6.d0)
	      star1(i,j)=4.d-8*star1(i,j)*xstar1(i,j)**2/clight
	    ENDIF 
 	    IF(lunt1==2)THEN
 	      READ(12,*,end=20)xstar1(i,j),star1(i,j)
	      IF(i>1)THEN
	        IF(.not.(xstar1(i,j)>xstar1(i-1,j)))&
              WRITE(*,*)' input error in starspec1 on line ',i,j
	        IF(i+1>mstarx) WRITE(*,*)' error in starspec1, ',&
              'dimension mstarx almost exceeded in starspec1'
	      ENDIF 	    
	      xstar1(i,j)=xstar1(i,j)*xunt1
	      star1(i,j)=star1(i,j)*yunt1
	    ENDIF
 	    IF(lunt1==3)THEN
 	      READ(12,*,end=20)idummy,xstar1(i,j),dummy,star1(i,j)
	      IF(i>1)THEN
	        IF(.not.(xstar1(i,j)>xstar1(i-1,j)))&
              WRITE(*,*)' input error in starspec1 on line ',i,j
	        IF(i+1>mstarx) WRITE(*,*)' error in starspec1, ',&
              'dimension mstarx almost exceeded in starspec1'
	      ENDIF
!	convert input of nu, Fnu from coolTlusty.21 to our units
	      xstar1(i,j)=xunt1*clight/xstar1(i,j)*1.d8
	      star1(i,j)=star1(i,j)*yunt1
	      star1(i,j)=star1(i,j)/pi/(1.d0-dlst/3.d0-dlst2/6.d0)	      
	    ENDIF	    
	    i=i+1
	    ENDDO
20	    nstar1(j)=i-1
	    WRITE(6,*)' nstar1=',j,nstar1(j)
	    CLOSE(12)
	  ENDDO  
!	an interpolation to tstar1 to get wstar1,fstar1 
!	these will be used e.g. for scattering or reflection
	  IF(tstar1>=tspec1(nspec1))THEN
	    DO i=1,nstar1(nspec1)
	      wstar1(i)=xstar1(i,nspec1)
	      fstar1(i)=star1(i,nspec1)
	      jstar1=nstar1(nspec1)
	    ENDDO  
	  ELSEIF(tstar1<=tspec1(1))THEN
	    DO i=1,nstar1(1)
	      wstar1(i)=xstar1(i,1)
	      fstar1(i)=star1(i,1)
	      jstar1=nstar1(1)
            ENDDO
	  ELSE
	    CALL locate(tspec1,nspec1,tstar1,jj)
	    jstar1=nstar1(jj)
	    DO i=1,nstar1(jj)
	      wstar1(i)=xstar1(i,jj)
	      fjj(i)=star1(i,jj)
	    ENDDO
	    DO i=1,nstar1(jj+1)
	      wjj1(i)=xstar1(i,jj+1) 
              fjj1(i)=star1(i,jj+1) 
            ENDDO
	    CALL interp(wjj1,fjj1,wstar1,fjj1n,nstar1(jj+1),nstar1(jj))
	    DO i=1,nstar1(jj)
	      der=(fjj1n(i)-fjj(i))/(tspec1(jj+1)-tspec1(jj))
	      fstar1(i)=der*(tstar1-tspec1(jj))+fjj(i)
	    ENDDO
	  ENDIF
	ENDIF
! 	star2 -intrinsic nonrotated intensity spectrum of the secondary
!	star in	erg/cm^2/s/Hz/sterad
!xstar2(i) -lambda in [A]
	IF(lunt2>0)THEN
	  OPEN(13,file='starspec2',status='old')
	  READ(13,*)
	  READ(13,*)nspec2
	  READ(13,*)
	  DO i=1,nspec2
	    READ(13,*)tspec2(i)
	  ENDDO
	  READ(13,*)	  
	  DO i=1,nspec2
	    READ(13,'(a)')in2(i)
	  ENDDO	  
	  CLOSE(13)
  	  IF(nspec2>1)THEN
	    DO i=1,nspec2-1
	      IF(tspec2(i+1)<=tspec2(i))THEN 
	        WRITE(*,*)'error: tspec2, STOP'
	        STOP
	      ENDIF  
            ENDDO
	  ENDIF	
          DO j=1,nspec2
            OPEN(13,file=in2(j),status='old')
  	    i=1
	    DO
            IF(lunt2==3)THEN
              READ(13,*,end=60)
              READ(13,*,end=60)
            ENDIF
	    IF(lunt2==1)THEN
	      READ(13,*,end=60)xstar2(i,j),star2(i,j)	    
	      IF(i>1)THEN
	        IF(.not.(xstar2(i,j)>xstar2(i-1,j)))&
              WRITE(*,*)' input error in starspec2 on line ',i,j
	        IF(i+1>mstarx) WRITE(*,*)' error in starspec2, ',&
              'dimension mstarx almost exceeded in starspec2'
	      ENDIF	    
!	convert input of Hlambda from synspec to our units
  	      xstar2(i,j)=xstar2(i,j)*xunt2
	      star2(i,j)=star2(i,j)*yunt2/(1.d0-dlcp/3.d0-dlcp2/6.d0)
	      star2(i,j)=4.d-8*star2(i,j)*xstar2(i,j)**2/clight
	    ENDIF
	    IF(lunt2==2)THEN
	      READ(13,*,end=60)xstar2(i,j),star2(i,j)
	      IF(i>1)THEN
	        IF(.not.(xstar2(i,j)>xstar2(i-1,j)))&
              WRITE(*,*)' input error in starspec2 on line ',i,j
	        IF(i+1>mstarx) WRITE(*,*)' error in starspec2, ',&
              'dimension mstarx almost exceeded in starspec2'
	      ENDIF
	      xstar2(i,j)=xstar2(i,j)*xunt2
	      star2(i,j)=star2(i,j)*yunt2
	    ENDIF
	    IF(lunt2==3)THEN
	      READ(13,*,end=60)idummy,xstar2(i,j),dummy,star2(i,j)	    
	      IF(i>1)THEN
	        IF(.not.(xstar2(i,j)>xstar2(i-1,j)))&
              WRITE(*,*)' input error in starspec2 on line ',i,j
	        IF(i+1>mstarx) WRITE(*,*)' error in starspec2, ',&
              'dimension mstarx almost exceeded in starspec2'
	      ENDIF
!            convert input of nu, Fnu from coolTlusty.21 to our units	    
	      xstar2(i,j)=xunt2*clight/xstar2(i,j)*1.d8
	      star2(i,j)=star2(i,j)*yunt2
              star2(i,j)=star2(i,j)/pi/(1.d0-dlcp/3.d0-dlcp2/6.d0)
	    ENDIF	    
	    i=i+1
	    ENDDO
60	    nstar2(j)=i-1
	    WRITE(6,*)' nstar2=',j,nstar2(j)
	    CLOSE(13)
          ENDDO
!	an interpolation to tstar2
	  IF(tstar2>=tspec2(nspec2))THEN
	    DO i=1,nstar2(nspec2)
	      wstar2(i)=xstar2(i,nspec2)
	      fstar2(i)=star2(i,nspec2)
	      jstar2=nstar2(nspec2)
	    ENDDO  
	  ELSEIF(tstar2<=tspec2(1))THEN
	    DO i=1,nstar2(1)
	      wstar2(i)=xstar2(i,1)
	      fstar2(i)=star2(i,1)
	      jstar2=nstar2(1)
            ENDDO
	  ELSE
	    CALL locate(tspec2,nspec2,tstar2,jj)
	    jstar2=nstar2(jj)
	    DO i=1,nstar2(jj)
	      wstar2(i)=xstar2(i,jj)
	      fjj(i)=star2(i,jj)
	    ENDDO
	    DO i=1,nstar2(jj+1)
	      wjj1(i)=xstar2(i,jj+1) 
              fjj1(i)=star2(i,jj+1) 
            ENDDO
	    CALL interp(wjj1,fjj1,wstar2,fjj1n,nstar2(jj+1),nstar2(jj))
	    DO i=1,nstar2(jj)
	      der=(fjj1n(i)-fjj(i))/(tspec2(jj+1)-tspec2(jj))
	      fstar2(i)=der*(tstar2-tspec2(jj))+fjj(i)
	    ENDDO
	  ENDIF          
	ENDIF
! 	star3 -intrinsic nonrotated 3. intensity spectrum 
!	in erg/cm^2/s/Hz/sterad
!xstar3(i) -lambda in [A]
	IF(lunt3>0)THEN
	  OPEN(14,FILE='starspec3',STATUS='OLD')
	  i=1
	  DO
          IF(lunt3==3)THEN
            READ(14,*,end=100)
            READ(14,*,end=100)
          ENDIF
90	  IF(lunt3==1)THEN
	    READ(14,*,end=100)xstar3(i),star3(i)	  
	    IF(i>1)THEN
	      IF(.not.(xstar3(i)>xstar3(i-1)))&
            WRITE(*,*)' input error in starspec on line ',i
	      IF(i+1>mstarx) WRITE(*,*)' error in starspec, ',&
            'dimension mstarx almost exceeded in starspec'
	    ENDIF	  
!	convert input of Hlambda from synspec to our units
!	with no limb darkening
            xstar3(i)=xstar3(i)*xunt3
	    star3(i)=star3(i)*yunt3*4.d-8
	    star3(i)=star3(i)*xstar3(i)*xstar3(i)/clight
	  ENDIF
	  IF(lunt3==2)THEN
	    READ(14,*,end=100)xstar3(i),star3(i)
	    IF(i>1)THEN
	      IF(.not.(xstar3(i)>xstar3(i-1)))&
            WRITE(*,*)' input error in starspec on line ',i
	      IF(i+1>mstarx) WRITE(*,*)' error in starspec, ',&
            'dimension mstarx almost exceeded in starspec'
	    ENDIF
	    xstar3(i)=xstar3(i)*xunt3
	    star3(i)=star3(i)*yunt3
	  ENDIF
	  IF(lunt3==3)THEN
	    READ(14,*,end=100)idummy,xstar3(i),dummy,star3(i)	  
	    IF(i>1)THEN
	      IF(.not.(xstar3(i)>xstar3(i-1)))&
            WRITE(*,*)' input error in starspec on line ',i
	      IF(i+1>mstarx) WRITE(*,*)' error in starspec, ',&
            'dimension mstarx almost exceeded in starspec'
	    ENDIF
!            convert input of nu, Fnu from coolTlusty.21 to our units
	    xstar3(i)=xunt3*clight/xstar3(i)*1.d8
	    star3(i)=star3(i)*yunt3/pi
	  ENDIF
	  i=i+1
	  ENDDO
100	  nstar3=i-1
	  WRITE(6,*)' nstar3=',nstar3
	  CLOSE(14)
	ENDIF
 END SUBROUTINE untsp
!----------------------------------------------------------------------

!----------------------------------------------------------------------
 SUBROUTINE interp(x,y,xnew,ynew,ndat,nnew)
!interpolates in y(x) and returns ynew=y(xnew) 	
!assumes that x is monotonically increasing
!input: x,y
!output:xnew,ynew
 IMPLICIT NONE
 INTEGER::jj,i,ndat,nnew
 REAL(KIND=8),DIMENSION(ndat)::x,y
 REAL(KIND=8),DIMENSION(nnew)::xnew,ynew
 REAL(KIND=8)::der

        jj=1
	DO i=1,nnew
	  IF(xnew(i)>=x(ndat))THEN
	    ynew(i)=y(ndat)
	  ELSEIF(xnew(i)<=x(1))THEN
	    ynew(i)=y(1)
	  ELSE
	    CALL hunt(x,ndat,xnew(i),jj)
	    der=(y(jj+1)-y(jj))/(x(jj+1)-x(jj))
	    ynew(i)=der*(xnew(i)-x(jj))+y(jj)
	  ENDIF  
	ENDDO	                        
 END SUBROUTINE interp	                        
!----------------------------------------------------------------------

SUBROUTINE albedo(albx,alby,nalb,ialb)
!	subroutine reads the albedo of the star and companion from 
!	the table
!	albx(i) -wavelength in Angstrom
!	alby(i) -monochromatic albedo
 
 USE param
 IMPLICIT NONE

 INTEGER::nalb,ialb,i
 REAL(KIND=8),DIMENSION(mstarx)::albx,alby

        IF(ialb==1)THEN
       	   OPEN(12,file='albedo1',status='old')
      	    i=1
		DO
     		   READ(12,*,err=20,end=20)albx(i),alby(i)
       			   i=i+1
	        ENDDO
20	    nalb=i-1          
            CLOSE(12)
        ENDIF
        IF(ialb==2)THEN
          OPEN(13,file='albedo2',status='old')
           i=1
	  DO
               READ(13,*,err=40,end=40)albx(i),alby(i)
                i=i+1
	  ENDDO
40	  nalb=i-1          
          CLOSE(13)
        ENDIF        
END SUBROUTINE albedo  
